@@ -12,7 +12,6 @@ if [ "$(id -u)" -eq 0 ]; then
     dnf install -y python3
     dnf install -y xvfb-run
     ./contrib/ci/fwupd_setup_helpers.py --yes -o fedora -v mingw64 install-dependencies
-    dnf install -y https://kojipkgs.fedoraproject.org//packages/msitools/0.106/1.fc42/x86_64/msitools-0.106-1.fc42.x86_64.rpm
 fi
 
 #prep
@@ -24,11 +23,6 @@ build=$root/build-win32
 rm -rf $DESTDIR $build
 mkdir -p $build $DESTDIR && cd $build
 
-# Hack for Fedora bug
-if [ "$(id -u)" -eq 0 ]; then
-    sed -i '/^Requires.private: termcap/d'  /usr/x86_64-w64-mingw32/sys-root/mingw/lib/pkgconfig/readline.pc
-fi
-
 # run before using meson
 export WINEPREFIX=$build/.wine
 
@@ -61,10 +55,6 @@ xvfb-run meson setup .. \
     -Dlibjcat:gpg=false \
     -Dlibjcat:tests=false \
     -Dlibjcat:introspection=false \
-    -Dgusb:tests=false \
-    -Dgusb:docs=false \
-    -Dgusb:introspection=false \
-    -Dgusb:vapi=false $@
     $@
 VERSION=$(meson introspect . --projectinfo | jq -r .version)
 ninja --verbose -C "$build" -v install
@@ -96,7 +86,6 @@ find $MINGW32BINDIR \
 	-o -name libgmp-10.dll \
 	-o -name libgnutls-30.dll \
 	-o -name libgobject-2.0-0.dll \
-	-o -name libgusb-2.dll \
 	-o -name "libhogweed-*.dll" \
 	-o -name libidn2-0.dll \
 	-o -name libintl-8.dll \
@@ -105,21 +94,16 @@ find $MINGW32BINDIR \
 	-o -name "libnettle-*.dll" \
 	-o -name libp11-kit-0.dll \
 	-o -name libpcre2-8-0.dll \
-	-o -name libpsl-5.dll \
 	-o -name libsqlite3-0.dll \
 	-o -name libssh2-1.dll \
 	-o -name libssl-3-x64.dll \
 	-o -name libssp-0.dll \
-	-o -name libtermcap-0.dll \
-	-o -name libreadline8.dll \
 	-o -name libtasn1-6.dll \
-	-o -name libunistring-2.dll \
 	-o -name libusb-1.0.dll \
 	-o -name libwinpthread-1.dll \
 	-o -name libxml2-2.dll \
 	-o -name libxmlb-2.dll \
 	-o -name libzstd.dll \
-	-o -name wldap32.dll \
 	-o -name zlib1.dll \
 	| wixl-heat \
 	-p $MINGW32BINDIR/ \
@@ -176,6 +160,9 @@ wixl -v \
 	-D DESTDIR="$DESTDIR" \
 	-o "${MSI_FILENAME}"
 
+#generate news release
+contrib/ci/generate_news.py $VERSION > $DESTDIR/news.txt
+
 # check the msi archive can be installed and removed (use "wine uninstaller" to do manually)
 wine msiexec /i "${MSI_FILENAME}"
 ls -R ${WINEPREFIX}/drive_c/Program\ Files/fwupd/
