@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2019 Richard Hughes <richard@hughsie.com>
+ * Copyright 2019 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuFirmware"
@@ -16,6 +16,7 @@
 #include "fu-crc.h"
 #include "fu-dfu-firmware-private.h"
 #include "fu-dfu-firmware-struct.h"
+#include "fu-input-stream.h"
 
 /**
  * FuDfuFirmware:
@@ -197,26 +198,40 @@ fu_dfu_firmware_set_version(FuDfuFirmware *self, guint16 version)
 }
 
 static gboolean
-fu_dfu_firmware_check_magic(FuFirmware *firmware, GBytes *fw, gsize offset, GError **error)
+fu_dfu_firmware_validate(FuFirmware *firmware, GInputStream *stream, gsize offset, GError **error)
 {
-	return fu_struct_dfu_ftr_validate_bytes(fw,
-						g_bytes_get_size(fw) - FU_STRUCT_DFU_FTR_SIZE,
-						error);
+	gsize streamsz = 0;
+	if (!fu_input_stream_size(stream, &streamsz, error))
+		return FALSE;
+	if (streamsz < FU_STRUCT_DFU_FTR_SIZE) {
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_FILE,
+				    "stream was too small");
+		return FALSE;
+	}
+	return fu_struct_dfu_ftr_validate_stream(stream, streamsz - FU_STRUCT_DFU_FTR_SIZE, error);
 }
 
 gboolean
 fu_dfu_firmware_parse_footer(FuDfuFirmware *self,
-			     GBytes *fw,
+			     GInputStream *stream,
 			     FwupdInstallFlags flags,
 			     GError **error)
 {
 	FuDfuFirmwarePrivate *priv = GET_PRIVATE(self);
 	gsize bufsz;
-	const guint8 *buf = g_bytes_get_data(fw, &bufsz);
+	const guint8 *buf;
 	g_autoptr(GByteArray) st = NULL;
+	g_autoptr(GBytes) fw = NULL;
+
+	fw = fu_input_stream_read_bytes(stream, 0, G_MAXSIZE, NULL, error);
+	if (fw == NULL)
+		return FALSE;
+	buf = g_bytes_get_data(fw, &bufsz);
 
 	/* parse */
-	st = fu_struct_dfu_ftr_parse(buf, bufsz, bufsz - FU_STRUCT_DFU_FTR_SIZE, error);
+	st = fu_struct_dfu_ftr_parse_stream(stream, bufsz - FU_STRUCT_DFU_FTR_SIZE, error);
 	if (st == NULL)
 		return FALSE;
 	priv->vid = fu_struct_dfu_ftr_get_vid(st);
@@ -227,7 +242,7 @@ fu_dfu_firmware_parse_footer(FuDfuFirmware *self,
 
 	/* verify the checksum */
 	if ((flags & FWUPD_INSTALL_FLAG_IGNORE_CHECKSUM) == 0) {
-		guint32 crc_new = ~fu_crc32(buf, bufsz - 4);
+		guint32 crc_new = fu_crc32(FU_CRC_KIND_B32_JAMCRC, buf, bufsz - 4);
 		if (fu_struct_dfu_ftr_get_crc(st) != crc_new) {
 			g_set_error(error,
 				    FWUPD_ERROR,
@@ -256,22 +271,23 @@ fu_dfu_firmware_parse_footer(FuDfuFirmware *self,
 
 static gboolean
 fu_dfu_firmware_parse(FuFirmware *firmware,
-		      GBytes *fw,
-		      gsize offset,
+		      GInputStream *stream,
 		      FwupdInstallFlags flags,
 		      GError **error)
 {
 	FuDfuFirmware *self = FU_DFU_FIRMWARE(firmware);
 	FuDfuFirmwarePrivate *priv = GET_PRIVATE(self);
-	gsize len = g_bytes_get_size(fw);
+	gsize streamsz = 0;
 	g_autoptr(GBytes) contents = NULL;
 
 	/* parse footer */
-	if (!fu_dfu_firmware_parse_footer(self, fw, flags, error))
+	if (!fu_dfu_firmware_parse_footer(self, stream, flags, error))
 		return FALSE;
 
 	/* trim footer off */
-	contents = fu_bytes_new_offset(fw, 0, len - priv->footer_len, error);
+	if (!fu_input_stream_size(stream, &streamsz, error))
+		return FALSE;
+	contents = fu_input_stream_read_bytes(stream, 0, streamsz - priv->footer_len, NULL, error);
 	if (contents == NULL)
 		return FALSE;
 	fu_firmware_set_bytes(firmware, contents);
@@ -292,7 +308,9 @@ fu_dfu_firmware_append_footer(FuDfuFirmware *self, GBytes *contents, GError **er
 	fu_struct_dfu_ftr_set_vid(st, priv->vid);
 	fu_struct_dfu_ftr_set_ver(st, priv->dfu_version);
 	g_byte_array_append(buf, st->data, st->len - sizeof(guint32));
-	fu_byte_array_append_uint32(buf, ~fu_crc32(buf->data, buf->len), G_LITTLE_ENDIAN);
+	fu_byte_array_append_uint32(buf,
+				    fu_crc32(FU_CRC_KIND_B32_JAMCRC, buf->data, buf->len),
+				    G_LITTLE_ENDIAN);
 	return g_steal_pointer(&buf);
 }
 
@@ -359,12 +377,12 @@ fu_dfu_firmware_init(FuDfuFirmware *self)
 static void
 fu_dfu_firmware_class_init(FuDfuFirmwareClass *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
-	klass_firmware->check_magic = fu_dfu_firmware_check_magic;
-	klass_firmware->export = fu_dfu_firmware_export;
-	klass_firmware->parse = fu_dfu_firmware_parse;
-	klass_firmware->write = fu_dfu_firmware_write;
-	klass_firmware->build = fu_dfu_firmware_build;
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->validate = fu_dfu_firmware_validate;
+	firmware_class->export = fu_dfu_firmware_export;
+	firmware_class->parse = fu_dfu_firmware_parse;
+	firmware_class->write = fu_dfu_firmware_write;
+	firmware_class->build = fu_dfu_firmware_build;
 }
 
 /**
