@@ -1,7 +1,8 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
-#[derive(New, ValidateBytes, Parse)]
+#[derive(New, ValidateStream, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructDfuFtr {
     release: u16le,
     pid: u16le,
@@ -11,14 +12,18 @@ struct FuStructDfuFtr {
     len: u8 = $struct_size,
     crc: u32le,
 }
-#[derive(New, ValidateBytes, ParseBytes)]
+
+#[derive(New, ValidateStream, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructDfuseHdr {
     sig: [char; 5] == "DfuSe",
     ver: u8 == 0x01,
     image_size: u32le,
     targets: u8,
 }
-#[derive(New, Validate, ParseBytes)]
+
+#[derive(New, Validate, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructDfuseImage {
     sig: [char; 6] == "Target",
     alt_setting: u8,
@@ -27,7 +32,9 @@ struct FuStructDfuseImage {
     target_size: u32le,
     chunks: u32le,
 }
-#[derive(New, Validate, Parse)]
+
+#[derive(New, ParseStream)]
+#[repr(C, packed)]
 struct FuStructDfuseElement {
     address: u32le,
     size: u32le,
