@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
+ * Copyright 2023 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuDpauxDevice"
@@ -43,13 +43,9 @@ fu_dpaux_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuDpauxDevice *self = FU_DPAUX_DEVICE(device);
 	FuDpauxDevicePrivate *priv = GET_PRIVATE(self);
-	FU_DEVICE_CLASS(fu_dpaux_device_parent_class)->to_string(device, idt, str);
-	if (priv->dpcd_ieee_oui != 0)
-		fu_string_append_kx(str, idt, "DpcdIeeeOui", priv->dpcd_ieee_oui);
-	if (priv->dpcd_hw_rev != 0)
-		fu_string_append_kx(str, idt, "DpcdHwRev", priv->dpcd_hw_rev);
-	if (priv->dpcd_dev_id != NULL)
-		fu_string_append(str, idt, "DpcdDevId", priv->dpcd_dev_id);
+	fwupd_codec_string_append_hex(str, idt, "DpcdIeeeOui", priv->dpcd_ieee_oui);
+	fwupd_codec_string_append_hex(str, idt, "DpcdHwRev", priv->dpcd_hw_rev);
+	fwupd_codec_string_append(str, idt, "DpcdDevId", priv->dpcd_dev_id);
 }
 
 static void
@@ -65,7 +61,7 @@ fu_dpaux_device_invalidate(FuDevice *device)
 static gboolean
 fu_dpaux_device_probe(FuDevice *device, GError **error)
 {
-	const gchar *tmp;
+	g_autofree gchar *attr_name = NULL;
 
 	/* FuUdevDevice->probe */
 	if (!FU_DEVICE_CLASS(fu_dpaux_device_parent_class)->probe(device, error))
@@ -79,14 +75,20 @@ fu_dpaux_device_probe(FuDevice *device, GError **error)
 		    g_path_get_basename(fu_udev_device_get_sysfs_path(FU_UDEV_DEVICE(device)));
 		fu_device_set_logical_id(device, logical_id);
 	}
-
-	if (!fu_udev_device_set_physical_id(FU_UDEV_DEVICE(device), "pci,drm_dp_aux_dev", error))
-		return FALSE;
+	if (fu_device_get_physical_id(device) == NULL) {
+		if (!fu_udev_device_set_physical_id(FU_UDEV_DEVICE(device),
+						    "pci,drm_dp_aux_dev",
+						    error))
+			return FALSE;
+	}
 
 	/* only populated on real system, test suite won't have udev_device set */
-	tmp = fu_udev_device_get_sysfs_attr(FU_UDEV_DEVICE(device), "name", NULL);
-	if (tmp != NULL)
-		fu_device_set_name(device, tmp);
+	attr_name = fu_udev_device_read_sysfs(FU_UDEV_DEVICE(device),
+					      "name",
+					      FU_UDEV_DEVICE_ATTR_READ_TIMEOUT_DEFAULT,
+					      NULL);
+	if (attr_name != NULL)
+		fu_device_set_name(device, attr_name);
 
 	return TRUE;
 }
@@ -104,8 +106,8 @@ fu_dpaux_device_setup(FuDevice *device, GError **error)
 	    g_str_has_prefix(fu_device_get_name(device), "AMDGPU DM") &&
 	    fu_context_has_hwid_guid(ctx, "32d49d99-414b-55d5-813b-12aaf0335b58")) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "reading %s DPCD is broken on this hardware, "
 			    "you need to update the system BIOS",
 			    fu_device_get_name(device));
@@ -129,6 +131,44 @@ fu_dpaux_device_setup(FuDevice *device, GError **error)
 	priv->dpcd_hw_rev = fu_struct_dpaux_dpcd_get_hw_rev(st);
 	priv->dpcd_dev_id = fu_struct_dpaux_dpcd_get_dev_id(st);
 	fu_device_set_version_raw(device, fu_struct_dpaux_dpcd_get_fw_ver(st));
+
+	/* build some extra GUIDs */
+	if (priv->dpcd_ieee_oui != 0x0)
+		fu_device_add_instance_u32(device, "OUI", priv->dpcd_ieee_oui);
+	if (priv->dpcd_hw_rev != 0x0)
+		fu_device_add_instance_u8(device, "HWREV", priv->dpcd_hw_rev);
+	if (priv->dpcd_dev_id != 0x0)
+		fu_device_add_instance_strup(device, "DEVID", priv->dpcd_dev_id);
+	fu_device_build_instance_id_full(device,
+					 FU_DEVICE_INSTANCE_FLAG_QUIRKS,
+					 NULL,
+					 "DPAUX",
+					 "OUI",
+					 NULL);
+	fu_device_build_instance_id_full(device,
+					 FU_DEVICE_INSTANCE_FLAG_QUIRKS,
+					 NULL,
+					 "DPAUX",
+					 "OUI",
+					 "HWREV",
+					 NULL);
+	fu_device_build_instance_id_full(device,
+					 FU_DEVICE_INSTANCE_FLAG_QUIRKS,
+					 NULL,
+					 "DPAUX",
+					 "OUI",
+					 "DEVID",
+					 NULL);
+	fu_device_build_instance_id_full(device,
+					 FU_DEVICE_INSTANCE_FLAG_QUIRKS,
+					 NULL,
+					 "DPAUX",
+					 "OUI",
+					 "HWREV",
+					 "DEVID",
+					 NULL);
+
+	/* success */
 	return TRUE;
 }
 
@@ -264,38 +304,22 @@ fu_dpaux_device_write(FuDpauxDevice *self,
 		      guint timeout_ms,
 		      GError **error)
 {
-	FuIOChannel *io_channel = fu_udev_device_get_io_channel(FU_UDEV_DEVICE(self));
 	g_autofree gchar *title = g_strdup_printf("DPAUX write @0x%x", (guint)offset);
 
 	g_return_val_if_fail(FU_IS_DPAUX_DEVICE(self), FALSE);
 	g_return_val_if_fail(buf != NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	/* sanity check */
-	if (io_channel == NULL) {
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_NOT_INITIALIZED,
-				    "device is not open");
-		return FALSE;
-	}
-
 	/* seek, then write */
 	fu_dump_raw(G_LOG_DOMAIN, title, buf, bufsz);
-	if (lseek(fu_io_channel_unix_get_fd(io_channel), offset, SEEK_SET) != offset) {
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
-			    "failed to lseek to 0x%x",
-			    (guint)offset);
+	if (!fu_udev_device_seek(FU_UDEV_DEVICE(self), offset, error))
 		return FALSE;
-	}
-	return fu_io_channel_write_raw(io_channel,
-				       buf,
-				       bufsz,
-				       timeout_ms,
-				       FU_IO_CHANNEL_FLAG_NONE,
-				       error);
+	return fu_udev_device_write(FU_UDEV_DEVICE(self),
+				    buf,
+				    bufsz,
+				    timeout_ms,
+				    FU_IO_CHANNEL_FLAG_NONE,
+				    error);
 }
 
 /**
@@ -319,37 +343,22 @@ fu_dpaux_device_read(FuDpauxDevice *self,
 		     guint timeout_ms,
 		     GError **error)
 {
-	FuIOChannel *io_channel = fu_udev_device_get_io_channel(FU_UDEV_DEVICE(self));
 	g_autofree gchar *title = g_strdup_printf("DPAUX read @0x%x", (guint)offset);
 
 	g_return_val_if_fail(FU_IS_DPAUX_DEVICE(self), FALSE);
 	g_return_val_if_fail(buf != NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	/* sanity check */
-	if (io_channel == NULL) {
-		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_NOT_INITIALIZED,
-				    "device is not open");
-		return FALSE;
-	}
 	/* seek, then read */
-	if (lseek(fu_io_channel_unix_get_fd(io_channel), offset, SEEK_SET) != offset) {
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
-			    "failed to lseek to 0x%x",
-			    (guint)offset);
+	if (!fu_udev_device_seek(FU_UDEV_DEVICE(self), offset, error))
 		return FALSE;
-	}
-	if (!fu_io_channel_read_raw(io_channel,
-				    buf,
-				    bufsz,
-				    NULL,
-				    timeout_ms,
-				    FU_IO_CHANNEL_FLAG_NONE,
-				    error))
+	if (!fu_udev_device_read(FU_UDEV_DEVICE(self),
+				 buf,
+				 bufsz,
+				 NULL,
+				 timeout_ms,
+				 FU_IO_CHANNEL_FLAG_NONE,
+				 error))
 		return FALSE;
 	fu_dump_raw(G_LOG_DOMAIN, title, buf, bufsz);
 
@@ -367,9 +376,6 @@ fu_dpaux_device_incorporate(FuDevice *device, FuDevice *donor)
 	g_return_if_fail(FU_IS_DPAUX_DEVICE(self));
 	g_return_if_fail(FU_IS_DPAUX_DEVICE(donor));
 
-	/* FuUdevDevice->incorporate */
-	FU_DEVICE_CLASS(fu_dpaux_device_parent_class)->incorporate(device, donor);
-
 	/* copy private instance data */
 	priv->dpcd_ieee_oui = priv_donor->dpcd_ieee_oui;
 	priv->dpcd_hw_rev = priv_donor->dpcd_hw_rev;
@@ -416,10 +422,11 @@ static void
 fu_dpaux_device_init(FuDpauxDevice *self)
 {
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_TRIPLET);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_NO_GENERIC_GUIDS);
-	fu_udev_device_set_flags(FU_UDEV_DEVICE(self),
-				 FU_UDEV_DEVICE_FLAG_OPEN_READ | FU_UDEV_DEVICE_FLAG_OPEN_WRITE |
-				     FU_UDEV_DEVICE_FLAG_OPEN_NONBLOCK);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_NO_GENERIC_GUIDS);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_EMULATED_REQUIRE_SETUP);
+	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_READ);
+	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_WRITE);
+	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_NONBLOCK);
 }
 
 static void
@@ -434,19 +441,19 @@ fu_dpaux_device_finalize(GObject *object)
 static void
 fu_dpaux_device_class_init(FuDpauxDeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	GParamSpec *pspec;
 
 	object_class->finalize = fu_dpaux_device_finalize;
 	object_class->get_property = fu_dpaux_device_get_property;
 	object_class->set_property = fu_dpaux_device_set_property;
-	klass_device->probe = fu_dpaux_device_probe;
-	klass_device->setup = fu_dpaux_device_setup;
-	klass_device->invalidate = fu_dpaux_device_invalidate;
-	klass_device->to_string = fu_dpaux_device_to_string;
-	klass_device->incorporate = fu_dpaux_device_incorporate;
-	klass_device->convert_version = fu_dpaux_device_convert_version;
+	device_class->probe = fu_dpaux_device_probe;
+	device_class->setup = fu_dpaux_device_setup;
+	device_class->invalidate = fu_dpaux_device_invalidate;
+	device_class->to_string = fu_dpaux_device_to_string;
+	device_class->incorporate = fu_dpaux_device_incorporate;
+	device_class->convert_version = fu_dpaux_device_convert_version;
 
 	/**
 	 * FuDpauxDevice:dpcd-ieee-oui:
