@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuEfiSignatureList"
@@ -16,6 +16,7 @@
 #include "fu-efi-signature-list.h"
 #include "fu-efi-signature-private.h"
 #include "fu-efi-struct.h"
+#include "fu-input-stream.h"
 #include "fu-mem.h"
 
 /**
@@ -34,70 +35,9 @@ G_DEFINE_TYPE(FuEfiSignatureList, fu_efi_signature_list, FU_TYPE_FIRMWARE)
 
 const guint8 FU_EFI_SIGLIST_HEADER_MAGIC[] = {0x26, 0x16, 0xC4, 0xC1, 0x4C};
 
-static gboolean
-fu_efi_signature_list_parse_item(FuEfiSignatureList *self,
-				 FuEfiSignatureKind sig_kind,
-				 const guint8 *buf,
-				 gsize bufsz,
-				 gsize offset,
-				 guint32 size,
-				 GError **error)
-{
-	fwupd_guid_t guid;
-	gsize sig_datasz;
-	g_autofree gchar *sig_owner = NULL;
-	g_autofree guint8 *sig_data = NULL;
-	g_autoptr(FuEfiSignature) sig = NULL;
-	g_autoptr(GBytes) data = NULL;
-
-	/* allocate data buf */
-	if (size <= sizeof(fwupd_guid_t)) {
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
-			    "SignatureSize invalid: 0x%x",
-			    (guint)size);
-		return FALSE;
-	}
-	sig_datasz = size - sizeof(fwupd_guid_t);
-	sig_data = g_malloc0(sig_datasz);
-
-	/* read both blocks of data */
-	if (!fu_memcpy_safe((guint8 *)&guid,
-			    sizeof(guid),
-			    0x0, /* dst */
-			    buf,
-			    bufsz,
-			    offset, /* src */
-			    sizeof(guid),
-			    error)) {
-		g_prefix_error(error, "failed to read signature GUID: ");
-		return FALSE;
-	}
-	if (!fu_memcpy_safe(sig_data,
-			    sig_datasz,
-			    0x0, /* dst */
-			    buf,
-			    bufsz,
-			    offset + sizeof(fwupd_guid_t), /* src */
-			    sig_datasz,
-			    error)) {
-		g_prefix_error(error, "failed to read signature data: ");
-		return FALSE;
-	}
-
-	/* create item */
-	sig_owner = fwupd_guid_to_string(&guid, FWUPD_GUID_FLAG_MIXED_ENDIAN);
-	data = g_bytes_new(sig_data, sig_datasz);
-	sig = fu_efi_signature_new(sig_kind, sig_owner);
-	fu_firmware_set_bytes(FU_FIRMWARE(sig), data);
-	return fu_firmware_add_image_full(FU_FIRMWARE(self), FU_FIRMWARE(sig), error);
-}
-
 static gboolean
 fu_efi_signature_list_parse_list(FuEfiSignatureList *self,
-				 const guint8 *buf,
-				 gsize bufsz,
+				 GInputStream *stream,
 				 gsize *offset,
 				 GError **error)
 {
@@ -110,7 +50,7 @@ fu_efi_signature_list_parse_list(FuEfiSignatureList *self,
 	g_autoptr(GByteArray) st = NULL;
 
 	/* read EFI_SIGNATURE_LIST */
-	st = fu_struct_efi_signature_list_parse(buf, bufsz, *offset, error);
+	st = fu_struct_efi_signature_list_parse_stream(stream, *offset, error);
 	if (st == NULL)
 		return FALSE;
 	sig_type = fwupd_guid_to_string(fu_struct_efi_signature_list_get_type(st),
@@ -123,8 +63,8 @@ fu_efi_signature_list_parse_list(FuEfiSignatureList *self,
 	list_size = fu_struct_efi_signature_list_get_list_size(st);
 	if (list_size < 0x1c || list_size > 1024 * 1024) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "SignatureListSize invalid: 0x%x",
 			    list_size);
 		return FALSE;
@@ -132,8 +72,8 @@ fu_efi_signature_list_parse_list(FuEfiSignatureList *self,
 	header_size = fu_struct_efi_signature_list_get_header_size(st);
 	if (header_size > 1024 * 1024) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "SignatureHeaderSize invalid: 0x%x",
 			    header_size);
 		return FALSE;
@@ -141,8 +81,8 @@ fu_efi_signature_list_parse_list(FuEfiSignatureList *self,
 	size = fu_struct_efi_signature_list_get_size(st);
 	if (size < sizeof(fwupd_guid_t) || size > 1024 * 1024) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "SignatureSize invalid: 0x%x",
 			    size);
 		return FALSE;
@@ -151,13 +91,15 @@ fu_efi_signature_list_parse_list(FuEfiSignatureList *self,
 	/* header is typically unused */
 	offset_tmp = *offset + 0x1c + header_size;
 	for (guint i = 0; i < (list_size - 0x1c) / size; i++) {
-		if (!fu_efi_signature_list_parse_item(self,
-						      sig_kind,
-						      buf,
-						      bufsz,
-						      offset_tmp,
-						      size,
-						      error))
+		g_autoptr(FuEfiSignature) sig = fu_efi_signature_new(sig_kind);
+		fu_firmware_set_size(FU_FIRMWARE(sig), size);
+		if (!fu_firmware_parse_stream(FU_FIRMWARE(sig),
+					      stream,
+					      offset_tmp,
+					      FWUPD_INSTALL_FLAG_NONE,
+					      error))
+			return FALSE;
+		if (!fu_firmware_add_image_full(FU_FIRMWARE(self), FU_FIRMWARE(sig), error))
 			return FALSE;
 		offset_tmp += size;
 	}
@@ -166,20 +108,21 @@ fu_efi_signature_list_parse_list(FuEfiSignatureList *self,
 }
 
 static gboolean
-fu_efi_signature_list_check_magic(FuFirmware *firmware, GBytes *fw, gsize offset, GError **error)
+fu_efi_signature_list_validate(FuFirmware *firmware,
+			       GInputStream *stream,
+			       gsize offset,
+			       GError **error)
 {
 	fwupd_guid_t guid = {0x0};
 	g_autofree gchar *sig_type = NULL;
 
-	/* read EFI_SIGNATURE_LIST */
-	if (!fu_memcpy_safe((guint8 *)&guid,
-			    sizeof(guid),
-			    0x0, /* dst */
-			    g_bytes_get_data(fw, NULL),
-			    g_bytes_get_size(fw),
-			    offset, /* src */
-			    sizeof(guid),
-			    error)) {
+	if (!fu_input_stream_read_safe(stream,
+				       (guint8 *)&guid,
+				       sizeof(guid),
+				       0,
+				       offset, /* seek */
+				       sizeof(guid),
+				       error)) {
 		g_prefix_error(error, "failed to read magic: ");
 		return FALSE;
 	}
@@ -199,18 +142,19 @@ fu_efi_signature_list_check_magic(FuFirmware *firmware, GBytes *fw, gsize offset
 
 static gboolean
 fu_efi_signature_list_parse(FuFirmware *firmware,
-			    GBytes *fw,
-			    gsize offset,
+			    GInputStream *stream,
 			    FwupdInstallFlags flags,
 			    GError **error)
 {
 	FuEfiSignatureList *self = FU_EFI_SIGNATURE_LIST(firmware);
-	gsize bufsz = 0;
-	const guint8 *buf = g_bytes_get_data(fw, &bufsz);
+	gsize offset = 0;
+	gsize streamsz = 0;
 
 	/* parse each EFI_SIGNATURE_LIST */
-	while (offset < bufsz) {
-		if (!fu_efi_signature_list_parse_list(self, buf, bufsz, &offset, error))
+	if (!fu_input_stream_size(stream, &streamsz, error))
+		return FALSE;
+	while (offset < streamsz) {
+		if (!fu_efi_signature_list_parse_list(self, stream, &offset, error))
 			return FALSE;
 	}
 
@@ -221,20 +165,43 @@ fu_efi_signature_list_parse(FuFirmware *firmware,
 static GByteArray *
 fu_efi_signature_list_write(FuFirmware *firmware, GError **error)
 {
-	g_autoptr(GByteArray) buf = fu_struct_efi_signature_list_new();
+	fwupd_guid_t guid = {0};
+	g_autoptr(FuStructEfiSignatureList) st = fu_struct_efi_signature_list_new();
+	g_autoptr(GPtrArray) images = fu_firmware_get_images(firmware);
 
 	/* entry */
-	fu_struct_efi_signature_list_set_list_size(buf, buf->len + 16 + 32);
-	fu_struct_efi_signature_list_set_header_size(buf, 0);
-	fu_struct_efi_signature_list_set_size(buf, 16 + 32);
+	if (!fwupd_guid_from_string("c1c41626-504c-4092-aca9-41f936934328",
+				    &guid,
+				    FWUPD_GUID_FLAG_MIXED_ENDIAN,
+				    error))
+		return NULL;
+	fu_struct_efi_signature_list_set_type(st, &guid);
+	fu_struct_efi_signature_list_set_header_size(st, 0);
+	fu_struct_efi_signature_list_set_list_size(st,
+						   FU_STRUCT_EFI_SIGNATURE_LIST_SIZE +
+						       (images->len * (16 + 32)));
+	fu_struct_efi_signature_list_set_size(st, sizeof(fwupd_guid_t) + 32); /* SHA256 */
 
 	/* SignatureOwner + SignatureData */
-	for (guint i = 0; i < 16; i++)
-		fu_byte_array_append_uint8(buf, '1');
-	for (guint i = 0; i < 16; i++)
-		fu_byte_array_append_uint8(buf, '2');
+	for (guint i = 0; i < images->len; i++) {
+		FuFirmware *img = g_ptr_array_index(images, i);
+		g_autoptr(GBytes) img_blob = NULL;
+		img_blob = fu_firmware_write(img, error);
+		if (img_blob == NULL)
+			return NULL;
+		if (g_bytes_get_size(img_blob) != sizeof(fwupd_guid_t) + 32) {
+			g_set_error(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
+				    "expected SHA256 hash as signature data, got 0x%x",
+				    (guint)(g_bytes_get_size(img_blob) - sizeof(fwupd_guid_t)));
+			return NULL;
+		}
+		fu_byte_array_append_bytes(st, img_blob);
+	}
 
-	return g_steal_pointer(&buf);
+	/* success */
+	return g_steal_pointer(&st);
 }
 
 /**
@@ -253,10 +220,10 @@ fu_efi_signature_list_new(void)
 static void
 fu_efi_signature_list_class_init(FuEfiSignatureListClass *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
-	klass_firmware->check_magic = fu_efi_signature_list_check_magic;
-	klass_firmware->parse = fu_efi_signature_list_parse;
-	klass_firmware->write = fu_efi_signature_list_write;
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->validate = fu_efi_signature_list_validate;
+	firmware_class->parse = fu_efi_signature_list_parse;
+	firmware_class->write = fu_efi_signature_list_write;
 }
 
 static void
@@ -264,4 +231,5 @@ fu_efi_signature_list_init(FuEfiSignatureList *self)
 {
 	fu_firmware_add_flag(FU_FIRMWARE(self), FU_FIRMWARE_FLAG_ALWAYS_SEARCH);
 	fu_firmware_set_images_max(FU_FIRMWARE(self), 2000);
+	g_type_ensure(FU_TYPE_EFI_SIGNATURE);
 }
