@@ -1,7 +1,7 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
-#[derive(ToString)]
+#[derive(ToString, FromString)]
 enum FuEfiSignatureKind {
     Unknown,
     Sha256,
@@ -40,7 +40,8 @@ enum FuEfiFileType {
     FfsPad = 0xF0,
 }
 
-#[derive(New, Validate, ParseBytes)]
+#[derive(New, Validate, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructEfiFile {
     name: Guid,
     hdr_checksum: u8,
@@ -48,15 +49,51 @@ struct FuStructEfiFile {
     type: FuEfiFileType,
     attrs: u8,
     size: u24le,
-    state: u8 == 0xF8,
+    state: u8 = 0xF8,
+}
+
+#[derive(ParseStream)]
+#[repr(C, packed)]
+struct FuStructEfiFile2 {
+    _base: FuStructEfiFile,
+    extended_size: u64le,
+}
+
+#[repr(u8)]
+enum FuEfiCompressionType {
+    NotCompressed = 0x00,
+    StandardCompression = 0x01,
+}
+
+#[derive(ParseStream)]
+#[repr(C, packed)]
+struct FuStructEfiSectionCompression {
+    uncompressed_length: u32le,
+    compression_type: FuEfiCompressionType,
+}
+
+#[derive(ToString)]
+enum FuEfiLz77DecompressorVersion {
+    None,
+    Legacy,
+    Tiano,
+}
+
+#[derive(ParseStream)]
+#[repr(C, packed)]
+struct FuStructEfiLz77DecompressorHeader {
+    src_size: u32le,
+    dst_size: u32le,
 }
 
 #[repr(u8)]
 #[derive(ToString)]
 enum FuEfiSectionType {
+    // encapsulation section type values
     Compression = 0x01,
     GuidDefined = 0x02,
     Disposable = 0x03,
+    // leaf section type values
     Pe32 = 0x10,
     Pic = 0x11,
     Te = 0x12,
@@ -69,20 +106,41 @@ enum FuEfiSectionType {
     Raw = 0x19,
     PeiDepex = 0x1B,
     MmDepex = 0x1C,
+    // vendor-specific
+    PhoenixSectionPostcode = 0xF0,  // Phoenix SCT
+    InsydeSectionPostcode = 0x20,   // Insyde H2O
 }
 
-#[derive(New, Validate, ParseBytes)]
+#[derive(New, ParseStream)]
+#[repr(C, packed)]
 struct FuStructEfiSection {
     size: u24le,
     type: FuEfiSectionType,
 }
-#[derive(New, Validate, ParseBytes)]
+
+#[derive(ParseStream)]
+#[repr(C, packed)]
+struct FuStructEfiSection2 {
+    _base: FuStructEfiSection,
+    extended_size: u32le,
+}
+
+#[derive(ParseStream)]
+#[repr(C, packed)]
+struct FuStructEfiSectionFreeformSubtypeGuid {
+    guid: Guid,
+}
+
+#[derive(New, ParseStream)]
+#[repr(C, packed)]
 struct FuStructEfiSectionGuidDefined {
     name: Guid,
     offset: u16le,
     attr: u16le,
 }
-#[derive(New, ValidateBytes, ParseBytes)]
+
+#[derive(New, ValidateStream, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructEfiVolume {
     zero_vector: Guid,
     guid: Guid,
@@ -95,12 +153,38 @@ struct FuStructEfiVolume {
     reserved: u8,
     revision: u8 == 0x02,
 }
-#[derive(New, Validate, ParseBytes)]
+
+#[derive(ParseStream)]
+#[repr(C, packed)]
+struct FuStructEfiVolumeExtHeader {
+    fv_name: Guid,
+    size: u32le,
+}
+
+#[repr(u16le)]
+#[derive(ToString)]
+enum FuEfiVolumeExtEntryType {
+    Oem = 0x01,
+    Guid = 0x02,
+    Size = 0x03,
+}
+
+#[derive(ParseStream)]
+#[repr(C, packed)]
+struct FuStructEfiVolumeExtEntry {
+    size: u16le,
+    type: FuEfiVolumeExtEntryType,
+}
+
+#[derive(New, ParseStream)]
+#[repr(C, packed)]
 struct FuStructEfiVolumeBlockMap {
     num_blocks: u32le,
     length: u32le,
 }
-#[derive(New, Validate, Parse)]
+
+#[derive(New, ParseStream)]
+#[repr(C, packed)]
 struct FuStructEfiSignatureList {
     type: Guid,
     list_size: u32le,
@@ -118,7 +202,16 @@ enum FuEfiLoadOptionAttrs {
     CategoryAp = 0x100,
 }
 
-#[derive(ParseBytes, New)]
+#[derive(ToString, FromString)]
+enum FuEfiLoadOptionKind {
+    Unknown,
+    Path,
+    Hive,
+    Data,
+}
+
+#[derive(ParseStream, New)]
+#[repr(C, packed)]
 struct FuStructEfiLoadOption {
     attrs: FuEfiLoadOptionAttrs,
     dp_size: u16le,
@@ -134,7 +227,8 @@ enum FuEfiDevicePathType {
     End = 0x7F,
 }
 
-#[derive(ParseBytes, New)]
+#[derive(ParseStream, New, Default)]
+#[repr(C, packed)]
 struct FuStructEfiDevicePath {
     type: FuEfiDevicePathType,
     subtype: u8 = 0xFF,
@@ -169,7 +263,8 @@ enum FuEfiHardDriveDevicePathSignatureType {
     Guid,
 }
 
-#[derive(ParseBytes, New)]
+#[derive(ParseStream, New, Default)]
+#[repr(C, packed)]
 struct FuStructEfiHardDriveDevicePath {
     type: FuEfiDevicePathType == Media,
     subtype: FuEfiHardDriveDevicePathSubtype = HardDrive,
@@ -181,3 +276,23 @@ struct FuStructEfiHardDriveDevicePath {
     partition_format: FuEfiHardDriveDevicePathPartitionFormat = GuidPartitionTable,
     signature_type: FuEfiHardDriveDevicePathSignatureType = Guid,
 }
+
+#[derive(ParseStream, New, Default)]
+#[repr(C, packed)]
+struct FuStructShimHive {
+    magic: [char; 4] == "HIVE",
+    header_version: u8 = 0x1,
+    items_count: u8,
+    items_offset: u8,    // for forwards and backwards compatibility
+    crc32: u32le,        // of the entire hive (excluding padding)
+    //items: [ShimHiveItems; items_count]
+}
+
+#[derive(ParseStream, New)]
+#[repr(C, packed)]
+struct FuStructShimHiveItem {
+    key_length: u8,
+    value_length: u32le,
+    // key string, no trailing NUL
+    // value string, no trailing NUL
+}
