@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2022 Richard Hughes <richard@hughsie.com>
+ * Copyright 2022 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuFirmware"
@@ -181,7 +181,7 @@ fu_fdt_image_get_attr(FuFdtImage *self, const gchar *key, GError **error)
 
 	blob = g_hash_table_lookup(priv->hash_attrs, key);
 	if (blob == NULL) {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND, "no data for %s", key);
+		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND, "no data for %s", key);
 		return NULL;
 	}
 
@@ -216,8 +216,8 @@ fu_fdt_image_get_attr_u32(FuFdtImage *self, const gchar *key, guint32 *val, GErr
 		return FALSE;
 	if (g_bytes_get_size(blob) != sizeof(guint32)) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "invalid data size for %s, got 0x%x, expected 0x%x",
 			    key,
 			    (guint)g_bytes_get_size(blob),
@@ -256,8 +256,8 @@ fu_fdt_image_get_attr_u64(FuFdtImage *self, const gchar *key, guint64 *val, GErr
 		return FALSE;
 	if (g_bytes_get_size(blob) != sizeof(guint64)) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "invalid data size for %s, got 0x%x, expected 0x%x",
 			    key,
 			    (guint)g_bytes_get_size(blob),
@@ -298,8 +298,8 @@ fu_fdt_image_get_attr_strlist(FuFdtImage *self, const gchar *key, gchar ***val,
 		return FALSE;
 	if (g_bytes_get_size(blob) == 0) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "invalid data size for %s, got 0x%x",
 			    key,
 			    (guint)g_bytes_get_size(blob));
@@ -311,8 +311,8 @@ fu_fdt_image_get_attr_strlist(FuFdtImage *self, const gchar *key, gchar ***val,
 	for (gsize i = 0; i < bufsz; i++) {
 		if (buf[i] != 0x0 && !g_ascii_isprint((gchar)buf[i])) {
 			g_set_error(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "nonprintable character 0x%02x at offset 0x%x in %s",
 				    buf[i],
 				    (guint)i,
@@ -356,8 +356,8 @@ fu_fdt_image_get_attr_str(FuFdtImage *self, const gchar *key, gchar **val, GErro
 		return FALSE;
 	if (g_bytes_get_size(blob) == 0) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "invalid data size for %s, got 0x%x",
 			    key,
 			    (guint)g_bytes_get_size(blob));
@@ -369,8 +369,8 @@ fu_fdt_image_get_attr_str(FuFdtImage *self, const gchar *key, gchar **val, GErro
 	for (gsize i = 0; i < bufsz; i++) {
 		if (buf[i] != 0x0 && !g_ascii_isprint((gchar)buf[i])) {
 			g_set_error(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "nonprintable character 0x%02x at offset 0x%x in %s",
 				    buf[i],
 				    (guint)i,
@@ -526,14 +526,14 @@ fu_fdt_image_build_metadata_node(FuFdtImage *self, XbNode *n, GError **error)
 
 	key = xb_node_get_attr(n, "key");
 	if (key == NULL) {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "key invalid");
+		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_INVALID_DATA, "key invalid");
 		return FALSE;
 	}
 	format = xb_node_get_attr(n, "format");
 	if (format == NULL) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "format unspecified for %s, expected uint64|uint32|str|strlist|data",
 			    key);
 		return FALSE;
@@ -544,7 +544,12 @@ fu_fdt_image_build_metadata_node(FuFdtImage *self, XbNode *n, GError **error)
 	if (g_strcmp0(format, FU_FDT_IMAGE_FORMAT_UINT32) == 0) {
 		guint64 tmp = 0;
 		if (value != NULL) {
-			if (!fu_strtoull(value, &tmp, 0x0, G_MAXUINT32, error))
+			if (!fu_strtoull(value,
+					 &tmp,
+					 0x0,
+					 G_MAXUINT32,
+					 FU_INTEGER_BASE_AUTO,
+					 error))
 				return FALSE;
 		}
 		fu_fdt_image_set_attr_uint32(self, key, tmp);
@@ -553,7 +558,12 @@ fu_fdt_image_build_metadata_node(FuFdtImage *self, XbNode *n, GError **error)
 	if (g_strcmp0(format, FU_FDT_IMAGE_FORMAT_UINT64) == 0) {
 		guint64 tmp = 0;
 		if (value != NULL) {
-			if (!fu_strtoull(value, &tmp, 0x0, G_MAXUINT64, error))
+			if (!fu_strtoull(value,
+					 &tmp,
+					 0x0,
+					 G_MAXUINT64,
+					 FU_INTEGER_BASE_AUTO,
+					 error))
 				return FALSE;
 		}
 		fu_fdt_image_set_attr_uint64(self, key, tmp);
@@ -593,8 +603,8 @@ fu_fdt_image_build_metadata_node(FuFdtImage *self, XbNode *n, GError **error)
 
 	/* failed */
 	g_set_error(error,
-		    G_IO_ERROR,
-		    G_IO_ERROR_INVALID_DATA,
+		    FWUPD_ERROR,
+		    FWUPD_ERROR_INVALID_DATA,
 		    "format for %s invalid, expected uint64|uint32|str|strlist|data",
 		    key);
 	return FALSE;
@@ -642,11 +652,11 @@ fu_fdt_image_finalize(GObject *object)
 static void
 fu_fdt_image_class_init(FuFdtImageClass *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	object_class->finalize = fu_fdt_image_finalize;
-	klass_firmware->export = fu_fdt_image_export;
-	klass_firmware->build = fu_fdt_image_build;
+	firmware_class->export = fu_fdt_image_export;
+	firmware_class->build = fu_fdt_image_build;
 }
 
 /**
