@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
+ * Copyright 2021 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuIfdBios"
@@ -10,8 +10,9 @@
 
 #include "fu-bytes.h"
 #include "fu-common.h"
-#include "fu-efi-firmware-volume.h"
+#include "fu-efi-volume.h"
 #include "fu-ifd-bios.h"
+#include "fu-input-stream.h"
 #include "fu-mem.h"
 
 /**
@@ -25,54 +26,38 @@
 G_DEFINE_TYPE(FuIfdBios, fu_ifd_bios, FU_TYPE_IFD_IMAGE)
 
 #define FU_IFD_BIOS_FIT_SIGNATURE 0x5449465F
-#define FU_IFD_BIOS_FIT_SIZE	  0x150000
 
 static gboolean
 fu_ifd_bios_parse(FuFirmware *firmware,
-		  GBytes *fw,
-		  gsize offset,
+		  GInputStream *stream,
 		  FwupdInstallFlags flags,
 		  GError **error)
 {
-	gsize bufsz = 0;
-	guint32 sig;
+	gsize offset = 0;
+	gsize streamsz = 0;
 	guint img_cnt = 0;
-	const guint8 *buf = g_bytes_get_data(fw, &bufsz);
 
-	/* jump 16MiB as required */
-	if (bufsz > 0x100000)
-		offset += 0x100000;
+	/* get size */
+	if (!fu_input_stream_size(stream, &streamsz, error))
+		return FALSE;
 
 	/* read each volume in order */
-	while (offset < bufsz) {
-		g_autoptr(FuFirmware) firmware_tmp = NULL;
-
-		/* ignore _FIT_ as EOF */
-		if (!fu_memread_uint32_safe(buf, bufsz, offset, &sig, G_LITTLE_ENDIAN, error)) {
-			g_prefix_error(error, "failed to read start signature: ");
-			return FALSE;
-		}
-		if (sig == FU_IFD_BIOS_FIT_SIGNATURE)
-			break;
-		if (sig == 0xffffffff)
-			break;
+	while (offset < streamsz) {
+		g_autoptr(FuFirmware) firmware_tmp = fu_efi_volume_new();
+		g_autoptr(GError) error_local = NULL;
 
 		/* FV */
-		firmware_tmp = fu_firmware_new_from_gtypes(fw,
-							   offset,
-							   flags,
-							   error,
-							   FU_TYPE_EFI_FIRMWARE_VOLUME,
-							   G_TYPE_INVALID);
-		if (firmware_tmp == NULL) {
-			g_prefix_error(error,
-				       "failed to read @0x%x of 0x%x: ",
-				       (guint)offset,
-				       (guint)bufsz);
-			return FALSE;
+		if (!fu_firmware_parse_stream(firmware_tmp, stream, offset, flags, &error_local)) {
+			g_debug("failed to read volume @0x%x of 0x%x: %s",
+				(guint)offset,
+				(guint)streamsz,
+				error_local->message);
+			offset += 0x1000;
+			continue;
 		}
 		fu_firmware_set_offset(firmware_tmp, offset);
-		fu_firmware_add_image(firmware, firmware_tmp);
+		if (!fu_firmware_add_image_full(firmware, firmware_tmp, error))
+			return FALSE;
 
 		/* next! */
 		offset += fu_firmware_get_size(firmware_tmp);
@@ -102,8 +87,8 @@ fu_ifd_bios_init(FuIfdBios *self)
 static void
 fu_ifd_bios_class_init(FuIfdBiosClass *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
-	klass_firmware->parse = fu_ifd_bios_parse;
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->parse = fu_ifd_bios_parse;
 }
 
 /**
