@@ -1,5 +1,5 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #[derive(ToString)]
 enum FuIfdRegion {
@@ -16,7 +16,8 @@ enum FuIfdRegion {
     Max = 0x0F,
 }
 
-#[derive(ParseBytes, New, ValidateBytes)]
+#[derive(ParseStream, New, ValidateStream, Default)]
+#[repr(C, packed)]
 struct FuStructIfdFdbar {
     reserved: [u8; 16] = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF,
     signature: u32le == 0x0FF0A55A,
@@ -25,7 +26,8 @@ struct FuStructIfdFdbar {
     descriptor_map2: u32le,
 }
 
-#[derive(ParseBytes, New)]
+#[derive(ParseStream, New)]
+#[repr(C, packed)]
 struct FuStructIfdFcba {
     flcomp: u32le,
     flill: u32le,
