@@ -1,10 +1,10 @@
 /*
- * Copyright (C) 2021 Dell Inc.
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2020 Mario Limonciello <mario.limonciello@dell.com>
- * Copyright (C) 2017 Intel Corporation.
+ * Copyright 2021 Dell Inc.
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2020 Mario Limonciello <mario.limonciello@dell.com>
+ * Copyright 2017 Intel Corporation.
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuFirmware"
@@ -12,8 +12,10 @@
 #include "config.h"
 
 #include "fu-byte-array.h"
+#include "fu-input-stream.h"
 #include "fu-intel-thunderbolt-firmware.h"
 #include "fu-mem.h"
+#include "fu-partial-input-stream.h"
 
 /**
  * FuIntelThunderboltFirmware:
@@ -29,32 +31,31 @@ G_DEFINE_TYPE(FuIntelThunderboltFirmware,
 	      FU_TYPE_INTEL_THUNDERBOLT_NVM)
 
 static gboolean
-fu_intel_thunderbolt_nvm_valid_farb_pointer(guint32 pointer)
+fu_intel_thunderbolt_firmware_nvm_valid_farb_pointer(guint32 pointer)
 {
 	return pointer != 0 && pointer != 0xFFFFFF;
 }
 
 static gboolean
 fu_intel_thunderbolt_firmware_parse(FuFirmware *firmware,
-				    GBytes *fw,
-				    gsize offset,
+				    GInputStream *stream,
 				    FwupdInstallFlags flags,
 				    GError **error)
 {
 	const guint32 farb_offsets[] = {0x0, 0x1000};
 	gboolean valid = FALSE;
 	guint32 farb_pointer = 0x0;
+	g_autoptr(GInputStream) partial_stream = NULL;
 
 	/* get header offset */
 	for (guint i = 0; i < G_N_ELEMENTS(farb_offsets); i++) {
-		if (!fu_memread_uint24_safe(g_bytes_get_data(fw, NULL),
-					    g_bytes_get_size(fw),
-					    offset + farb_offsets[i],
-					    &farb_pointer,
-					    G_LITTLE_ENDIAN,
-					    error))
+		if (!fu_input_stream_read_u24(stream,
+					      farb_offsets[i],
+					      &farb_pointer,
+					      G_LITTLE_ENDIAN,
+					      error))
 			return FALSE;
-		if (fu_intel_thunderbolt_nvm_valid_farb_pointer(farb_pointer)) {
+		if (fu_intel_thunderbolt_firmware_nvm_valid_farb_pointer(farb_pointer)) {
 			valid = TRUE;
 			break;
 		}
@@ -70,8 +71,13 @@ fu_intel_thunderbolt_firmware_parse(FuFirmware *firmware,
 	fu_firmware_set_offset(firmware, farb_pointer);
 
 	/* FuIntelThunderboltNvm->parse */
+	partial_stream = fu_partial_input_stream_new(stream, farb_pointer, G_MAXSIZE, error);
+	if (partial_stream == NULL) {
+		g_prefix_error(error, "failed to cut from NVM: ");
+		return FALSE;
+	}
 	return FU_FIRMWARE_CLASS(fu_intel_thunderbolt_firmware_parent_class)
-	    ->parse(firmware, fw, offset + farb_pointer, flags, error);
+	    ->parse(firmware, partial_stream, flags, error);
 }
 
 static GByteArray *
@@ -112,9 +118,9 @@ fu_intel_thunderbolt_firmware_init(FuIntelThunderboltFirmware *self)
 static void
 fu_intel_thunderbolt_firmware_class_init(FuIntelThunderboltFirmwareClass *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
-	klass_firmware->parse = fu_intel_thunderbolt_firmware_parse;
-	klass_firmware->write = fu_intel_thunderbolt_firmware_write;
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->parse = fu_intel_thunderbolt_firmware_parse;
+	firmware_class->write = fu_intel_thunderbolt_firmware_write;
 }
 
 /**
