@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuIOChannel"
@@ -17,11 +17,16 @@
 #endif
 #include <string.h>
 #include <sys/stat.h>
+#ifdef HAVE_MEMFD_CREATE
+#include <sys/mman.h>
+#endif
 
 #include "fwupd-error.h"
 
 #include "fu-common.h"
+#include "fu-input-stream.h"
 #include "fu-io-channel.h"
+#include "fu-string.h"
 
 /**
  * FuIOChannel:
@@ -78,6 +83,50 @@ fu_io_channel_shutdown(FuIOChannel *self, GError **error)
 	return TRUE;
 }
 
+/**
+ * fu_io_channel_seek:
+ * @self: a #FuIOChannel
+ * @offset: an absolute offset in bytes
+ * @error: (nullable): optional return location for an error
+ *
+ * Seeks the file descriptor to a specific offset.
+ *
+ * Returns: %TRUE if all the seek worked.
+ *
+ * Since: 2.0.0
+ **/
+gboolean
+fu_io_channel_seek(FuIOChannel *self, gsize offset, GError **error)
+{
+	g_return_val_if_fail(FU_IS_IO_CHANNEL(self), FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	if (self->fd == -1) {
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_NOT_SUPPORTED,
+				    "channel is not open");
+		return FALSE;
+	}
+	if (lseek(self->fd, offset, SEEK_SET) < 0) {
+		g_set_error(error,
+			    G_IO_ERROR, /* nocheck:error */
+#ifdef HAVE_ERRNO_H
+			    g_io_error_from_errno(errno),
+#else
+			    G_IO_ERROR_FAILED, /* nocheck:blocked */
+#endif
+			    "failed to seek to 0x%04x: %s",
+			    (guint)offset,
+			    g_strerror(errno));
+		fwupd_error_convert(error);
+		return FALSE;
+	}
+
+	/* success */
+	return TRUE;
+}
+
 static gboolean
 fu_io_channel_flush_input(FuIOChannel *self, GError **error)
 {
@@ -120,6 +169,54 @@ fu_io_channel_write_bytes(FuIOChannel *self,
 	return fu_io_channel_write_raw(self, buf, bufsz, timeout_ms, flags, error);
 }
 
+typedef struct {
+	FuIOChannel *self;
+	guint timeout_ms;
+	FuIOChannelFlags flags;
+} FuIOChannelWriteStreamHelper;
+
+static gboolean
+fu_io_channel_write_stream_cb(const guint8 *buf, gsize bufsz, gpointer user_data, GError **error)
+{
+	FuIOChannelWriteStreamHelper *helper = (FuIOChannelWriteStreamHelper *)user_data;
+	return fu_io_channel_write_raw(helper->self,
+				       buf,
+				       bufsz,
+				       helper->timeout_ms,
+				       helper->flags,
+				       error);
+}
+
+/**
+ * fu_io_channel_write_stream:
+ * @self: a #FuIOChannel
+ * @stream: #GInputStream to write
+ * @timeout_ms: timeout in ms
+ * @flags: channel flags, e.g. %FU_IO_CHANNEL_FLAG_SINGLE_SHOT
+ * @error: (nullable): optional return location for an error
+ *
+ * Writes the stream to the fd, chucking when required.
+ *
+ * Returns: %TRUE if all the bytes was written
+ *
+ * Since: 2.0.0
+ **/
+gboolean
+fu_io_channel_write_stream(FuIOChannel *self,
+			   GInputStream *stream,
+			   guint timeout_ms,
+			   FuIOChannelFlags flags,
+			   GError **error)
+{
+	FuIOChannelWriteStreamHelper helper = {.self = self,
+					       .timeout_ms = timeout_ms,
+					       .flags = flags};
+	g_return_val_if_fail(FU_IS_IO_CHANNEL(self), FALSE);
+	g_return_val_if_fail(G_IS_INPUT_STREAM(stream), FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+	return fu_input_stream_chunkify(stream, fu_io_channel_write_stream_cb, &helper, error);
+}
+
 /**
  * fu_io_channel_write_byte_array:
  * @self: a #FuIOChannel
@@ -191,8 +288,8 @@ fu_io_channel_write_raw(FuIOChannel *self,
 				return FALSE;
 			}
 			g_set_error(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_FAILED,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_WRITE,
 				    "failed to write: "
 				    "wrote %" G_GSSIZE_FORMAT " of %" G_GSIZE_FORMAT,
 				    wrote,
@@ -324,11 +421,16 @@ fu_io_channel_read_byte_array(FuIOChannel *self,
 			gssize len = read(self->fd, buf_tmp->data, buf_tmp->len);
 			if (len < 0) {
 				g_set_error(error,
-					    FWUPD_ERROR,
-					    FWUPD_ERROR_READ,
+					    G_IO_ERROR, /* nocheck:error */
+#ifdef HAVE_ERRNO_H
+					    g_io_error_from_errno(errno),
+#else
+					    G_IO_ERROR_FAILED, /* nocheck:blocked */
+#endif
 					    "failed to read %i: %s",
 					    self->fd,
 					    g_strerror(errno));
+				fwupd_error_convert(error);
 				return NULL;
 			}
 			if (len == 0)
@@ -345,7 +447,7 @@ fu_io_channel_read_byte_array(FuIOChannel *self,
 		/* wait for data to appear */
 		gint rc = g_poll(&fds, 1, (gint)timeout_ms);
 		if (rc == 0) {
-			g_set_error(error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT, "timeout");
+			g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_TIMED_OUT, "timeout");
 			return NULL;
 		}
 		if (rc < 0) {
@@ -368,11 +470,16 @@ fu_io_channel_read_byte_array(FuIOChannel *self,
 				if (errno == EAGAIN)
 					continue;
 				g_set_error(error,
-					    FWUPD_ERROR,
-					    FWUPD_ERROR_READ,
+					    G_IO_ERROR, /* nocheck:error */
+#ifdef HAVE_ERRNO_H
+					    g_io_error_from_errno(errno),
+#else
+					    G_IO_ERROR_FAILED, /* nocheck:blocked */
+#endif
 					    "failed to read %i: %s",
 					    self->fd,
 					    g_strerror(errno));
+				fwupd_error_convert(error);
 				return NULL;
 			}
 			if (len == 0)
@@ -414,7 +521,7 @@ fu_io_channel_read_byte_array(FuIOChannel *self,
 	if (buf->len == 0) {
 		g_set_error(error,
 			    FWUPD_ERROR,
-			    FWUPD_ERROR_READ,
+			    FWUPD_ERROR_TIMED_OUT,
 			    "no data received from device in %ums",
 			    timeout_ms);
 		return NULL;
@@ -457,7 +564,7 @@ fu_io_channel_read_raw(FuIOChannel *self,
 	if (tmp == NULL)
 		return FALSE;
 	if (buf != NULL)
-		memcpy(buf, tmp->data, MIN(tmp->len, bufsz));
+		memcpy(buf, tmp->data, MIN(tmp->len, bufsz)); /* nocheck:blocked */
 	if (bytes_read != NULL)
 		*bytes_read = tmp->len;
 	return TRUE;
@@ -507,40 +614,96 @@ fu_io_channel_unix_new(gint fd)
 /**
  * fu_io_channel_new_file:
  * @filename: device file
+ * @open_flags: some #FuIoChannelOpenFlag typically %FU_IO_CHANNEL_OPEN_FLAG_READ
  * @error: (nullable): optional return location for an error
  *
- * Creates a new object to write and read from.
+ * Creates a new object to write and/or read from.
  *
  * Returns: a #FuIOChannel
  *
- * Since: 1.2.2
+ * Since: 2.0.0
  **/
 FuIOChannel *
-fu_io_channel_new_file(const gchar *filename, GError **error)
+fu_io_channel_new_file(const gchar *filename, FuIoChannelOpenFlag open_flags, GError **error)
 {
-#ifdef HAVE_POLL_H
 	gint fd;
+	int flags = 0;
 
 	g_return_val_if_fail(filename != NULL, NULL);
 	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
-	fd = g_open(filename, O_RDWR | O_NONBLOCK, S_IRWXU);
+	if (open_flags & FU_IO_CHANNEL_OPEN_FLAG_READ &&
+	    open_flags & FU_IO_CHANNEL_OPEN_FLAG_WRITE) {
+		flags |= O_RDWR;
+	} else if (open_flags & FU_IO_CHANNEL_OPEN_FLAG_READ) {
+		flags |= O_RDONLY;
+	} else if (open_flags & FU_IO_CHANNEL_OPEN_FLAG_WRITE) {
+		flags |= O_WRONLY;
+	}
+#ifdef O_NONBLOCK
+	if (open_flags & FU_IO_CHANNEL_OPEN_FLAG_NONBLOCK)
+		flags |= O_NONBLOCK;
+#endif
+#ifdef O_SYNC
+	if (open_flags & FU_IO_CHANNEL_OPEN_FLAG_SYNC)
+		flags |= O_SYNC;
+#endif
+	fd = g_open(filename, flags, S_IRWXU);
 	if (fd < 0) {
 		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_INVALID_FILE,
-			    "failed to open %s",
-			    filename);
+			    G_IO_ERROR, /* nocheck:error */
+#ifdef HAVE_ERRNO_H
+			    g_io_error_from_errno(errno),
+#else
+			    G_IO_ERROR_FAILED, /* nocheck:blocked */
+#endif
+			    "failed to open %s: %s",
+			    filename,
+			    g_strerror(errno));
+		fwupd_error_convert(error);
+		return NULL;
+	}
+	return fu_io_channel_unix_new(fd);
+}
+
+/**
+ * fu_io_channel_virtual_new:
+ * @name: (not nullable): memfd name
+ * @error: (nullable): optional return location for an error
+ *
+ * Creates a new virtual object to write and/or read from.
+ *
+ * Returns: a #FuIOChannel
+ *
+ * Since: 2.0.0
+ **/
+FuIOChannel *
+fu_io_channel_virtual_new(const gchar *name, GError **error)
+{
+#ifdef HAVE_MEMFD_CREATE
+	gint fd;
+
+	g_return_val_if_fail(name != NULL, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+
+	fd = memfd_create(name, MFD_CLOEXEC);
+	if (fd < 0) {
+		g_set_error(error,
+			    G_IO_ERROR, /* nocheck:error */
+#ifdef HAVE_ERRNO_H
+			    g_io_error_from_errno(errno),
+#else
+			    G_IO_ERROR_FAILED, /* nocheck:blocked */
+#endif
+			    "failed to create %s: %s",
+			    name,
+			    g_strerror(errno));
+		fwupd_error_convert(error);
 		return NULL;
 	}
 	return fu_io_channel_unix_new(fd);
 #else
-	g_return_val_if_fail(filename != NULL, NULL);
-	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
-	g_set_error_literal(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "Not supported as <poll.h> is unavailable");
+	g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "memfd not supported");
 	return NULL;
 #endif
 }
