@@ -1,7 +1,8 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
-#[derive(New, ValidateBytes, ParseBytes)]
+#[derive(New, ValidateStream, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructOprom {
     signature: u16le == 0xAA55,
     image_size: u16le,		// of 512 bytes
@@ -14,7 +15,9 @@ struct FuStructOprom {
     pci_header_offset: u16le = $struct_size,
     expansion_header_offset: u16le,
 }
-#[derive(New, ParseBytes)]
+
+#[derive(New, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructOpromPci {
     signature: u32le == 0x52494350,
     vendor_id: u16le,
@@ -22,7 +25,7 @@ struct FuStructOpromPci {
     device_list_pointer: u16le,
     structure_length: u16le,
     structure_revision: u8,
-    class_code: u24,
+    class_code: u24le,
     image_length: u16le,		// of 512 bytes
     image_revision: u16le,
     code_type: u8,
