@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #pragma once
@@ -21,13 +21,13 @@
  * /usr/lib/[triplet]/fwupd-plugins-3)
  * @FU_PATH_KIND_SYSCONFDIR:		The configuration location (IE /etc)
  * @FU_PATH_KIND_SYSCONFDIR_PKG:	The package configuration location (IE /etc/fwupd)
+ * @FU_PATH_KIND_SYSFSDIR:	The sysfs base location (IE /sys)
  * @FU_PATH_KIND_SYSFSDIR_FW:		The sysfs firmware location (IE /sys/firmware)
  * @FU_PATH_KIND_SYSFSDIR_DRIVERS:	The platform sysfs directory (IE /sys/bus/platform/drivers)
  * @FU_PATH_KIND_SYSFSDIR_TPM:		The TPM sysfs directory (IE /sys/class/tpm)
  * @FU_PATH_KIND_PROCFS:		The procfs location (IE /proc)
  * @FU_PATH_KIND_POLKIT_ACTIONS:	The directory for policy kit actions (IE
  * /usr/share/polkit-1/actions/)
- * @FU_PATH_KIND_OFFLINE_TRIGGER:	The file for the offline trigger (IE /system-update)
  * @FU_PATH_KIND_SYSFSDIR_SECURITY:	The sysfs security location (IE /sys/kernel/security)
  * @FU_PATH_KIND_ACPI_TABLES:		The location of the ACPI tables
  * @FU_PATH_KIND_LOCKDIR:		The lock directory (IE /run/lock)
@@ -49,6 +49,9 @@
  * @FU_PATH_KIND_HOSTFS_BOOT:		The host boot directory, (IE /boot)
  * @FU_PATH_KIND_DEVFS: 		The host dev directory, (IE /dev)
  * @FU_PATH_KIND_LOCALTIME:		The timezone symlink (IE /etc/localtime)
+ * @FU_PATH_KIND_LIBEXECDIR:		The directory to launch executables
+ * @FU_PATH_KIND_LIBEXECDIR_PKG		The directory launch executables packaged with daemon
+ * @FU_PATH_KIND_DATADIR_VENDOR_IDS:	The vendor ID store (IE /usr/share/hwdata)
  *
  * Path types to use when dynamically determining a path at runtime
  **/
@@ -61,12 +64,12 @@ typedef enum {
 	FU_PATH_KIND_LIBDIR_PKG,
 	FU_PATH_KIND_SYSCONFDIR,
 	FU_PATH_KIND_SYSCONFDIR_PKG,
+	FU_PATH_KIND_SYSFSDIR,
 	FU_PATH_KIND_SYSFSDIR_FW,
 	FU_PATH_KIND_SYSFSDIR_DRIVERS,
 	FU_PATH_KIND_SYSFSDIR_TPM,
 	FU_PATH_KIND_PROCFS,
 	FU_PATH_KIND_POLKIT_ACTIONS,
-	FU_PATH_KIND_OFFLINE_TRIGGER,
 	FU_PATH_KIND_SYSFSDIR_SECURITY,
 	FU_PATH_KIND_ACPI_TABLES,
 	FU_PATH_KIND_LOCKDIR,
@@ -83,6 +86,9 @@ typedef enum {
 	FU_PATH_KIND_HOSTFS_BOOT,
 	FU_PATH_KIND_DEVFS,
 	FU_PATH_KIND_LOCALTIME,
+	FU_PATH_KIND_LIBEXECDIR,
+	FU_PATH_KIND_LIBEXECDIR_PKG,
+	FU_PATH_KIND_DATADIR_VENDOR_IDS,
 	/*< private >*/
 	FU_PATH_KIND_LAST
 } FuPathKind;
@@ -104,3 +110,9 @@ fu_path_mkdir_parent(const gchar *filename, GError **error) G_GNUC_WARN_UNUSED_R
 gchar *
 fu_path_find_program(const gchar *basename, GError **error) G_GNUC_WARN_UNUSED_RESULT
     G_GNUC_NON_NULL(1);
+gchar *
+fu_path_make_absolute(const gchar *filename, GError **error) G_GNUC_WARN_UNUSED_RESULT
+    G_GNUC_NON_NULL(1);
+gchar *
+fu_path_get_symlink_target(const gchar *filename, GError **error) G_GNUC_WARN_UNUSED_RESULT
+    G_GNUC_NON_NULL(1);
