@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuSecurityAttrs"
@@ -13,6 +13,7 @@
 
 #include "fwupd-security-attr-private.h"
 
+#include "fu-security-attr.h"
 #include "fu-security-attrs-private.h"
 #include "fu-security-attrs.h"
 
@@ -30,7 +31,13 @@ struct _FuSecurityAttrs {
 /* probably sane to *not* make this part of the ABI */
 #define FWUPD_SECURITY_ATTR_ID_DOC_URL "https://fwupd.github.io/libfwupdplugin/hsi.html"
 
-G_DEFINE_TYPE(FuSecurityAttrs, fu_security_attrs, G_TYPE_OBJECT)
+static void
+fu_security_attrs_codec_iface_init(FwupdCodecInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE(FuSecurityAttrs,
+			fu_security_attrs,
+			G_TYPE_OBJECT,
+			G_IMPLEMENT_INTERFACE(FWUPD_TYPE_CODEC, fu_security_attrs_codec_iface_init))
 
 static void
 fu_security_attrs_finalize(GObject *obj)
@@ -70,6 +77,23 @@ fu_security_attrs_append_internal(FuSecurityAttrs *self, FwupdSecurityAttr *attr
 	g_ptr_array_add(self->attrs, g_object_ref(attr));
 }
 
+/**
+ * fu_security_attrs_is_valid:
+ * @self: a #FuSecurityAttrs
+ *
+ * Adds a #FwupdSecurityAttr to the array with no sanity checks.
+ *
+ * Returns: %TRUE if the collection is valid
+ *
+ * Since: 2.0.7
+ **/
+gboolean
+fu_security_attrs_is_valid(FuSecurityAttrs *self)
+{
+	g_return_val_if_fail(FU_IS_SECURITY_ATTRS(self), FALSE);
+	return self->attrs->len > 0;
+}
+
 /**
  * fu_security_attrs_append:
  * @self: a #FuSecurityAttrs
@@ -127,8 +151,8 @@ fu_security_attrs_get_by_appstream_id(FuSecurityAttrs *self,
 	g_return_val_if_fail(FU_IS_SECURITY_ATTRS(self), NULL);
 	if (self->attrs->len == 0) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_NOT_FOUND,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_NOT_FOUND,
 				    "no attributes are loaded");
 		return NULL;
 	}
@@ -137,7 +161,7 @@ fu_security_attrs_get_by_appstream_id(FuSecurityAttrs *self,
 		if (g_strcmp0(fwupd_security_attr_get_appstream_id(attr), appstream_id) == 0)
 			return g_object_ref(attr);
 	}
-	g_set_error(error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND, "no attr with ID %s", appstream_id);
+	g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND, "no attr with ID %s", appstream_id);
 	return NULL;
 }
 
@@ -163,7 +187,9 @@ fu_security_attrs_to_variant(FuSecurityAttrs *self)
 		FwupdSecurityAttr *attr = g_ptr_array_index(self->attrs, i);
 		if (fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_OBSOLETED))
 			continue;
-		g_variant_builder_add_value(&builder, fwupd_security_attr_to_variant(attr));
+		g_variant_builder_add_value(
+		    &builder,
+		    fwupd_codec_to_variant(FWUPD_CODEC(attr), FWUPD_CODEC_FLAG_NONE));
 	}
 	return g_variant_new("(aa{sv})", &builder);
 }
@@ -171,6 +197,7 @@ fu_security_attrs_to_variant(FuSecurityAttrs *self)
 /**
  * fu_security_attrs_get_all:
  * @self: a #FuSecurityAttrs
+ * @fwupd_version: (nullable): fwupd version string, e.g. `2.0.7`
  *
  * Gets all the non-obsoleted attributes in the object.
  *
@@ -179,7 +206,7 @@ fu_security_attrs_to_variant(FuSecurityAttrs *self)
  * Since: 1.5.0
  **/
 GPtrArray *
-fu_security_attrs_get_all(FuSecurityAttrs *self)
+fu_security_attrs_get_all(FuSecurityAttrs *self, const gchar *fwupd_version)
 {
 	g_autoptr(GPtrArray) all = g_ptr_array_new_with_free_func((GDestroyNotify)g_object_unref);
 	g_return_val_if_fail(FU_IS_SECURITY_ATTRS(self), NULL);
@@ -187,6 +214,8 @@ fu_security_attrs_get_all(FuSecurityAttrs *self)
 		FwupdSecurityAttr *attr = g_ptr_array_index(self->attrs, i);
 		if (fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_OBSOLETED))
 			continue;
+		if (!fu_security_attr_check_fwupd_version(attr, fwupd_version))
+			continue;
 		g_ptr_array_add(all, g_object_ref(attr));
 	}
 	return g_steal_pointer(&all);
@@ -210,16 +239,19 @@ fu_security_attrs_remove_all(FuSecurityAttrs *self)
 /**
  * fu_security_attrs_calculate_hsi:
  * @self: a #FuSecurityAttrs
+ * @fwupd_version: fwupd version, e.g. `2.0.7`
  * @flags: HSI attribute flags
  *
  * Calculates the HSI string from the appended attributes.
  *
  * Returns: (transfer full): a string or %NULL
  *
- * Since: 1.5.0
+ * Since: 2.0.7
  **/
 gchar *
-fu_security_attrs_calculate_hsi(FuSecurityAttrs *self, FuSecurityAttrsFlags flags)
+fu_security_attrs_calculate_hsi(FuSecurityAttrs *self,
+				const gchar *fwupd_version,
+				FuSecurityAttrsFlags flags)
 {
 	guint hsi_number = 0;
 	FwupdSecurityAttrFlags attr_flags = FWUPD_SECURITY_ATTR_FLAG_NONE;
@@ -240,6 +272,8 @@ fu_security_attrs_calculate_hsi(FuSecurityAttrs *self, FuSecurityAttrsFlags flag
 			FwupdSecurityAttr *attr = g_ptr_array_index(self->attrs, i);
 			if (fwupd_security_attr_get_level(attr) != j)
 				continue;
+			if (!fu_security_attr_check_fwupd_version(attr, fwupd_version))
+				continue;
 			if (fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_SUCCESS))
 				success_cnt++;
 			else if (!fwupd_security_attr_has_flag(attr,
@@ -264,6 +298,8 @@ fu_security_attrs_calculate_hsi(FuSecurityAttrs *self, FuSecurityAttrsFlags flag
 		if (fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_RUNTIME_ISSUE) &&
 		    fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_SUCCESS))
 			continue;
+		if (!fu_security_attr_check_fwupd_version(attr, fwupd_version))
+			continue;
 
 		attr_flags |= fwupd_security_attr_get_flags(attr);
 	}
@@ -339,10 +375,10 @@ static struct {
     {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_OTP, FWUPD_SECURITY_ATTR_LEVEL_IMPORTANT},
     {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_POLICY, FWUPD_SECURITY_ATTR_LEVEL_THEORETICAL},
     {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_VERIFIED, FWUPD_SECURITY_ATTR_LEVEL_IMPORTANT},
-    {FWUPD_SECURITY_ATTR_ID_INTEL_CET_ACTIVE, FWUPD_SECURITY_ATTR_LEVEL_THEORETICAL},
-    {FWUPD_SECURITY_ATTR_ID_INTEL_CET_ENABLED, FWUPD_SECURITY_ATTR_LEVEL_THEORETICAL},
+    {FWUPD_SECURITY_ATTR_ID_CET_ACTIVE, FWUPD_SECURITY_ATTR_LEVEL_THEORETICAL},
+    {FWUPD_SECURITY_ATTR_ID_CET_ENABLED, FWUPD_SECURITY_ATTR_LEVEL_THEORETICAL},
     {FWUPD_SECURITY_ATTR_ID_INTEL_GDS, FWUPD_SECURITY_ATTR_LEVEL_IMPORTANT},
-    {FWUPD_SECURITY_ATTR_ID_INTEL_SMAP, FWUPD_SECURITY_ATTR_LEVEL_SYSTEM_PROTECTION},
+    {FWUPD_SECURITY_ATTR_ID_SMAP, FWUPD_SECURITY_ATTR_LEVEL_SYSTEM_PROTECTION},
     {FWUPD_SECURITY_ATTR_ID_IOMMU, FWUPD_SECURITY_ATTR_LEVEL_IMPORTANT},
     {FWUPD_SECURITY_ATTR_ID_MEI_MANUFACTURING_MODE, FWUPD_SECURITY_ATTR_LEVEL_CRITICAL},
     {FWUPD_SECURITY_ATTR_ID_MEI_OVERRIDE_STRAP, FWUPD_SECURITY_ATTR_LEVEL_CRITICAL},
@@ -367,7 +403,9 @@ static struct {
     {FWUPD_SECURITY_ATTR_ID_UEFI_BOOTSERVICE_VARS, FWUPD_SECURITY_ATTR_LEVEL_CRITICAL},
     {FWUPD_SECURITY_ATTR_ID_BIOS_ROLLBACK_PROTECTION, FWUPD_SECURITY_ATTR_LEVEL_IMPORTANT},
     {FWUPD_SECURITY_ATTR_ID_BIOS_CAPSULE_UPDATES, FWUPD_SECURITY_ATTR_LEVEL_CRITICAL},
-    {NULL, FWUPD_SECURITY_ATTR_LEVEL_NONE}};
+    {FWUPD_SECURITY_ATTR_ID_AMD_SMM_LOCKED, FWUPD_SECURITY_ATTR_LEVEL_CRITICAL},
+    {FWUPD_SECURITY_ATTR_ID_UEFI_MEMORY_PROTECTION, FWUPD_SECURITY_ATTR_LEVEL_IMPORTANT},
+};
 
 static void
 fu_security_attrs_ensure_level(FwupdSecurityAttr *attr)
@@ -383,7 +421,7 @@ fu_security_attrs_ensure_level(FwupdSecurityAttr *attr)
 		return;
 
 	/* map ID to level in one place */
-	for (guint i = 0; appstream_id_level_map[i].appstream_id != NULL; i++) {
+	for (guint i = 0; i < G_N_ELEMENTS(appstream_id_level_map); i++) {
 		if (g_strcmp0(appstream_id, appstream_id_level_map[i].appstream_id) == 0) {
 			fwupd_security_attr_set_level(attr, appstream_id_level_map[i].level);
 			return;
@@ -395,6 +433,80 @@ fu_security_attrs_ensure_level(FwupdSecurityAttr *attr)
 	fwupd_security_attr_set_level(attr, FWUPD_SECURITY_ATTR_LEVEL_CRITICAL);
 }
 
+static struct {
+	const gchar *appstream_id;
+	const gchar *fwupd_version;
+} appstream_id_version_map[] = {
+    {FWUPD_SECURITY_ATTR_ID_AMD_ROLLBACK_PROTECTION, "1.8.0"},
+    {FWUPD_SECURITY_ATTR_ID_AMD_SMM_LOCKED, "2.0.2"},
+    {FWUPD_SECURITY_ATTR_ID_AMD_SPI_REPLAY_PROTECTION, "1.8.0"},
+    {FWUPD_SECURITY_ATTR_ID_AMD_SPI_WRITE_PROTECTION, "1.8.0"},
+    {FWUPD_SECURITY_ATTR_ID_BIOS_CAPSULE_UPDATES, "1.9.6"},
+    {FWUPD_SECURITY_ATTR_ID_BIOS_ROLLBACK_PROTECTION, "1.8.8"},
+    {FWUPD_SECURITY_ATTR_ID_CET_ACTIVE, "2.0.0"},
+    {FWUPD_SECURITY_ATTR_ID_CET_ENABLED, "2.0.0"},
+    {FWUPD_SECURITY_ATTR_ID_ENCRYPTED_RAM, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_ACM, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_ENABLED, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_OTP, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_POLICY, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_INTEL_BOOTGUARD_VERIFIED, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_INTEL_GDS, "1.9.4"},
+    {FWUPD_SECURITY_ATTR_ID_IOMMU, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_KERNEL_LOCKDOWN, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_KERNEL_TAINTED, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_MEI_KEY_MANIFEST, "1.8.7"},
+    {FWUPD_SECURITY_ATTR_ID_MEI_MANUFACTURING_MODE, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_MEI_OVERRIDE_STRAP, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_MEI_VERSION, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_PLATFORM_DEBUG_ENABLED, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_PLATFORM_DEBUG_LOCKED, "1.8.0"},
+    {FWUPD_SECURITY_ATTR_ID_PLATFORM_FUSED, "1.8.0"},
+    {FWUPD_SECURITY_ATTR_ID_PREBOOT_DMA_PROTECTION, "1.8.0"},
+    {FWUPD_SECURITY_ATTR_ID_SMAP, "2.0.0"},
+    {FWUPD_SECURITY_ATTR_ID_SPI_BIOSWE, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_SPI_BLE, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_SPI_DESCRIPTOR, "1.6.0"},
+    {FWUPD_SECURITY_ATTR_ID_SPI_SMM_BWP, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_SUPPORTED_CPU, "1.8.0"},
+    {FWUPD_SECURITY_ATTR_ID_SUSPEND_TO_IDLE, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_SUSPEND_TO_RAM, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_TPM_EMPTY_PCR, "1.7.2"},
+    {FWUPD_SECURITY_ATTR_ID_TPM_RECONSTRUCTION_PCR0, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_TPM_VERSION_20, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_UEFI_BOOTSERVICE_VARS, "1.9.3"},
+    {FWUPD_SECURITY_ATTR_ID_UEFI_MEMORY_PROTECTION, "2.0.7"},
+    {FWUPD_SECURITY_ATTR_ID_UEFI_PK, "1.5.0"},
+    {FWUPD_SECURITY_ATTR_ID_UEFI_SECUREBOOT, "1.5.0"},
+};
+
+static void
+fu_security_attrs_ensure_fwupd_version(FwupdSecurityAttr *attr)
+{
+	const gchar *appstream_id = fwupd_security_attr_get_appstream_id(attr);
+
+	/* already set */
+	if (fwupd_security_attr_get_fwupd_version(attr) != NULL)
+		return;
+
+	/* not required */
+	if (fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_RUNTIME_ISSUE))
+		return;
+
+	/* map ID to fwupd version in one place */
+	for (guint i = 0; i < G_N_ELEMENTS(appstream_id_version_map); i++) {
+		if (g_strcmp0(appstream_id, appstream_id_version_map[i].appstream_id) == 0) {
+			fwupd_security_attr_set_fwupd_version(
+			    attr,
+			    appstream_id_version_map[i].fwupd_version);
+			return;
+		}
+	}
+
+	/* somebody forgot to add to the level map... */
+	g_warning("cannot map %s to a fwupd version", appstream_id);
+}
+
 /**
  * fu_security_attrs_depsolve:
  * @self: a #FuSecurityAttrs
@@ -416,6 +528,7 @@ fu_security_attrs_depsolve(FuSecurityAttrs *self)
 	for (guint i = 0; i < self->attrs->len; i++) {
 		FwupdSecurityAttr *attr = g_ptr_array_index(self->attrs, i);
 		fu_security_attrs_ensure_level(attr);
+		fu_security_attrs_ensure_fwupd_version(attr);
 	}
 
 	/* set flat where required */
@@ -483,100 +596,41 @@ fu_security_attrs_depsolve(FuSecurityAttrs *self)
 }
 
 static void
-fu_security_attrs_to_json(FuSecurityAttrs *self, JsonBuilder *builder)
+fu_security_attrs_add_json(FwupdCodec *codec, JsonBuilder *builder, FwupdCodecFlags flags)
 {
+	FuSecurityAttrs *self = FU_SECURITY_ATTRS(codec);
 	g_autoptr(GPtrArray) items = NULL;
 
-	json_builder_begin_object(builder);
 	json_builder_set_member_name(builder, "SecurityAttributes");
 	json_builder_begin_array(builder);
-	items = fu_security_attrs_get_all(self);
+	items = fu_security_attrs_get_all(self, NULL);
 	for (guint i = 0; i < items->len; i++) {
 		FwupdSecurityAttr *attr = g_ptr_array_index(items, i);
 		guint64 created = fwupd_security_attr_get_created(attr);
 		if (fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_OBSOLETED))
 			continue;
-		json_builder_begin_object(builder);
 		fwupd_security_attr_set_created(attr, 0);
-		fwupd_security_attr_to_json(attr, builder);
-		fwupd_security_attr_set_created(attr, created);
+		json_builder_begin_object(builder);
+		fwupd_codec_to_json(FWUPD_CODEC(attr), builder, FWUPD_CODEC_FLAG_NONE);
 		json_builder_end_object(builder);
+		fwupd_security_attr_set_created(attr, created);
 	}
 	json_builder_end_array(builder);
-	json_builder_end_object(builder);
 }
 
-/**
- * fu_security_attrs_to_json_string:
- * @self: a pointer for a FuSecurityAttrs data structure.
- * @error: (nullable): optional return location for an error
- *
- * Convert security attribute to JSON string. e.g.:
- *    {
- *      "SecurityAttributes": [
- *        {
- *          "name": "aaa",
- *          "value": "bbb"
- *        }
- *      ]
- *    }
- *
- * Returns: (transfer full): JSON string
- *
- * Since: 1.9.2
- */
-gchar *
-fu_security_attrs_to_json_string(FuSecurityAttrs *self, GError **error)
-{
-	g_autofree gchar *data = NULL;
-	g_autoptr(JsonGenerator) json_generator = NULL;
-	g_autoptr(JsonBuilder) builder = json_builder_new();
-	g_autoptr(JsonNode) json_root = NULL;
-
-	g_return_val_if_fail(FU_IS_SECURITY_ATTRS(self), NULL);
-	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
-
-	fu_security_attrs_to_json(self, builder);
-	json_root = json_builder_get_root(builder);
-	json_generator = json_generator_new();
-	json_generator_set_pretty(json_generator, TRUE);
-	json_generator_set_root(json_generator, json_root);
-	data = json_generator_to_data(json_generator, NULL);
-	if (data == NULL) {
-		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_INTERNAL,
-			    "Failed to convert security attribute to json.");
-		return NULL;
-	}
-	return g_steal_pointer(&data);
-}
-
-/**
- * fu_security_attrs_from_json:
- * @self: a #FuSecurityAttrs
- * @json_node: a #JsonNode
- * @error: (nullable): optional return location for an error
- *
- * Imports a JSON node.
- *
- * Returns: %TRUE on success
- *
- * Since: 1.9.2
- */
-gboolean
-fu_security_attrs_from_json(FuSecurityAttrs *self, JsonNode *json_node, GError **error)
+static gboolean
+fu_security_attrs_from_json(FwupdCodec *codec, JsonNode *json_node, GError **error)
 {
+	FuSecurityAttrs *self = FU_SECURITY_ATTRS(codec);
 	JsonArray *array;
 	JsonObject *obj;
 
-	g_return_val_if_fail(FU_IS_SECURITY_ATTRS(self), FALSE);
-	g_return_val_if_fail(json_node != NULL, FALSE);
-	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
-
 	/* sanity check */
 	if (!JSON_NODE_HOLDS_OBJECT(json_node)) {
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "not JSON object");
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
+				    "not JSON object");
 		return FALSE;
 	}
 	obj = json_node_get_object(json_node);
@@ -584,8 +638,8 @@ fu_security_attrs_from_json(FuSecurityAttrs *self, JsonNode *json_node, GError *
 	/* this has to exist */
 	if (!json_object_has_member(obj, "SecurityAttributes")) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "no SecurityAttributes property in object");
 		return FALSE;
 	}
@@ -593,7 +647,7 @@ fu_security_attrs_from_json(FuSecurityAttrs *self, JsonNode *json_node, GError *
 	for (guint i = 0; i < json_array_get_length(array); i++) {
 		JsonNode *node_tmp = json_array_get_element(array, i);
 		g_autoptr(FwupdSecurityAttr) attr = fwupd_security_attr_new(NULL);
-		if (!fwupd_security_attr_from_json(attr, node_tmp, error))
+		if (!fwupd_codec_from_json(FWUPD_CODEC(attr), node_tmp, error))
 			return FALSE;
 		if (fwupd_security_attr_has_flag(attr, FWUPD_SECURITY_ATTR_FLAG_OBSOLETED))
 			continue;
@@ -604,6 +658,13 @@ fu_security_attrs_from_json(FuSecurityAttrs *self, JsonNode *json_node, GError *
 	return TRUE;
 }
 
+static void
+fu_security_attrs_codec_iface_init(FwupdCodecInterface *iface)
+{
+	iface->add_json = fu_security_attrs_add_json;
+	iface->from_json = fu_security_attrs_from_json;
+}
+
 /**
  * fu_security_attrs_compare:
  * @attrs1: a #FuSecurityAttrs
@@ -623,8 +684,8 @@ fu_security_attrs_compare(FuSecurityAttrs *attrs1, FuSecurityAttrs *attrs2)
 {
 	g_autoptr(GHashTable) hash1 = g_hash_table_new(g_str_hash, g_str_equal);
 	g_autoptr(GHashTable) hash2 = g_hash_table_new(g_str_hash, g_str_equal);
-	g_autoptr(GPtrArray) array1 = fu_security_attrs_get_all(attrs1);
-	g_autoptr(GPtrArray) array2 = fu_security_attrs_get_all(attrs2);
+	g_autoptr(GPtrArray) array1 = fu_security_attrs_get_all(attrs1, NULL);
+	g_autoptr(GPtrArray) array2 = fu_security_attrs_get_all(attrs2, NULL);
 	g_autoptr(GPtrArray) results =
 	    g_ptr_array_new_with_free_func((GDestroyNotify)g_object_unref);
 
