@@ -16,6 +16,7 @@
 #include "fu-device-private.h"
 #include "fu-dump.h"
 #include "fu-input-stream.h"
+#include "fu-linear-firmware.h"
 #include "fu-mem.h"
 #include "fu-string.h"
 #include "fu-usb-bos-descriptor-private.h"
@@ -49,6 +50,7 @@ typedef struct {
 	gint configuration;
 	GPtrArray *device_interfaces; /* (nullable) (element-type FuUsbDeviceInterface) */
 	guint claim_retry_count;
+	GPtrArray *descriptor_streams; /* (nullable) (element-type GInputStream) */
 } FuUsbDevicePrivate;
 
 typedef struct {
@@ -56,17 +58,10 @@ typedef struct {
 	gboolean claimed;
 } FuUsbDeviceInterface;
 
-static void
-fu_usb_device_codec_iface_init(FwupdCodecInterface *iface);
 static gboolean
 fu_usb_device_ensure_interfaces(FuUsbDevice *self, GError **error);
 
-G_DEFINE_TYPE_EXTENDED(FuUsbDevice,
-		       fu_usb_device,
-		       FU_TYPE_UDEV_DEVICE,
-		       0,
-		       G_ADD_PRIVATE(FuUsbDevice)
-			   G_IMPLEMENT_INTERFACE(FWUPD_TYPE_CODEC, fu_usb_device_codec_iface_init));
+G_DEFINE_TYPE_WITH_PRIVATE(FuUsbDevice, fu_usb_device, FU_TYPE_UDEV_DEVICE);
 
 enum { PROP_0, PROP_LIBUSB_DEVICE, PROP_LAST };
 
@@ -277,6 +272,8 @@ fu_usb_device_finalize(GObject *object)
 		libusb_unref_device(priv->usb_device);
 	if (priv->device_interfaces != NULL)
 		g_ptr_array_unref(priv->device_interfaces);
+	if (priv->descriptor_streams != NULL)
+		g_ptr_array_unref(priv->descriptor_streams);
 	g_ptr_array_unref(priv->interfaces);
 	g_ptr_array_unref(priv->bos_descriptors);
 	g_ptr_array_unref(priv->hid_descriptors);
@@ -889,6 +886,10 @@ fu_usb_device_probe_bos_descriptor(FuUsbDevice *self, FuUsbBosDescriptor *bos, G
 	str = fu_firmware_to_string(ds20);
 	g_debug("DS20: %s", str);
 
+	/* Microsoft descriptors are not useful at the moment */
+	if (FU_IS_USB_DEVICE_MS_DS20(ds20))
+		return TRUE;
+
 	/* set the quirks onto the device */
 	usb_locker = fu_device_locker_new_full(self,
 					       (FuDeviceLockerFunc)fu_usb_device_open,
@@ -966,33 +967,47 @@ static gboolean
 fu_usb_device_parse_bos_descriptor(FuUsbDevice *self, GInputStream *stream, GError **error)
 {
 	FuUsbDevicePrivate *priv = GET_PRIVATE(self);
-	gsize offset = 0;
-	gsize streamsz = 0;
+	g_autoptr(FuFirmware) firmware = fu_linear_firmware_new(FU_TYPE_USB_BOS_DESCRIPTOR);
+	g_autoptr(GError) error_local = NULL;
+	g_autoptr(GPtrArray) imgs = NULL;
 
-	if (!fu_input_stream_size(stream, &streamsz, error))
-		return FALSE;
-	while (offset < streamsz) {
-		g_autoptr(GError) error_local = NULL;
-		g_autoptr(FuUsbBosDescriptor) bos_descriptor =
-		    g_object_new(FU_TYPE_USB_BOS_DESCRIPTOR, NULL);
-		if (!fu_firmware_parse_stream(FU_FIRMWARE(bos_descriptor),
-					      stream,
-					      offset,
-					      FWUPD_INSTALL_FLAG_NONE,
-					      &error_local)) {
-			if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_INVALID_FILE))
-				break;
-			if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_INVALID_DATA))
-				break;
+	if (!fu_firmware_parse_stream(firmware,
+				      stream,
+				      0x0,
+				      FWUPD_INSTALL_FLAG_NONE,
+				      &error_local)) {
+		if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_INVALID_FILE) ||
+		    g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_INVALID_DATA)) {
+			g_debug("ignoring: %s", error_local->message);
+		} else {
 			g_propagate_error(error, g_steal_pointer(&error_local));
 			return FALSE;
 		}
-		offset += fu_firmware_get_size(FU_FIRMWARE(bos_descriptor));
-		g_ptr_array_add(priv->bos_descriptors, g_steal_pointer(&bos_descriptor));
 	}
+
+	/* copy from container */
+	imgs = fu_firmware_get_images(firmware);
+	for (guint i = 0; i < imgs->len; i++) {
+		FuFirmware *img = g_ptr_array_index(imgs, i);
+		g_ptr_array_add(priv->bos_descriptors, g_object_ref(img));
+	}
+
+	/* success */
 	return TRUE;
 }
 
+/* these are closed on FuDevice->probe_complete() */
+static void
+fu_usb_device_add_descriptor_stream(FuUsbDevice *self, GInputStream *stream)
+{
+	FuUsbDevicePrivate *priv = GET_PRIVATE(self);
+	if (priv->descriptor_streams == NULL) {
+		priv->descriptor_streams =
+		    g_ptr_array_new_with_free_func((GDestroyNotify)fu_input_stream_locker_unref);
+	}
+	g_ptr_array_add(priv->descriptor_streams, g_object_ref(stream));
+}
+
 static gboolean
 fu_usb_device_ensure_bos_descriptors(FuUsbDevice *self, GError **error)
 {
@@ -1043,8 +1058,8 @@ fu_usb_device_ensure_bos_descriptors(FuUsbDevice *self, GError **error)
 		}
 		libusb_free_bos_descriptor(bos);
 	} else {
-		g_autoptr(GInputStream) stream = NULL;
 		g_autoptr(GError) error_local = NULL;
+		g_autoptr(GInputStream) stream = NULL;
 
 		/* this is optional */
 		stream =
@@ -1056,6 +1071,7 @@ fu_usb_device_ensure_bos_descriptors(FuUsbDevice *self, GError **error)
 				return FALSE;
 			}
 		} else {
+			fu_usb_device_add_descriptor_stream(self, stream); /* ->probe_complete */
 			if (!fu_usb_device_parse_bos_descriptor(self, stream, error))
 				return FALSE;
 		}
@@ -1297,6 +1313,18 @@ fu_usb_device_probe(FuDevice *device, GError **error)
 	return TRUE;
 }
 
+static void
+fu_usb_device_probe_complete(FuDevice *device)
+{
+	FuUsbDevice *self = FU_USB_DEVICE(device);
+	FuUsbDevicePrivate *priv = GET_PRIVATE(self);
+
+	/* FuUdevDevice->probe_complete */
+	FU_DEVICE_CLASS(fu_usb_device_parent_class)->probe_complete(device);
+	if (priv->descriptor_streams != NULL)
+		g_ptr_array_set_size(priv->descriptor_streams, 0);
+}
+
 /**
  * fu_usb_device_get_release:
  * @self: a #FuUsbDevice
@@ -1914,7 +1942,7 @@ fu_usb_device_ensure_interfaces(FuUsbDevice *self, GError **error)
 		}
 		libusb_free_config_descriptor(config);
 	} else {
-		g_autoptr(GInputStream) stream = NULL;
+		g_autoptr(FuInputStreamLocker) stream = NULL;
 		g_autoptr(GError) error_local = NULL;
 
 		stream = fu_usb_device_load_descriptor_stream(self, "descriptors", &error_local);
@@ -2744,42 +2772,16 @@ fu_usb_device_get_hid_descriptors(FuUsbDevice *self, GError **error)
 }
 
 static gboolean
-fu_usb_device_from_json(FwupdCodec *codec, JsonNode *json_node, GError **error)
+fu_usb_device_from_json(FuDevice *device, JsonObject *json_object, GError **error)
 {
-	FuUsbDevice *self = FU_USB_DEVICE(codec);
+	FuUsbDevice *self = FU_USB_DEVICE(device);
 	FuUsbDevicePrivate *priv = GET_PRIVATE(self);
 	const gchar *tmp;
-	JsonObject *json_object;
-
-	/* sanity check */
-	if (!JSON_NODE_HOLDS_OBJECT(json_node)) {
-		g_set_error_literal(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_INVALID_DATA,
-				    "not JSON object");
-		return FALSE;
-	}
-	json_object = json_node_get_object(json_node);
 
 	/* optional properties */
 	tmp = json_object_get_string_member_with_default(json_object, "PlatformId", NULL);
 	if (tmp != NULL)
 		fu_device_set_physical_id(FU_DEVICE(self), tmp);
-#if GLIB_CHECK_VERSION(2, 80, 0)
-	tmp = json_object_get_string_member_with_default(json_object, "Created", NULL);
-	if (tmp != NULL) {
-		g_autoptr(GDateTime) created_new = g_date_time_new_from_iso8601(tmp, NULL);
-		if (created_new == NULL) {
-			g_set_error(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_INVALID_DATA,
-				    "cannot parse ISO8601 date: %s",
-				    tmp);
-			return FALSE;
-		}
-		fu_device_set_created_usec(FU_DEVICE(self), g_date_time_to_unix_usec(created_new));
-	}
-#endif
 	fu_device_set_vid(FU_DEVICE(self),
 			  json_object_get_int_member_with_default(json_object, "IdVendor", 0x0));
 	fu_device_set_pid(FU_DEVICE(self),
@@ -2871,11 +2873,11 @@ fu_usb_device_from_json(FwupdCodec *codec, JsonNode *json_node, GError **error)
 }
 
 static void
-fu_usb_device_add_json(FwupdCodec *codec, JsonBuilder *builder, FwupdCodecFlags flags)
+fu_usb_device_add_json(FuDevice *device, JsonBuilder *builder, FwupdCodecFlags flags)
 {
-	FuUsbDevice *self = FU_USB_DEVICE(codec);
+	FuUsbDevice *self = FU_USB_DEVICE(device);
 	FuUsbDevicePrivate *priv = GET_PRIVATE(self);
-	GPtrArray *events = fu_device_get_events(FU_DEVICE(self));
+	GPtrArray *events = fu_device_get_events(device);
 	g_autoptr(GPtrArray) interfaces = NULL;
 	g_autoptr(GError) error_bos = NULL;
 	g_autoptr(GError) error_hid = NULL;
@@ -2884,14 +2886,17 @@ fu_usb_device_add_json(FwupdCodec *codec, JsonBuilder *builder, FwupdCodecFlags
 	/* optional properties */
 	fwupd_codec_json_append(builder, "GType", "FuUsbDevice");
 	fwupd_codec_json_append(builder, "PlatformId", fu_device_get_physical_id(FU_DEVICE(self)));
-#if GLIB_CHECK_VERSION(2, 80, 0)
 	if (fu_device_get_created_usec(FU_DEVICE(self)) != 0) {
+#if GLIB_CHECK_VERSION(2, 80, 0)
 		g_autoptr(GDateTime) dt =
 		    g_date_time_new_from_unix_utc_usec(fu_device_get_created_usec(FU_DEVICE(self)));
+#else
+		g_autoptr(GDateTime) dt = g_date_time_new_from_unix_utc(
+		    fu_device_get_created_usec(FU_DEVICE(self)) / G_USEC_PER_SEC);
+#endif
 		g_autofree gchar *str = g_date_time_format_iso8601(dt);
 		fwupd_codec_json_append(builder, "Created", str);
 	}
-#endif
 	if (fu_device_get_vid(FU_DEVICE(self)) != 0) {
 		fwupd_codec_json_append_int(builder,
 					    "IdVendor",
@@ -2993,13 +2998,6 @@ fu_usb_device_add_json(FwupdCodec *codec, JsonBuilder *builder, FwupdCodecFlags
 	}
 }
 
-static void
-fu_usb_device_codec_iface_init(FwupdCodecInterface *iface)
-{
-	iface->add_json = fu_usb_device_add_json;
-	iface->from_json = fu_usb_device_from_json;
-}
-
 /**
  * fu_usb_device_new: (skip):
  * @ctx: (nullable): a #FuContext
@@ -3089,10 +3087,13 @@ fu_usb_device_class_init(FuUsbDeviceClass *klass)
 	device_class->ready = fu_usb_device_ready;
 	device_class->close = fu_usb_device_close;
 	device_class->probe = fu_usb_device_probe;
+	device_class->probe_complete = fu_usb_device_probe_complete;
 	device_class->invalidate = fu_usb_device_invalidate;
 	device_class->to_string = fu_usb_device_to_string;
 	device_class->incorporate = fu_usb_device_incorporate;
 	device_class->convert_version = fu_usb_device_convert_version;
+	device_class->from_json = fu_usb_device_from_json;
+	device_class->add_json = fu_usb_device_add_json;
 
 	/**
 	 * FuUsbDevice:libusb-device:
