@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuCommon"
@@ -13,12 +13,13 @@
 #include "fwupd-enums.h"
 #include "fwupd-error.h"
 
+#include "fu-string.h"
 #include "fu-version-common.h"
 
-#define FU_COMMON_VERSION_DECODE_BCD(val) ((((val) >> 4) & 0x0f) * 10 + ((val)&0x0f))
+#define FU_COMMON_VERSION_DECODE_BCD(val) ((((val) >> 4) & 0x0f) * 10 + ((val) & 0x0f))
 
 static gchar *
-fu_common_version_ensure_semver(const gchar *version);
+fu_version_ensure_semver_internal(const gchar *version);
 
 /**
  * fu_version_from_uint64:
@@ -294,6 +295,13 @@ fu_version_from_uint16(guint16 val, FwupdVersionFormat kind)
 	if (kind == FWUPD_VERSION_FORMAT_PAIR) {
 		return g_strdup_printf("%u.%u", (guint)(val >> 8) & 0xff, (guint)val & 0xff);
 	}
+	if (kind == FWUPD_VERSION_FORMAT_QUAD) {
+		return g_strdup_printf("%u.%u.%u.%u",
+				       (guint)(val >> 12) & 0xF,
+				       (guint)(val >> 8) & 0xF,
+				       (guint)(val >> 4) & 0xF,
+				       (guint)val & 0xF);
+	}
 	if (kind == FWUPD_VERSION_FORMAT_TRIPLET) {
 		return g_strdup_printf("%u.%u.%u",
 				       (guint)(val >> 12) & 0xF,
@@ -435,7 +443,7 @@ fu_version_ensure_semver(const gchar *version, FwupdVersionFormat fmt)
 	g_autoptr(GString) str = g_string_new(NULL);
 
 	/* split into all sections */
-	tmp = fu_common_version_ensure_semver(version);
+	tmp = fu_version_ensure_semver_internal(version);
 	if (tmp == NULL)
 		return NULL;
 	if (fmt == FWUPD_VERSION_FORMAT_UNKNOWN)
@@ -463,16 +471,8 @@ fu_version_ensure_semver(const gchar *version, FwupdVersionFormat fmt)
 	return g_string_free(g_steal_pointer(&str), FALSE);
 }
 
-/**
- * fu_common_version_ensure_semver:
- * @version: (nullable): a version number, e.g. ` V1.2.3 `
- *
- * Builds a semver from the possibly crazy version number.
- *
- * Returns: a version number, e.g. `1.2.3`, or %NULL if the version was not valid
- */
 static gchar *
-fu_common_version_ensure_semver(const gchar *version)
+fu_version_ensure_semver_internal(const gchar *version)
 {
 	gboolean dot_valid = FALSE;
 	guint digit_cnt = 0;
@@ -540,10 +540,7 @@ fu_common_version_ensure_semver(const gchar *version)
 gchar *
 fu_version_parse_from_format(const gchar *version, FwupdVersionFormat fmt)
 {
-	const gchar *version_noprefix = version;
-	gchar *endptr = NULL;
-	guint64 tmp;
-	guint base;
+	guint64 tmp = 0;
 
 	/* sanity check */
 	if (version == NULL)
@@ -558,19 +555,7 @@ fu_version_parse_from_format(const gchar *version, FwupdVersionFormat fmt)
 		return g_strdup(version);
 
 	/* convert 0x prefixed strings to dotted decimal */
-	if (g_str_has_prefix(version, "0x")) {
-		version_noprefix += 2;
-		base = 16;
-	} else {
-		/* for non-numeric content, just return the string */
-		if (!_g_ascii_is_digits(version))
-			return g_strdup(version);
-		base = 10;
-	}
-
-	/* convert */
-	tmp = g_ascii_strtoull(version_noprefix, &endptr, base);
-	if (endptr != NULL && endptr[0] != '\0')
+	if (!fu_strtoull(version, &tmp, 0, G_MAXUINT32, FU_INTEGER_BASE_AUTO, NULL))
 		return g_strdup(version);
 	if (tmp == 0)
 		return g_strdup(version);
@@ -660,6 +645,7 @@ fu_version_format_convert_base(FwupdVersionFormat fmt)
 gboolean
 fu_version_verify_format(const gchar *version, FwupdVersionFormat fmt, GError **error)
 {
+	FwupdVersionFormat fmt_base = fu_version_format_convert_base(fmt);
 	FwupdVersionFormat fmt_guess;
 
 	g_return_val_if_fail(version != NULL, FALSE);
@@ -675,13 +661,10 @@ fu_version_verify_format(const gchar *version, FwupdVersionFormat fmt, GError **
 
 	/* check the base format */
 	fmt_guess = fu_version_guess_format(version);
-	if (fmt == FWUPD_VERSION_FORMAT_BCD &&
-	    (fmt_guess == FWUPD_VERSION_FORMAT_PAIR || fmt_guess == FWUPD_VERSION_FORMAT_QUAD))
-		return TRUE;
-	if (fmt_guess != fu_version_format_convert_base(fmt)) {
+	if (fmt_guess != fmt_base) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "%s is not a valid %s (guessed %s)",
 			    version,
 			    fwupd_version_format_to_string(fmt),
@@ -723,8 +706,8 @@ fu_version_compare_safe(const gchar *version_a, const gchar *version_b)
 			return 1;
 
 		/* compare integers */
-		ver_a = g_ascii_strtoll(split_a[i], &endptr_a, 10);
-		ver_b = g_ascii_strtoll(split_b[i], &endptr_b, 10);
+		ver_a = g_ascii_strtoll(split_a[i], &endptr_a, 10); /* nocheck:blocked */
+		ver_b = g_ascii_strtoll(split_b[i], &endptr_b, 10); /* nocheck:blocked */
 		if (ver_a < ver_b)
 			return -1;
 		if (ver_a > ver_b)
