@@ -1,13 +1,12 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
 # pylint: disable=invalid-name,missing-docstring
 #
-# Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
+# Copyright 2023 Richard Hughes <richard@hughsie.com>
 #
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import os
 import sys
-import uuid
 import argparse
 
 from enum import Enum
@@ -46,7 +45,6 @@ class Export(Enum):
 
 # convert a CamelCase name into snake_case
 def _camel_to_snake(name: str) -> str:
-
     # specified as all caps
     if name.upper() == name:
         return name.lower()
@@ -94,6 +92,20 @@ class EnumObj:
                 return True
         return False
 
+    def check(self):
+        # check we're prefixed with something sane
+        if not self.name.startswith("Fu"):
+            raise ValueError(f"enum {self.name} does not have 'Fu' prefix")
+
+        # check we'd not just done ZERO=0, ONE=1, TWO=2, etc
+        indexed = True
+        for i, item in enumerate(self.items):
+            if str(i) != item.default:
+                indexed = False
+                break
+        if indexed:
+            raise ValueError(f"enum {self.name} does not need explicit defaults")
+
     def item(self, name: str) -> Optional["EnumItem"]:
         for item in self.items:
             if item.name == name:
@@ -128,7 +140,6 @@ class EnumItem:
         return f"{name_snake.upper()}_{_camel_to_snake(self.name).replace('-', '_').upper()}"
 
     def parse_default(self, val: str) -> None:
-
         val = {
             "u64::MAX": "G_MAXUINT64",
             "u32::MAX": "G_MAXUINT32",
@@ -156,8 +167,12 @@ class StructObj:
         self._exports: Dict[str, Export] = {
             "Validate": Export.NONE,
             "ValidateBytes": Export.NONE,
+            "ValidateStream": Export.NONE,
+            "ValidateInternal": Export.NONE,
             "Parse": Export.NONE,
             "ParseBytes": Export.NONE,
+            "ParseStream": Export.NONE,
+            "ParseInternal": Export.NONE,
             "New": Export.NONE,
             "ToString": Export.NONE,
             "Default": Export.NONE,
@@ -192,11 +207,22 @@ class StructObj:
                 return True
         return False
 
+    def check(self):
+        # check we're prefixed with something sane
+        if not self.name.startswith("Fu"):
+            raise ValueError(f"struct {self.name} does not have 'Fu' prefix")
+
     def add_private_export(self, derive: str) -> None:
         if self._exports[derive] == Export.PUBLIC:
             return
         self._exports[derive] = Export.PRIVATE
         if derive == "Validate":
+            self.add_private_export("ValidateInternal")
+        elif derive == "ValidateStream":
+            self.add_private_export("ValidateInternal")
+        elif derive == "ValidateBytes":
+            self.add_private_export("Validate")
+        elif derive == "ValidateInternal":
             for item in self.items:
                 if (
                     item.constant
@@ -205,19 +231,22 @@ class StructObj:
                 ):
                     item.add_private_export("Getters")
                 if item.struct_obj:
-                    item.struct_obj.add_private_export("Validate")
-        elif derive == "ValidateBytes":
-            self.add_private_export("Validate")
+                    item.struct_obj.add_private_export("ValidateInternal")
         elif derive == "ToString":
             for item in self.items:
                 if item.struct_obj:
                     item.struct_obj.add_private_export("ToString")
                 if item.enum_obj and not item.constant and item.enabled:
                     item.enum_obj.add_private_export("ToString")
+        elif derive == "Parse":
+            self.add_private_export("ParseInternal")
+        elif derive == "ParseStream":
+            self.add_private_export("ParseInternal")
         elif derive == "ParseBytes":
             self.add_private_export("Parse")
-        elif derive == "Parse":
+        elif derive == "ParseInternal":
             self.add_private_export("ToString")
+            self.add_private_export("ValidateInternal")
             for item in self.items:
                 if (
                     item.constant
@@ -226,7 +255,7 @@ class StructObj:
                 ):
                     item.add_private_export("Getters")
                 if item.struct_obj:
-                    item.struct_obj.add_private_export("Validate")
+                    item.struct_obj.add_private_export("ValidateInternal")
         elif derive == "New":
             for item in self.items:
                 if item.constant and not (item.type == Type.U8 and item.n_elements):
@@ -235,7 +264,6 @@ class StructObj:
                     item.struct_obj.add_private_export("New")
 
     def add_public_export(self, derive: str) -> None:
-
         # Getters and Setters are special as we do not want public exports of const
         if derive in ["Getters", "Setters"]:
             for item in self.items:
@@ -246,7 +274,7 @@ class StructObj:
             self._exports[derive] = Export.PUBLIC
 
         # for convenience
-        if derive in ["Parse", "ParseBytes"]:
+        if derive in ["Parse", "ParseBytes", "ParseStream"]:
             self.add_public_export("Getters")
             for item in self.items:
                 if item.struct_obj:
@@ -408,7 +436,6 @@ class StructItem:
         return ""
 
     def _parse_default(self, val: str) -> str:
-
         if self.enum_obj:
             enum_item = self.enum_obj.item(val)
             if not enum_item:
@@ -419,18 +446,7 @@ class StructItem:
             if val.startswith('"') and val.endswith('"'):
                 return val[1:-1]
             raise ValueError(f"string default {val} needs double quotes")
-        if self.type == Type.GUID:
-            if val.startswith("0x"):
-                guid = uuid.UUID(bytes_le=bytes.fromhex(val[2:]))
-                raise ValueError(f"integer {val} expected, expected: {guid}")
-            if not val.startswith('"'):
-                raise ValueError(f"string expected, got: {val}")
-            uuid2 = uuid.UUID(val[1:-1])
-            val_hex = ""
-            for value in uuid2.bytes_le:
-                val_hex += f"\\x{value:x}"
-            return val_hex
-        if self.type == Type.U8 and self.n_elements:
+        if self.type == Type.GUID or (self.type == Type.U8 and self.n_elements):
             if not val.startswith("0x"):
                 raise ValueError(f"0x prefix for hex number expected, got: {val}")
             if len(val) != (self.size * 2) + 2:
@@ -453,7 +469,6 @@ class StructItem:
         raise ValueError(f"do not know how to parse value for type: {self.type}")
 
     def parse_default(self, val: str) -> None:
-
         if (
             self.type == Type.U8
             and self.n_elements
@@ -465,14 +480,12 @@ class StructItem:
         self.default = self._parse_default(val)
 
     def parse_constant(self, val: str) -> None:
-
         self.default = self._parse_default(val)
         self.constant = self.default
 
     def parse_type(
         self, val: str, enum_objs: Dict[str, EnumObj], struct_objs: Dict[str, StructObj]
     ) -> None:
-
         # is array
         if val.startswith("[") and val.endswith("]"):
             typestr, n_elements = val[1:-1].split(";", maxsplit=1)
@@ -559,7 +572,6 @@ class Generator:
         )
 
     def _process_enums(self, enum_obj: EnumObj) -> Tuple[str, str]:
-
         # render
         subst = {
             "Type": Type,
@@ -571,7 +583,6 @@ class Generator:
         return template_c.render(subst), template_h.render(subst)
 
     def _process_structs(self, struct_obj: StructObj) -> Tuple[str, str]:
-
         # render
         subst = {
             "Type": Type,
@@ -593,7 +604,6 @@ class Generator:
         enum_cur: Optional[EnumObj] = None
 
         for line_num, line in enumerate(contents.split("\n")):
-
             # replace all tabs with spaces
             line = line.replace("\t", "  ")
 
@@ -639,6 +649,7 @@ class Generator:
             # end of structure
             if line.startswith("}"):
                 if struct_cur:
+                    struct_cur.check()
                     for derive in derives:
                         struct_cur.add_public_export(derive)
                     for item in struct_cur.items:
@@ -647,6 +658,7 @@ class Generator:
                         if item.constant == "$struct_size":
                             item.constant = str(offset)
                 if enum_cur:
+                    enum_cur.check()
                     for derive in derives:
                         enum_cur.add_public_export(derive)
                 struct_cur = None
@@ -676,7 +688,6 @@ class Generator:
 
             # split structure into sections
             if struct_cur:
-
                 # parse "signature: u32be == 0x12345678"
                 parts = line.replace(" ", "").split(":", maxsplit=2)
                 if len(parts) == 1:
@@ -699,6 +710,11 @@ class Generator:
                     )
                 except ValueError as e:
                     raise ValueError(f"{str(e)} on line {line_num}: {line}")
+                if len(type_parts) > 1:
+                    if "Default" not in derives:
+                        raise ValueError(
+                            f"struct requires #[derive(Default)] for line {line_num}: {line}"
+                        )
                 if len(type_parts) == 3:
                     item.parse_constant(type_parts[2])
                 elif len(type_parts) == 2:
@@ -713,6 +729,8 @@ class Generator:
         # process the templates here
         subst = {
             "basename": self.basename,
+            "enum_objs": self.enum_objs,
+            "struct_objs": self.struct_objs,
         }
         template_h = self._env.get_template(os.path.basename("fu-rustgen.h.in"))
         template_c = self._env.get_template(os.path.basename("fu-rustgen.c.in"))
@@ -732,7 +750,6 @@ class Generator:
 
 
 if __name__ == "__main__":
-
     parser = argparse.ArgumentParser()
     parser.add_argument("src", action="store", type=str, help="source")
     parser.add_argument("dst_c", action="store", type=str, help="destination .c")
