@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2022 Mario Limonciello <mario.limonciello@amd.com>
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2022 Mario Limonciello <mario.limonciello@amd.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -21,9 +21,9 @@ fu_acpi_ivrs_plugin_add_security_attrs(FuPlugin *plugin, FuSecurityAttrs *attrs)
 {
 	g_autofree gchar *fn = NULL;
 	g_autofree gchar *path = NULL;
-	g_autoptr(FuAcpiIvrs) ivrs = NULL;
+	g_autoptr(FuAcpiIvrs) ivrs = fu_acpi_ivrs_new();
 	g_autoptr(FwupdSecurityAttr) attr = NULL;
-	g_autoptr(GBytes) blob = NULL;
+	g_autoptr(GInputStream) stream = NULL;
 	g_autoptr(GError) error_local = NULL;
 
 	/* only AMD */
@@ -38,14 +38,17 @@ fu_acpi_ivrs_plugin_add_security_attrs(FuPlugin *plugin, FuSecurityAttrs *attrs)
 	/* load IVRS table */
 	path = fu_path_from_kind(FU_PATH_KIND_ACPI_TABLES);
 	fn = g_build_filename(path, "IVRS", NULL);
-	blob = fu_bytes_get_contents(fn, &error_local);
-	if (blob == NULL) {
+	stream = fu_input_stream_from_path(fn, &error_local);
+	if (stream == NULL) {
 		g_debug("failed to load %s: %s", fn, error_local->message);
 		fwupd_security_attr_set_result(attr, FWUPD_SECURITY_ATTR_RESULT_NOT_VALID);
 		return;
 	}
-	ivrs = fu_acpi_ivrs_new(blob, &error_local);
-	if (ivrs == NULL) {
+	if (!fu_firmware_parse_stream(FU_FIRMWARE(ivrs),
+				      stream,
+				      0x0,
+				      FWUPD_INSTALL_FLAG_NONE,
+				      &error_local)) {
 		g_warning("failed to parse %s: %s", fn, error_local->message);
 		fwupd_security_attr_set_result(attr, FWUPD_SECURITY_ATTR_RESULT_NOT_VALID);
 		return;
