@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2022 Mario Limonciello <mario.limonciello@amd.com>
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2022 Mario Limonciello <mario.limonciello@amd.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -19,33 +19,40 @@ G_DEFINE_TYPE(FuAcpiIvrs, fu_acpi_ivrs, FU_TYPE_ACPI_TABLE)
 /* IVINfo field */
 #define IVRS_DMA_REMAP_SUPPORT_FLAG 0x2
 
-FuAcpiIvrs *
-fu_acpi_ivrs_new(GBytes *blob, GError **error)
+static gboolean
+fu_acpi_ivrs_parse(FuFirmware *firmware,
+		   GInputStream *stream,
+		   FwupdInstallFlags flags,
+		   GError **error)
 {
-	FuAcpiIvrs *self = g_object_new(FU_TYPE_ACPI_IVRS, NULL);
-	gsize bufsz = 0;
+	FuAcpiIvrs *self = FU_ACPI_IVRS(firmware);
 	guint8 ivinfo = 0;
-	const guint8 *buf = g_bytes_get_data(blob, &bufsz);
 
 	/* FuAcpiTable->parse */
 	if (!FU_FIRMWARE_CLASS(fu_acpi_ivrs_parent_class)
-		 ->parse(FU_FIRMWARE(self), blob, 0x0, FWUPD_INSTALL_FLAG_NONE, error))
-		return NULL;
+		 ->parse(FU_FIRMWARE(self), stream, FWUPD_INSTALL_FLAG_NONE, error))
+		return FALSE;
 
 	/* check signature and read flags */
 	if (g_strcmp0(fu_firmware_get_id(FU_FIRMWARE(self)), "IVRS") != 0) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "Not a IVRS table, got %s",
 			    fu_firmware_get_id(FU_FIRMWARE(self)));
-		return NULL;
+		return FALSE;
 	}
-	if (!fu_memread_uint8_safe(buf, bufsz, 0x24, &ivinfo, error))
-		return NULL;
+	if (!fu_input_stream_read_u8(stream, 0x24, &ivinfo, error))
+		return FALSE;
 	g_debug("Flags: 0x%02x", ivinfo);
 	self->remap_support = ivinfo & IVRS_DMA_REMAP_SUPPORT_FLAG;
-	return self;
+	return TRUE;
+}
+
+FuAcpiIvrs *
+fu_acpi_ivrs_new(void)
+{
+	return g_object_new(FU_TYPE_ACPI_IVRS, NULL);
 }
 
 gboolean
@@ -58,6 +65,8 @@ fu_acpi_ivrs_get_dma_remap(FuAcpiIvrs *self)
 static void
 fu_acpi_ivrs_class_init(FuAcpiIvrsClass *klass)
 {
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->parse = fu_acpi_ivrs_parse;
 }
 
 static void
