@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2018 Evan Lojewski
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2018 Evan Lojewski
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -17,9 +17,6 @@
 #include <linux/sockios.h>
 #include <net/if.h>
 #endif
-#ifdef HAVE_IOCTL_H
-#include <sys/ioctl.h>
-#endif
 #ifdef HAVE_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -33,60 +30,68 @@
 #define FU_BCM57XX_BLOCK_SZ 0x4000 /* 16kb */
 
 struct _FuBcm57xxDevice {
-	FuUdevDevice parent_instance;
-	FuBcm57xxRecoveryDevice *recovery;
+	FuPciDevice parent_instance;
 	gchar *ethtool_iface;
-	int ethtool_fd;
 };
 
-G_DEFINE_TYPE(FuBcm57xxDevice, fu_bcm57xx_device, FU_TYPE_UDEV_DEVICE)
+G_DEFINE_TYPE(FuBcm57xxDevice, fu_bcm57xx_device, FU_TYPE_PCI_DEVICE)
+
+enum { PROP_0, PROP_IFACE, PROP_LAST };
 
 static void
 fu_bcm57xx_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(device);
-	FU_DEVICE_CLASS(fu_bcm57xx_device_parent_class)->to_string(device, idt, str);
-	fu_string_append(str, idt, "EthtoolIface", self->ethtool_iface);
+	fwupd_codec_string_append(str, idt, "EthtoolIface", self->ethtool_iface);
 }
 
 static gboolean
 fu_bcm57xx_device_probe(FuDevice *device, GError **error)
 {
-	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(device);
-	g_autofree gchar *fn = NULL;
-	g_autoptr(GPtrArray) ifaces = NULL;
+	return fu_udev_device_set_physical_id(FU_UDEV_DEVICE(device), "pci", error);
+}
 
-	/* only enumerate number 0 */
-	if (fu_udev_device_get_number(FU_UDEV_DEVICE(device)) != 0) {
-		g_set_error_literal(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_NOT_SUPPORTED,
-				    "only device 0 supported on multi-device card");
+#ifdef HAVE_ETHTOOL_H
+static gboolean
+fu_bcm57xx_device_ioctl_buffer_cb(FuIoctl *self,
+				  gpointer ptr,
+				  guint8 *buf,
+				  gsize bufsz,
+				  GError **error)
+{
+	struct ifreq *ifr = (struct ifreq *)ptr;
+	ifr->ifr_data = (char *)buf;
+	return TRUE;
+}
+
+static gboolean
+fu_bcm57xx_device_submit_ifreq(FuBcm57xxDevice *self, guint8 *buf, gsize bufsz, GError **error)
+{
+	struct ifreq ifr = {0};
+	g_autoptr(FuIoctl) ioctl = fu_udev_device_ioctl_new(FU_UDEV_DEVICE(self));
+
+	g_return_val_if_fail(buf != NULL, FALSE);
+
+	/* include these when generating the emulation event */
+	strncpy(ifr.ifr_name, self->ethtool_iface, IFNAMSIZ - 1);
+	fu_ioctl_add_key_as_u16(ioctl, "Request", SIOCETHTOOL);
+	fu_ioctl_add_mutable_buffer(ioctl, NULL, buf, bufsz, fu_bcm57xx_device_ioctl_buffer_cb);
+	if (!fu_ioctl_execute(ioctl,
+			      SIOCETHTOOL,
+			      (guint8 *)&ifr,
+			      sizeof(ifr),
+			      NULL,
+			      500, /* ms */
+			      FU_IOCTL_FLAG_NONE,
+			      error)) {
+		g_prefix_error(error, "failed to SIOCETHTOOL: ");
 		return FALSE;
 	}
 
-	/* we need this even for non-recovery to reset APE */
-	fu_device_set_context(FU_DEVICE(self->recovery), fu_device_get_context(FU_DEVICE(self)));
-	fu_device_incorporate(FU_DEVICE(self->recovery), FU_DEVICE(self));
-	if (!fu_device_probe(FU_DEVICE(self->recovery), error))
-		return FALSE;
-
-	/* only if has an interface */
-	fn = g_build_filename(fu_udev_device_get_sysfs_path(FU_UDEV_DEVICE(device)), "net", NULL);
-	if (!g_file_test(fn, G_FILE_TEST_EXISTS)) {
-		g_debug("waiting for net devices to appear");
-		fu_device_sleep(device, 50); /* ms */
-	}
-	ifaces = fu_path_glob(fn, "en*", NULL);
-	if (ifaces == NULL || ifaces->len == 0) {
-		fu_device_add_child(FU_DEVICE(self), FU_DEVICE(self->recovery));
-	} else {
-		self->ethtool_iface = g_path_get_basename(g_ptr_array_index(ifaces, 0));
-	}
-
 	/* success */
-	return fu_udev_device_set_physical_id(FU_UDEV_DEVICE(device), "pci", error);
+	return TRUE;
 }
+#endif
 
 static gboolean
 fu_bcm57xx_device_nvram_write(FuBcm57xxDevice *self,
@@ -97,8 +102,6 @@ fu_bcm57xx_device_nvram_write(FuBcm57xxDevice *self,
 {
 #ifdef HAVE_ETHTOOL_H
 	gsize eepromsz;
-	gint rc = -1;
-	struct ifreq ifr = {0};
 	g_autofree struct ethtool_eeprom *eeprom = NULL;
 
 	/* failed to load tg3 */
@@ -113,8 +116,8 @@ fu_bcm57xx_device_nvram_write(FuBcm57xxDevice *self,
 	/* sanity check */
 	if (address + bufsz > fu_device_get_firmware_size_max(FU_DEVICE(self))) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "tried to read outside of EEPROM size [0x%x]",
 			    (guint)fu_device_get_firmware_size_max(FU_DEVICE(self)));
 		return FALSE;
@@ -127,20 +130,12 @@ fu_bcm57xx_device_nvram_write(FuBcm57xxDevice *self,
 	eeprom->magic = BCM_NVRAM_MAGIC;
 	eeprom->len = bufsz;
 	eeprom->offset = address;
-	memcpy(eeprom->data, buf, eeprom->len);
-	strncpy(ifr.ifr_name, self->ethtool_iface, IFNAMSIZ - 1);
-	ifr.ifr_data = (char *)eeprom;
-#ifdef HAVE_IOCTL_H
-	rc = ioctl(self->ethtool_fd, SIOCETHTOOL, &ifr);
-#else
-	g_set_error_literal(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "Not supported as <sys/ioctl.h> not found");
-	return FALSE;
-#endif
-	if (rc < 0) {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_FAILED, "cannot write eeprom [%i]", rc);
+	memcpy(eeprom->data, buf, eeprom->len); /* nocheck:blocked */
+	if (!fu_bcm57xx_device_submit_ifreq(FU_BCM57XX_DEVICE(self),
+					    (guint8 *)eeprom,
+					    eepromsz,
+					    error)) {
+		g_prefix_error(error, "cannot write eeprom: ");
 		return FALSE;
 	}
 
@@ -164,8 +159,6 @@ fu_bcm57xx_device_nvram_read(FuBcm57xxDevice *self,
 {
 #ifdef HAVE_ETHTOOL_H
 	gsize eepromsz;
-	gint rc = -1;
-	struct ifreq ifr = {0};
 	g_autofree struct ethtool_eeprom *eeprom = NULL;
 
 	/* failed to load tg3 */
@@ -180,8 +173,8 @@ fu_bcm57xx_device_nvram_read(FuBcm57xxDevice *self,
 	/* sanity check */
 	if (address + bufsz > fu_device_get_firmware_size_max(FU_DEVICE(self))) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "tried to read outside of EEPROM size [0x%x]",
 			    (guint)fu_device_get_firmware_size_max(FU_DEVICE(self)));
 		return FALSE;
@@ -193,19 +186,11 @@ fu_bcm57xx_device_nvram_read(FuBcm57xxDevice *self,
 	eeprom->cmd = ETHTOOL_GEEPROM;
 	eeprom->len = bufsz;
 	eeprom->offset = address;
-	strncpy(ifr.ifr_name, self->ethtool_iface, IFNAMSIZ - 1);
-	ifr.ifr_data = (char *)eeprom;
-#ifdef HAVE_IOCTL_H
-	rc = ioctl(self->ethtool_fd, SIOCETHTOOL, &ifr);
-#else
-	g_set_error_literal(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "Not supported as <sys/ioctl.h> not found");
-	return FALSE;
-#endif
-	if (rc < 0) {
-		g_set_error(error, G_IO_ERROR, G_IO_ERROR_FAILED, "cannot read eeprom [%i]", rc);
+	if (!fu_bcm57xx_device_submit_ifreq(FU_BCM57XX_DEVICE(self),
+					    (guint8 *)eeprom,
+					    eepromsz,
+					    error)) {
+		g_prefix_error(error, "cannot read eeprom: ");
 		return FALSE;
 	}
 
@@ -235,9 +220,7 @@ static gboolean
 fu_bcm57xx_device_nvram_check(FuBcm57xxDevice *self, GError **error)
 {
 #ifdef HAVE_ETHTOOL_H
-	gint rc = -1;
-	struct ethtool_drvinfo drvinfo = {0};
-	struct ifreq ifr = {0};
+	struct ethtool_drvinfo drvinfo = {.cmd = ETHTOOL_GDRVINFO};
 
 	/* failed to load tg3 */
 	if (self->ethtool_iface == NULL) {
@@ -249,24 +232,11 @@ fu_bcm57xx_device_nvram_check(FuBcm57xxDevice *self, GError **error)
 	}
 
 	/* get driver info */
-	drvinfo.cmd = ETHTOOL_GDRVINFO;
-	strncpy(ifr.ifr_name, self->ethtool_iface, IFNAMSIZ - 1);
-	ifr.ifr_data = (char *)&drvinfo;
-#ifdef HAVE_IOCTL_H
-	rc = ioctl(self->ethtool_fd, SIOCETHTOOL, &ifr);
-#else
-	g_set_error_literal(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "Not supported as <sys/ioctl.h> not found");
-	return FALSE;
-#endif
-	if (rc < 0) {
-		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
-			    "cannot get driver information [%i]",
-			    rc);
+	if (!fu_bcm57xx_device_submit_ifreq(FU_BCM57XX_DEVICE(self),
+					    (guint8 *)&drvinfo,
+					    sizeof(drvinfo),
+					    error)) {
+		g_prefix_error(error, "cannot get driver information: ");
 		return FALSE;
 	}
 	g_debug("FW version %s", drvinfo.fw_version);
@@ -275,14 +245,14 @@ fu_bcm57xx_device_nvram_check(FuBcm57xxDevice *self, GError **error)
 	if (drvinfo.eedump_len == fu_device_get_firmware_size_max(FU_DEVICE(self)) * 2) {
 		g_autofree gchar *subsys =
 		    g_strdup_printf("%04X%04X",
-				    fu_udev_device_get_subsystem_vendor(FU_UDEV_DEVICE(self)),
-				    fu_udev_device_get_subsystem_model(FU_UDEV_DEVICE(self)));
+				    fu_pci_device_get_subsystem_vid(FU_PCI_DEVICE(self)),
+				    fu_pci_device_get_subsystem_pid(FU_PCI_DEVICE(self)));
 		g_debug("auto-sizing expected EEPROM size for OEM SUBSYS %s", subsys);
 		fu_device_set_firmware_size(FU_DEVICE(self), drvinfo.eedump_len);
 	} else if (drvinfo.eedump_len != fu_device_get_firmware_size_max(FU_DEVICE(self))) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "EEPROM size invalid, got 0x%x, expected 0x%x",
 			    drvinfo.eedump_len,
 			    (guint)fu_device_get_firmware_size_max(FU_DEVICE(self)));
@@ -300,40 +270,6 @@ fu_bcm57xx_device_nvram_check(FuBcm57xxDevice *self, GError **error)
 #endif
 }
 
-static gboolean
-fu_bcm57xx_device_activate(FuDevice *device, FuProgress *progress, GError **error)
-{
-	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(device);
-	g_autoptr(FuDeviceLocker) locker1 = NULL;
-	g_autoptr(FuDeviceLocker) locker2 = NULL;
-
-	/* the only way to do this is using the mmap method */
-	locker2 = fu_device_locker_new_full(FU_DEVICE(self->recovery),
-					    (FuDeviceLockerFunc)fu_device_detach,
-					    (FuDeviceLockerFunc)fu_device_attach,
-					    error);
-	if (locker2 == NULL)
-		return FALSE;
-
-	/* open */
-	locker1 = fu_device_locker_new(FU_DEVICE(self->recovery), error);
-	if (locker1 == NULL)
-		return FALSE;
-
-	/* activate, causing APE reset, then close, then attach */
-	if (!fu_device_activate(FU_DEVICE(self->recovery), progress, error))
-		return FALSE;
-
-	/* ensure we attach before we close */
-	if (!fu_device_locker_close(locker2, error))
-		return FALSE;
-
-	/* wait for the device to restart before calling reload() */
-	fu_progress_set_status(progress, FWUPD_STATUS_DEVICE_BUSY);
-	fu_device_sleep_full(device, 5000, progress); /* ms */
-	return TRUE;
-}
-
 static GBytes *
 fu_bcm57xx_device_dump_firmware(FuDevice *device, FuProgress *progress, GError **error)
 {
@@ -371,7 +307,7 @@ fu_bcm57xx_device_read_firmware(FuDevice *device, FuProgress *progress, GError *
 	fw = fu_bcm57xx_device_dump_firmware(device, progress, error);
 	if (fw == NULL)
 		return NULL;
-	if (!fu_firmware_parse(firmware, fw, FWUPD_INSTALL_FLAG_NO_SEARCH, error))
+	if (!fu_firmware_parse_bytes(firmware, fw, 0x0, FWUPD_INSTALL_FLAG_NO_SEARCH, error))
 		return NULL;
 
 	/* remove images that will contain user-data */
@@ -386,7 +322,8 @@ fu_bcm57xx_device_read_firmware(FuDevice *device, FuProgress *progress, GError *
 
 static FuFirmware *
 fu_bcm57xx_device_prepare_firmware(FuDevice *device,
-				   GBytes *fw,
+				   GInputStream *stream,
+				   FuProgress *progress,
 				   FwupdInstallFlags flags,
 				   GError **error)
 {
@@ -399,11 +336,10 @@ fu_bcm57xx_device_prepare_firmware(FuDevice *device,
 	g_autoptr(FuFirmware) img_ape = NULL;
 	g_autoptr(FuFirmware) img_stage1 = NULL;
 	g_autoptr(FuFirmware) img_stage2 = NULL;
-	g_autoptr(FuProgress) progress = fu_progress_new(G_STRLOC);
 	g_autoptr(GPtrArray) images = NULL;
 
 	/* try to parse NVRAM, stage1 or APE */
-	if (!fu_firmware_parse(firmware_tmp, fw, flags, error)) {
+	if (!fu_firmware_parse_stream(firmware_tmp, stream, 0x0, flags, error)) {
 		g_prefix_error(error, "failed to parse new firmware: ");
 		return NULL;
 	}
@@ -413,8 +349,7 @@ fu_bcm57xx_device_prepare_firmware(FuDevice *device,
 		guint16 vid = fu_bcm57xx_firmware_get_vendor(FU_BCM57XX_FIRMWARE(firmware_tmp));
 		guint16 did = fu_bcm57xx_firmware_get_model(FU_BCM57XX_FIRMWARE(firmware_tmp));
 		if (vid != 0x0 && did != 0x0 &&
-		    (fu_udev_device_get_vendor(FU_UDEV_DEVICE(device)) != vid ||
-		     fu_udev_device_get_model(FU_UDEV_DEVICE(device)) != did)) {
+		    (fu_device_get_vid(device) != vid || fu_device_get_pid(device) != did)) {
 			g_set_error(error,
 				    FWUPD_ERROR,
 				    FWUPD_ERROR_NOT_SUPPORTED,
@@ -422,17 +357,18 @@ fu_bcm57xx_device_prepare_firmware(FuDevice *device,
 				    "got: %04X:%04X expected %04X:%04X",
 				    vid,
 				    did,
-				    fu_udev_device_get_vendor(FU_UDEV_DEVICE(device)),
-				    fu_udev_device_get_model(FU_UDEV_DEVICE(device)));
+				    fu_device_get_vid(device),
+				    fu_device_get_pid(device));
 			return NULL;
 		}
 	}
 
 	/* get the existing firmware from the device */
+	fu_progress_set_status(progress, FWUPD_STATUS_DEVICE_READ);
 	fw_old = fu_bcm57xx_device_dump_firmware(device, progress, error);
 	if (fw_old == NULL)
 		return NULL;
-	if (!fu_firmware_parse(firmware, fw_old, FWUPD_INSTALL_FLAG_NO_SEARCH, error)) {
+	if (!fu_firmware_parse_bytes(firmware, fw_old, 0x0, FWUPD_INSTALL_FLAG_NO_SEARCH, error)) {
 		g_prefix_error(error, "failed to parse existing firmware: ");
 		return NULL;
 	}
@@ -475,7 +411,12 @@ fu_bcm57xx_device_write_chunks(FuBcm57xxDevice *self,
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_set_steps(progress, fu_chunk_array_length(chunks));
 	for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
-		g_autoptr(FuChunk) chk = fu_chunk_array_index(chunks, i);
+		g_autoptr(FuChunk) chk = NULL;
+
+		/* prepare chunk */
+		chk = fu_chunk_array_index(chunks, i, error);
+		if (chk == NULL)
+			return FALSE;
 		if (!fu_bcm57xx_device_nvram_write(self,
 						   fu_chunk_get_address(chk),
 						   fu_chunk_get_data(chk),
@@ -505,7 +446,6 @@ fu_bcm57xx_device_write_firmware(FuDevice *device,
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 1, "build-img");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 80, "write-chunks");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_VERIFY, 19, NULL);
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, NULL);
 
 	/* build the images into one linear blob of the correct size */
 	blob = fu_firmware_write(firmware, error);
@@ -514,7 +454,10 @@ fu_bcm57xx_device_write_firmware(FuDevice *device,
 	fu_progress_step_done(progress);
 
 	/* hit hardware */
-	chunks = fu_chunk_array_new_from_bytes(blob, 0x0, FU_BCM57XX_BLOCK_SZ);
+	chunks = fu_chunk_array_new_from_bytes(blob,
+					       FU_CHUNK_ADDR_OFFSET_NONE,
+					       FU_CHUNK_PAGESZ_NONE,
+					       FU_BCM57XX_BLOCK_SZ);
 	if (!fu_bcm57xx_device_write_chunks(self, chunks, fu_progress_get_child(progress), error))
 		return FALSE;
 	fu_progress_step_done(progress);
@@ -528,10 +471,23 @@ fu_bcm57xx_device_write_firmware(FuDevice *device,
 		return FALSE;
 	fu_progress_step_done(progress);
 
-	/* reset APE */
-	if (!fu_device_activate(device, fu_progress_get_child(progress), error))
+	/* success */
+	return TRUE;
+}
+
+static gboolean
+fu_bcm57xx_device_attach(FuDevice *device, FuProgress *progress, GError **error)
+{
+	g_autoptr(FwupdRequest) request = fwupd_request_new();
+
+	/* APE reset cannot be done at runtime */
+	fwupd_request_set_kind(request, FWUPD_REQUEST_KIND_POST);
+	fwupd_request_add_flag(request, FWUPD_REQUEST_FLAG_NON_GENERIC_MESSAGE);
+	fwupd_request_set_message(request,
+				  "After shutting down, disconnect the computer from all "
+				  "power sources for 30 seconds to complete the update.");
+	if (!fu_device_emit_request(device, request, progress, error))
 		return FALSE;
-	fu_progress_step_done(progress);
 
 	/* success */
 	return TRUE;
@@ -543,16 +499,6 @@ fu_bcm57xx_device_setup(FuDevice *device, GError **error)
 	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(device);
 	guint32 fwversion = 0;
 
-	/* device is in recovery mode */
-	if (self->ethtool_iface == NULL) {
-		g_autoptr(FuDeviceLocker) locker = NULL;
-		g_info("device in recovery mode, use alternate device");
-		locker = fu_device_locker_new(FU_DEVICE(self->recovery), error);
-		if (locker == NULL)
-			return FALSE;
-		return fu_device_setup(FU_DEVICE(self->recovery), error);
-	}
-
 	/* check the EEPROM size */
 	if (!fu_bcm57xx_device_nvram_check(self, error))
 		return FALSE;
@@ -567,7 +513,7 @@ fu_bcm57xx_device_setup(FuDevice *device, GError **error)
 	if (fwversion != 0x0) {
 		/* this is only set on the OSS firmware */
 		fu_device_set_version_format(device, FWUPD_VERSION_FORMAT_TRIPLET);
-		fu_device_set_version_u32(device, GUINT32_FROM_BE(fwversion));
+		fu_device_set_version_raw(device, GUINT32_FROM_BE(fwversion)); /* nocheck:blocked */
 		fu_device_set_branch(device, BCM_FW_BRANCH_OSS_FIRMWARE);
 	} else {
 		guint8 bufver[16] = {0x0};
@@ -581,7 +527,7 @@ fu_bcm57xx_device_setup(FuDevice *device, GError **error)
 						  sizeof(guint32),
 						  error))
 			return FALSE;
-		veraddr = GUINT32_FROM_BE(veraddr);
+		veraddr = GUINT32_FROM_BE(veraddr); /* nocheck:blocked */
 		if (veraddr > BCM_PHYS_ADDR_DEFAULT)
 			veraddr -= BCM_PHYS_ADDR_DEFAULT;
 		if (!fu_bcm57xx_device_nvram_read(self,
@@ -593,7 +539,7 @@ fu_bcm57xx_device_setup(FuDevice *device, GError **error)
 		veritem = fu_bcm57xx_veritem_new(bufver, sizeof(bufver));
 		if (veritem != NULL) {
 			fu_device_set_version_format(device, veritem->verfmt);
-			fu_device_set_version(device, veritem->version);
+			fu_device_set_version(device, veritem->version); /* nocheck:set-version */
 			fu_device_set_branch(device, veritem->branch);
 		}
 	}
@@ -611,11 +557,14 @@ fu_bcm57xx_device_open(FuDevice *device, GError **error)
 {
 #ifdef HAVE_SOCKET_H
 	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(device);
-	self->ethtool_fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (self->ethtool_fd < 0) {
+	gint fd;
+	g_autoptr(FuIOChannel) io_channel = NULL;
+
+	fd = socket(AF_INET, SOCK_DGRAM, 0);
+	if (fd < 0) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "failed to open socket: %s",
 #ifdef HAVE_ERRNO_H
 			    g_strerror(errno));
@@ -624,46 +573,79 @@ fu_bcm57xx_device_open(FuDevice *device, GError **error)
 #endif
 		return FALSE;
 	}
+	io_channel = fu_io_channel_unix_new(fd);
+	fu_udev_device_set_io_channel(FU_UDEV_DEVICE(self), io_channel);
 	return TRUE;
 #else
 	g_set_error_literal(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "socket() not supported as sys/socket.h not available");
 	return FALSE;
 #endif
 }
 
-static gboolean
-fu_bcm57xx_device_close(FuDevice *device, GError **error)
-{
-	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(device);
-	return g_close(self->ethtool_fd, error);
-}
-
 static void
 fu_bcm57xx_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 98, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "attach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 2, "reload");
 }
 
+static gchar *
+fu_bcm57xx_device_convert_version(FuDevice *device, guint64 version_raw)
+{
+	return fu_version_from_uint32(version_raw, fu_device_get_version_format(device));
+}
+
+static void
+fu_bcm57xx_device_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(object);
+	switch (prop_id) {
+	case PROP_IFACE:
+		g_value_set_string(value, self->ethtool_iface);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+fu_bcm57xx_device_set_property(GObject *object,
+			       guint prop_id,
+			       const GValue *value,
+			       GParamSpec *pspec)
+{
+	FuBcm57xxDevice *self = FU_BCM57XX_DEVICE(object);
+	switch (prop_id) {
+	case PROP_IFACE:
+		g_free(self->ethtool_iface);
+		self->ethtool_iface = g_value_dup_string(value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 fu_bcm57xx_device_init(FuBcm57xxDevice *self)
 {
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
+	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_NEEDS_SHUTDOWN);
+	fu_device_add_request_flag(FU_DEVICE(self), FWUPD_REQUEST_FLAG_NON_GENERIC_MESSAGE);
 	fu_device_add_protocol(FU_DEVICE(self), "com.broadcom.bcm57xx");
 	fu_device_add_icon(FU_DEVICE(self), "network-wired");
 
 	/* other values are set from a quirk */
 	fu_device_set_firmware_size(FU_DEVICE(self), BCM_FIRMWARE_SIZE);
-
-	/* used for recovery in case of ethtool failure and for APE reset */
-	self->recovery = fu_bcm57xx_recovery_device_new();
 }
 
 static void
@@ -678,18 +660,30 @@ static void
 fu_bcm57xx_device_class_init(FuBcm57xxDeviceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	GParamSpec *pspec;
+
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	object_class->get_property = fu_bcm57xx_device_get_property;
+	object_class->set_property = fu_bcm57xx_device_set_property;
 	object_class->finalize = fu_bcm57xx_device_finalize;
-	klass_device->prepare_firmware = fu_bcm57xx_device_prepare_firmware;
-	klass_device->setup = fu_bcm57xx_device_setup;
-	klass_device->reload = fu_bcm57xx_device_setup;
-	klass_device->open = fu_bcm57xx_device_open;
-	klass_device->close = fu_bcm57xx_device_close;
-	klass_device->activate = fu_bcm57xx_device_activate;
-	klass_device->write_firmware = fu_bcm57xx_device_write_firmware;
-	klass_device->read_firmware = fu_bcm57xx_device_read_firmware;
-	klass_device->dump_firmware = fu_bcm57xx_device_dump_firmware;
-	klass_device->probe = fu_bcm57xx_device_probe;
-	klass_device->to_string = fu_bcm57xx_device_to_string;
-	klass_device->set_progress = fu_bcm57xx_device_set_progress;
+	device_class->prepare_firmware = fu_bcm57xx_device_prepare_firmware;
+	device_class->setup = fu_bcm57xx_device_setup;
+	device_class->reload = fu_bcm57xx_device_setup;
+	device_class->open = fu_bcm57xx_device_open;
+	device_class->write_firmware = fu_bcm57xx_device_write_firmware;
+	device_class->attach = fu_bcm57xx_device_attach;
+	device_class->read_firmware = fu_bcm57xx_device_read_firmware;
+	device_class->dump_firmware = fu_bcm57xx_device_dump_firmware;
+	device_class->probe = fu_bcm57xx_device_probe;
+	device_class->to_string = fu_bcm57xx_device_to_string;
+	device_class->set_progress = fu_bcm57xx_device_set_progress;
+	device_class->convert_version = fu_bcm57xx_device_convert_version;
+
+	pspec =
+	    g_param_spec_string("iface",
+				NULL,
+				NULL,
+				NULL,
+				G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME);
+	g_object_class_install_property(object_class, PROP_IFACE, pspec);
 }
