@@ -1,13 +1,14 @@
 /*
- * Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
+ * Copyright 2023 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
 
 #include "fu-ch347-cfi-device.h"
 #include "fu-ch347-device.h"
+#include "fu-ch347-struct.h"
 
 struct _FuCh347Device {
 	FuUsbDevice parent_instance;
@@ -18,13 +19,6 @@ G_DEFINE_TYPE(FuCh347Device, fu_ch347_device, FU_TYPE_USB_DEVICE)
 
 #define FU_CH347_USB_TIMEOUT 1000
 
-#define FU_CH347_CMD_SPI_SET_CFG 0xC0
-#define FU_CH347_CMD_SPI_CS_CTRL 0xC1
-#define FU_CH347_CMD_SPI_OUT_IN	 0xC2
-#define FU_CH347_CMD_SPI_IN	 0xC3
-#define FU_CH347_CMD_SPI_OUT	 0xC4
-#define FU_CH347_CMD_SPI_GET_CFG 0xCA
-
 #define FU_CH347_CS_ASSERT   0x00
 #define FU_CH347_CS_DEASSERT 0x40
 #define FU_CH347_CS_CHANGE   0x80
@@ -43,50 +37,45 @@ static void
 fu_ch347_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuCh347Device *self = FU_CH347_DEVICE(device);
-
-	/* FuUsbDevice->to_string */
-	FU_DEVICE_CLASS(fu_ch347_device_parent_class)->to_string(device, idt, str);
-
-	fu_string_append_kx(str, idt, "Divisor", self->divisor);
+	fwupd_codec_string_append_hex(str, idt, "Divisor", self->divisor);
 }
 
 static gboolean
 fu_ch347_device_write(FuCh347Device *self,
-		      guint8 cmd,
+		      FuCh347CmdSpi cmd,
 		      const guint8 *buf,
 		      gsize bufsz,
 		      GError **error)
 {
-	GUsbDevice *usb_device = fu_usb_device_get_dev(FU_USB_DEVICE(self));
 	gsize actual_length = 0;
-	g_autoptr(GByteArray) cmdbuf = g_byte_array_new();
+	g_autoptr(FuStructCh347Req) st = fu_struct_ch347_req_new();
 
 	/* pack */
-	fu_byte_array_append_uint8(cmdbuf, cmd);
-	fu_byte_array_append_uint16(cmdbuf, bufsz, G_LITTLE_ENDIAN);
+	fu_struct_ch347_req_set_cmd(st, cmd);
+	fu_struct_ch347_req_set_payloadsz(st, bufsz);
 	if (bufsz > 0)
-		g_byte_array_append(cmdbuf, buf, bufsz);
+		g_byte_array_append(st, buf, bufsz);
 
 	/* debug */
-	fu_dump_raw(G_LOG_DOMAIN, "write", cmdbuf->data, cmdbuf->len);
-	if (!g_usb_device_bulk_transfer(usb_device,
-					FU_CH347_EP_OUT,
-					cmdbuf->data,
-					cmdbuf->len,
-					&actual_length,
-					FU_CH347_USB_TIMEOUT,
-					NULL,
-					error)) {
+	fu_dump_raw(G_LOG_DOMAIN, "write", st->data, st->len);
+	if (!fu_usb_device_bulk_transfer(FU_USB_DEVICE(self),
+					 FU_CH347_EP_OUT,
+					 st->data,
+					 st->len,
+					 &actual_length,
+					 FU_CH347_USB_TIMEOUT,
+					 NULL,
+					 error)) {
 		g_prefix_error(error, "failed to write 0x%x bytes: ", (guint)bufsz);
 		return FALSE;
 	}
-	if (cmdbuf->len != actual_length) {
+	if (st->len != actual_length) {
 		g_set_error(error,
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_INTERNAL,
 			    "only wrote 0x%x of 0x%x",
 			    (guint)actual_length,
-			    (guint)cmdbuf->len);
+			    (guint)st->len);
 		return FALSE;
 	}
 
@@ -95,53 +84,54 @@ fu_ch347_device_write(FuCh347Device *self,
 }
 
 static gboolean
-fu_ch347_device_read(FuCh347Device *self, guint8 cmd, guint8 *buf, gsize bufsz, GError **error)
+fu_ch347_device_read(FuCh347Device *self,
+		     FuCh347CmdSpi cmd,
+		     guint8 *buf,
+		     gsize bufsz,
+		     GError **error)
 {
-	GUsbDevice *usb_device = fu_usb_device_get_dev(FU_USB_DEVICE(self));
 	gsize actual_length = 0;
-	guint8 cmd_rsp;
 	guint16 size_rsp;
-	g_autoptr(GByteArray) cmdbuf = g_byte_array_new();
+	g_autoptr(GByteArray) st = fu_struct_ch347_req_new();
 
 	/* pack */
-	fu_byte_array_append_uint8(cmdbuf, cmd);
-	fu_byte_array_append_uint16(cmdbuf, sizeof(guint32), G_LITTLE_ENDIAN);
-	fu_byte_array_append_uint32(cmdbuf, bufsz, G_LITTLE_ENDIAN);
-	fu_byte_array_set_size(cmdbuf, FU_CH347_PACKET_SIZE, 0x0);
+	fu_struct_ch347_req_set_cmd(st, cmd);
+	fu_struct_ch347_req_set_payloadsz(st, sizeof(guint32));
+	fu_byte_array_append_uint32(st, bufsz, G_LITTLE_ENDIAN);
+	fu_byte_array_set_size(st, FU_CH347_PACKET_SIZE, 0x0);
 
-	if (!g_usb_device_bulk_transfer(usb_device,
-					FU_CH347_EP_IN,
-					cmdbuf->data,
-					cmdbuf->len,
-					&actual_length,
-					FU_CH347_USB_TIMEOUT,
-					NULL,
-					error)) {
+	if (!fu_usb_device_bulk_transfer(FU_USB_DEVICE(self),
+					 FU_CH347_EP_IN,
+					 st->data,
+					 st->len,
+					 &actual_length,
+					 FU_CH347_USB_TIMEOUT,
+					 NULL,
+					 error)) {
 		g_prefix_error(error, "failed to read 0x%x bytes: ", (guint)bufsz);
 		return FALSE;
 	}
-	fu_dump_raw(G_LOG_DOMAIN, "read", cmdbuf->data, actual_length);
+	fu_dump_raw(G_LOG_DOMAIN, "read", st->data, actual_length);
 	if (actual_length == 0) {
 		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "returned 0 bytes");
 		return FALSE;
 	}
 
 	/* debug */
-	cmd_rsp = cmdbuf->data[0];
-	if (cmd_rsp != cmd) {
+	if (fu_struct_ch347_req_get_cmd(st) != cmd) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "invalid cmd, got 0x%02x, expected 0x%02x",
-			    cmd_rsp,
+			    fu_struct_ch347_req_get_cmd(st),
 			    cmd);
 		return FALSE;
 	}
-	size_rsp = fu_memread_uint16(cmdbuf->data + 0x1, G_LITTLE_ENDIAN);
+	size_rsp = fu_struct_ch347_req_get_payloadsz(st);
 	if (size_rsp != bufsz) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "size invalid, got 0x%04x, expected 0x04%x",
 			    size_rsp,
 			    (guint)bufsz);
@@ -149,8 +139,14 @@ fu_ch347_device_read(FuCh347Device *self, guint8 cmd, guint8 *buf, gsize bufsz,
 	}
 
 	/* success */
-	memcpy(buf, cmdbuf->data + 0x3, size_rsp);
-	return TRUE;
+	return fu_memcpy_safe(buf,
+			      bufsz,
+			      0x0,
+			      st->data,
+			      st->len,
+			      FU_STRUCT_CH347_REQ_SIZE,
+			      size_rsp,
+			      error);
 }
 
 gboolean
@@ -166,10 +162,18 @@ fu_ch347_device_send_command(FuCh347Device *self,
 	if (wbufsz > 0) {
 		g_autoptr(GBytes) wblob = g_bytes_new_static(wbuf, wbufsz);
 		g_autoptr(FuChunkArray) chunks =
-		    fu_chunk_array_new_from_bytes(wblob, 0x0, FU_CH347_PAYLOAD_SIZE);
+		    fu_chunk_array_new_from_bytes(wblob,
+						  FU_CHUNK_ADDR_OFFSET_NONE,
+						  FU_CHUNK_PAGESZ_NONE,
+						  FU_CH347_PAYLOAD_SIZE);
 		for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
 			guint8 buf[1] = {0x0};
-			g_autoptr(FuChunk) chk = fu_chunk_array_index(chunks, i);
+			g_autoptr(FuChunk) chk = NULL;
+
+			/* prepare chunk */
+			chk = fu_chunk_array_index(chunks, i, error);
+			if (chk == NULL)
+				return FALSE;
 			if (!fu_ch347_device_write(self,
 						   FU_CH347_CMD_SPI_OUT,
 						   fu_chunk_get_data(chk),
@@ -253,20 +257,9 @@ fu_ch347_device_chip_select(FuCh347Device *self, gboolean val, GError **error)
 }
 
 static gboolean
-fu_ch347_device_setup(FuDevice *device, GError **error)
+fu_ch347_device_probe(FuDevice *device, GError **error)
 {
-	FuCh347Device *self = FU_CH347_DEVICE(device);
 	g_autoptr(FuCh347CfiDevice) cfi_device = NULL;
-
-	/* FuUsbDevice->setup */
-	if (!FU_DEVICE_CLASS(fu_ch347_device_parent_class)->setup(device, error))
-		return FALSE;
-
-	/* set divisor */
-	if (!fu_ch347_device_configure_stream(self, error))
-		return FALSE;
-
-	/* setup SPI chip */
 	cfi_device = g_object_new(FU_TYPE_CH347_CFI_DEVICE,
 				  "context",
 				  fu_device_get_context(device),
@@ -277,9 +270,22 @@ fu_ch347_device_setup(FuDevice *device, GError **error)
 				  "logical-id",
 				  "SPI",
 				  NULL);
-	if (!fu_device_setup(FU_DEVICE(cfi_device), error))
-		return FALSE;
 	fu_device_add_child(device, FU_DEVICE(cfi_device));
+	return TRUE;
+}
+
+static gboolean
+fu_ch347_device_setup(FuDevice *device, GError **error)
+{
+	FuCh347Device *self = FU_CH347_DEVICE(device);
+
+	/* FuUsbDevice->setup */
+	if (!FU_DEVICE_CLASS(fu_ch347_device_parent_class)->setup(device, error))
+		return FALSE;
+
+	/* set divisor */
+	if (!fu_ch347_device_configure_stream(self, error))
+		return FALSE;
 
 	/* success */
 	return TRUE;
@@ -297,7 +303,8 @@ fu_ch347_device_init(FuCh347Device *self)
 static void
 fu_ch347_device_class_init(FuCh347DeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
-	klass_device->setup = fu_ch347_device_setup;
-	klass_device->to_string = fu_ch347_device_to_string;
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	device_class->probe = fu_ch347_device_probe;
+	device_class->setup = fu_ch347_device_setup;
+	device_class->to_string = fu_ch347_device_to_string;
 }
