@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2020 H.J. Lu <hjl.tools@gmail.com>
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2020 H.J. Lu <hjl.tools@gmail.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -14,7 +14,7 @@
 
 #ifdef HAVE_SIGACTION
 static __attribute__((noreturn)) void
-segfault_sigaction(int signal, siginfo_t *si, void *arg)
+fu_cpu_helper_cet_segfault_sigaction(int signal, siginfo_t *si, void *arg)
 {
 	/* CET did exactly as it should to protect the system */
 	exit(0);
@@ -28,7 +28,7 @@ main(int argc, char *argv[])
 	struct sigaction sa = {0};
 
 	sigemptyset(&sa.sa_mask);
-	sa.sa_sigaction = segfault_sigaction;
+	sa.sa_sigaction = fu_cpu_helper_cet_segfault_sigaction;
 	sa.sa_flags = SA_SIGINFO;
 	sigaction(SIGSEGV, &sa, NULL);
 #endif
