@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2021 Daniel Campello <campello@chromium.org>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2021 Daniel Campello <campello@chromium.org>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -12,17 +12,8 @@
 #include "fu-flashrom-cmos.h"
 #include "fu-flashrom-device.h"
 
-/*
- * Flag to determine if the CMOS checksum should be reset after the flash
- * is reprogrammed.  This will force the CMOS defaults to be reloaded on
- * the next boot.
- */
-#define FU_FLASHROM_DEVICE_FLAG_RESET_CMOS (1 << 0)
-
-/*
- * Flag to determine if manual ME unlocking by pressing Fn + M is supported.
- */
-#define FU_FLASHROM_DEVICE_FLAG_FN_M_ME_UNLOCK (1 << 1)
+#define FU_FLASHROM_DEVICE_FLAG_RESET_CMOS     "reset-cmos"
+#define FU_FLASHROM_DEVICE_FLAG_FN_M_ME_UNLOCK "fn-m-me-unlock"
 
 struct _FuFlashromDevice {
 	FuUdevDevice parent_instance;
@@ -43,19 +34,19 @@ fu_flashrom_device_set_quirk_kv(FuDevice *device,
 {
 	if (g_strcmp0(key, "PciBcrAddr") == 0) {
 		guint64 tmp = 0;
-		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT32, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT32, FU_INTEGER_BASE_AUTO, error))
 			return FALSE;
 		fu_device_set_metadata_integer(device, "PciBcrAddr", tmp);
 		return TRUE;
 	}
-	g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "no supported");
+	g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no supported");
 	return FALSE;
 }
 
 static gboolean
 fu_flashrom_device_probe(FuDevice *device, GError **error)
 {
-	const gchar *dev_name = NULL;
+	g_autofree gchar *dev_name = NULL;
 	const gchar *sysfs_path = NULL;
 
 	sysfs_path = fu_udev_device_get_sysfs_path(FU_UDEV_DEVICE(device));
@@ -64,10 +55,12 @@ fu_flashrom_device_probe(FuDevice *device, GError **error)
 		physical_id = g_strdup_printf("DEVNAME=%s", sysfs_path);
 		fu_device_set_physical_id(device, physical_id);
 	}
-	dev_name = fu_udev_device_get_sysfs_attr(FU_UDEV_DEVICE(device), "name", NULL);
-	if (dev_name != NULL) {
+	dev_name = fu_udev_device_read_sysfs(FU_UDEV_DEVICE(device),
+					     "name",
+					     FU_UDEV_DEVICE_ATTR_READ_TIMEOUT_DEFAULT,
+					     NULL);
+	if (dev_name != NULL)
 		fu_device_add_instance_id_full(device, dev_name, FU_DEVICE_INSTANCE_FLAG_QUIRKS);
-	}
 	return TRUE;
 }
 
@@ -76,6 +69,12 @@ fu_flashrom_device_open(FuDevice *device, GError **error)
 {
 	FuFlashromDevice *self = FU_FLASHROM_DEVICE(device);
 
+	/* sanity check */
+	if (self->flashctx == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no flashctx");
+		return FALSE;
+	}
+
 	/* get the flash size from the device if not already been quirked */
 	if (fu_device_get_firmware_size_max(device) == 0) {
 		gsize flash_size = flashrom_flash_getsize(self->flashctx);
@@ -132,6 +131,28 @@ fu_flashrom_device_close(FuDevice *device, GError **error)
 	return TRUE;
 }
 
+#ifdef HAVE_FLASHROM_SET_PROGRESS_CALLBACK_V2
+static void
+fu_flashrom_device_progress_cb(enum flashrom_progress_stage stage,
+			       size_t current,
+			       size_t total,
+			       void *user_data)
+{
+	FuProgress *progress = FU_PROGRESS(user_data);
+
+	/* status */
+	if (stage == FLASHROM_PROGRESS_READ)
+		fu_progress_set_status(progress, FWUPD_STATUS_DEVICE_READ);
+	else if (stage == FLASHROM_PROGRESS_WRITE)
+		fu_progress_set_status(progress, FWUPD_STATUS_DEVICE_WRITE);
+	else if (stage == FLASHROM_PROGRESS_ERASE)
+		fu_progress_set_status(progress, FWUPD_STATUS_DEVICE_ERASE);
+
+	/* progress */
+	fu_progress_set_percentage_full(progress, current, total);
+}
+#endif
+
 static GBytes *
 fu_flashrom_device_dump_firmware(FuDevice *device, FuProgress *progress, GError **error)
 {
@@ -141,7 +162,13 @@ fu_flashrom_device_dump_firmware(FuDevice *device, FuProgress *progress, GError
 	g_autofree guint8 *buf = g_malloc0(bufsz);
 
 	fu_progress_set_status(progress, FWUPD_STATUS_DEVICE_READ);
+#ifdef HAVE_FLASHROM_SET_PROGRESS_CALLBACK_V2
+	flashrom_set_progress_callback_v2(self->flashctx, fu_flashrom_device_progress_cb, progress);
+#endif
 	rc = flashrom_image_read(self->flashctx, buf, bufsz);
+#ifdef HAVE_FLASHROM_SET_PROGRESS_CALLBACK_V2
+	flashrom_set_progress_callback_v2(self->flashctx, NULL, NULL);
+#endif
 	if (rc != 0) {
 		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_READ, "failed to read flash [%i]", rc);
 		return NULL;
@@ -155,6 +182,7 @@ fu_flashrom_device_prepare(FuDevice *device,
 			   FwupdInstallFlags flags,
 			   GError **error)
 {
+	gboolean exists_orig = FALSE;
 	g_autofree gchar *firmware_orig = NULL;
 	g_autofree gchar *localstatedir = NULL;
 	g_autofree gchar *basename = NULL;
@@ -165,7 +193,9 @@ fu_flashrom_device_prepare(FuDevice *device,
 	firmware_orig = g_build_filename(localstatedir, "builder", basename, NULL);
 	if (!fu_path_mkdir_parent(firmware_orig, error))
 		return FALSE;
-	if (!g_file_test(firmware_orig, G_FILE_TEST_EXISTS)) {
+	if (!fu_device_query_file_exists(device, firmware_orig, &exists_orig, error))
+		return FALSE;
+	if (!exists_orig) {
 		g_autoptr(GBytes) buf = NULL;
 		buf = fu_flashrom_device_dump_firmware(device, progress, error);
 		if (buf == NULL) {
@@ -214,7 +244,13 @@ fu_flashrom_device_write_firmware(FuDevice *device,
 			    (guint)fu_device_get_firmware_size_max(device));
 		return FALSE;
 	}
+#ifdef HAVE_FLASHROM_SET_PROGRESS_CALLBACK_V2
+	flashrom_set_progress_callback_v2(self->flashctx, fu_flashrom_device_progress_cb, progress);
+#endif
 	rc = flashrom_image_write(self->flashctx, (void *)buf, sz, NULL /* refbuffer */);
+#ifdef HAVE_FLASHROM_SET_PROGRESS_CALLBACK_V2
+	flashrom_set_progress_callback_v2(self->flashctx, NULL, NULL);
+#endif
 	if (rc != 0) {
 		g_set_error(error,
 			    FWUPD_ERROR,
@@ -249,6 +285,7 @@ fu_flashrom_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 100, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "attach");
@@ -264,25 +301,15 @@ fu_flashrom_device_init(FuFlashromDevice *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_CAN_VERIFY_IMAGE);
 	fu_device_add_protocol(FU_DEVICE(self), "org.flashrom");
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_ENSURE_SEMVER);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_SIGNED);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_FLAGS);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_ENSURE_SEMVER);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_SIGNED);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_MD_SET_FLAGS);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_HOST_FIRMWARE);
 	fu_device_set_physical_id(FU_DEVICE(self), "flashrom");
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_PAIR);
 	fu_device_add_icon(FU_DEVICE(self), "computer");
-	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_FLASHROM_DEVICE_FLAG_RESET_CMOS,
-					"reset-cmos");
-	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_FLASHROM_DEVICE_FLAG_FN_M_ME_UNLOCK,
-					"fn-m-me-unlock");
-}
-
-static void
-fu_flashrom_device_constructed(GObject *obj)
-{
-	FuFlashromDevice *self = FU_FLASHROM_DEVICE(obj);
-	fu_device_add_instance_id(FU_DEVICE(self), "main-system-firmware");
+	fu_device_register_private_flag(FU_DEVICE(self), FU_FLASHROM_DEVICE_FLAG_RESET_CMOS);
+	fu_device_register_private_flag(FU_DEVICE(self), FU_FLASHROM_DEVICE_FLAG_FN_M_ME_UNLOCK);
 }
 
 static void
@@ -338,7 +365,7 @@ fu_flashrom_device_class_init(FuFlashromDeviceClass *klass)
 {
 	GParamSpec *pspec;
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 
 	object_class->get_property = fu_flashrom_device_get_property;
 	object_class->set_property = fu_flashrom_device_set_property;
@@ -369,16 +396,15 @@ fu_flashrom_device_class_init(FuFlashromDeviceClass *klass)
 				 G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NAME);
 	g_object_class_install_property(object_class, PROP_FLASHCTX, pspec);
 
-	object_class->constructed = fu_flashrom_device_constructed;
 	object_class->finalize = fu_flashrom_device_finalize;
-	klass_device->set_quirk_kv = fu_flashrom_device_set_quirk_kv;
-	klass_device->probe = fu_flashrom_device_probe;
-	klass_device->open = fu_flashrom_device_open;
-	klass_device->close = fu_flashrom_device_close;
-	klass_device->set_progress = fu_flashrom_device_set_progress;
-	klass_device->prepare = fu_flashrom_device_prepare;
-	klass_device->dump_firmware = fu_flashrom_device_dump_firmware;
-	klass_device->write_firmware = fu_flashrom_device_write_firmware;
+	device_class->set_quirk_kv = fu_flashrom_device_set_quirk_kv;
+	device_class->probe = fu_flashrom_device_probe;
+	device_class->open = fu_flashrom_device_open;
+	device_class->close = fu_flashrom_device_close;
+	device_class->set_progress = fu_flashrom_device_set_progress;
+	device_class->prepare = fu_flashrom_device_prepare;
+	device_class->dump_firmware = fu_flashrom_device_dump_firmware;
+	device_class->write_firmware = fu_flashrom_device_write_firmware;
 }
 
 FuDevice *
