@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2022 Shihwei Huang <shihwei.huang@focaltech-electronics.com>
+ * Copyright 2022 Shihwei Huang <shihwei.huang@focaltech-electronics.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -36,24 +36,36 @@ fu_focalfp_firmware_export(FuFirmware *firmware, FuFirmwareExportFlags flags, Xb
 	fu_xmlb_builder_insert_kx(bn, "checksum", self->checksum);
 }
 
+static gboolean
+fu_focalfp_firmware_compute_checksum_cb(const guint8 *buf,
+					gsize bufsz,
+					gpointer user_data,
+					GError **error)
+{
+	guint32 *value = (guint32 *)user_data;
+	for (guint32 i = 0; i < bufsz; i += 4) {
+		guint32 tmp = 0;
+		if (!fu_memread_uint32_safe(buf, bufsz, i, &tmp, G_LITTLE_ENDIAN, error))
+			return FALSE;
+		*value ^= tmp;
+	}
+	return TRUE;
+}
+
 static gboolean
 fu_focalfp_firmware_parse(FuFirmware *firmware,
-			  GBytes *fw,
-			  gsize offset,
+			  GInputStream *stream,
 			  FwupdInstallFlags flags,
 			  GError **error)
 {
 	FuFocalfpFirmware *self = FU_FOCALFP_FIRMWARE(firmware);
-	gsize bufsz = 0;
-	const guint8 *buf = g_bytes_get_data(fw, &bufsz);
 
 	/* start address */
-	if (!fu_memread_uint16_safe(buf,
-				    bufsz,
-				    FOCAL_NAME_START_ADDR_WRDS,
-				    &self->start_address,
-				    G_BIG_ENDIAN,
-				    error)) {
+	if (!fu_input_stream_read_u16(stream,
+				      FOCAL_NAME_START_ADDR_WRDS,
+				      &self->start_address,
+				      G_BIG_ENDIAN,
+				      error)) {
 		return FALSE;
 	}
 	if (self->start_address != 0x582e) {
@@ -64,13 +76,13 @@ fu_focalfp_firmware_parse(FuFirmware *firmware,
 			    self->start_address);
 		return FALSE;
 	}
+
 	/* calculate checksum */
-	for (guint32 i = 0; i < bufsz; i += 4) {
-		guint32 value = 0;
-		if (!fu_memread_uint32_safe(buf, bufsz, i, &value, G_LITTLE_ENDIAN, error))
-			return FALSE;
-		self->checksum ^= value;
-	}
+	if (!fu_input_stream_chunkify(stream,
+				      fu_focalfp_firmware_compute_checksum_cb,
+				      &self->checksum,
+				      error))
+		return FALSE;
 	self->checksum += 1;
 
 	/* success */
@@ -85,7 +97,7 @@ fu_focalfp_firmware_init(FuFocalfpFirmware *self)
 static void
 fu_focalfp_firmware_class_init(FuFocalfpFirmwareClass *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
-	klass_firmware->parse = fu_focalfp_firmware_parse;
-	klass_firmware->export = fu_focalfp_firmware_export;
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->parse = fu_focalfp_firmware_parse;
+	firmware_class->export = fu_focalfp_firmware_export;
 }
