@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2016 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2020 boger wang <boger@goodix.com>
+ * Copyright 2016 Richard Hughes <richard@hughsie.com>
+ * Copyright 2020 boger wang <boger@goodix.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -26,14 +26,13 @@ G_DEFINE_TYPE(FuGoodixMocDevice, fu_goodixmoc_device, FU_TYPE_USB_DEVICE)
 #define GX_FLASH_TRANSFER_BLOCK_SIZE 1000 /* 1000 */
 
 static gboolean
-goodixmoc_device_cmd_send(FuGoodixMocDevice *self,
-			  guint8 cmd0,
-			  guint8 cmd1,
-			  GxPkgType type,
-			  GByteArray *req,
-			  GError **error)
+fu_goodixmoc_device_cmd_send(FuGoodixMocDevice *self,
+			     guint8 cmd0,
+			     guint8 cmd1,
+			     GxPkgType type,
+			     GByteArray *req,
+			     GError **error)
 {
-	GUsbDevice *usb_device = fu_usb_device_get_dev(FU_USB_DEVICE(self));
 	guint32 crc_all = 0;
 	guint32 crc_hdr = 0;
 	gsize actual_len = 0;
@@ -45,36 +44,36 @@ goodixmoc_device_cmd_send(FuGoodixMocDevice *self,
 	fu_byte_array_append_uint8(buf, type);		    /* pkg_flag */
 	fu_byte_array_append_uint8(buf, self->dummy_seq++); /* reserved */
 	fu_byte_array_append_uint16(buf, req->len + GX_SIZE_CRC32, G_LITTLE_ENDIAN);
-	crc_hdr = fu_crc8(buf->data, buf->len);
+	crc_hdr = ~fu_crc8(FU_CRC_KIND_B8_STANDARD, buf->data, buf->len);
 	fu_byte_array_append_uint8(buf, crc_hdr);
 	fu_byte_array_append_uint8(buf, ~crc_hdr);
 	g_byte_array_append(buf, req->data, req->len);
-	crc_all = fu_crc32(buf->data, buf->len);
+	crc_all = fu_crc32(FU_CRC_KIND_B32_STANDARD, buf->data, buf->len);
 	fu_byte_array_append_uint32(buf, crc_all, G_LITTLE_ENDIAN);
 
 	/* send zero length package */
-	if (!g_usb_device_bulk_transfer(usb_device,
-					GX_USB_BULK_EP_OUT,
-					NULL,
-					0,
-					NULL,
-					GX_USB_DATAOUT_TIMEOUT,
-					NULL,
-					error)) {
+	if (!fu_usb_device_bulk_transfer(FU_USB_DEVICE(self),
+					 GX_USB_BULK_EP_OUT,
+					 NULL,
+					 0,
+					 NULL,
+					 GX_USB_DATAOUT_TIMEOUT,
+					 NULL,
+					 error)) {
 		g_prefix_error(error, "failed to req: ");
 		return FALSE;
 	}
 	fu_dump_full(G_LOG_DOMAIN, "REQST", buf->data, buf->len, 16, FU_DUMP_FLAGS_SHOW_ADDRESSES);
 
 	/* send data */
-	if (!g_usb_device_bulk_transfer(usb_device,
-					GX_USB_BULK_EP_OUT,
-					buf->data,
-					buf->len,
-					&actual_len,
-					GX_USB_DATAOUT_TIMEOUT,
-					NULL,
-					error)) {
+	if (!fu_usb_device_bulk_transfer(FU_USB_DEVICE(self),
+					 GX_USB_BULK_EP_OUT,
+					 buf->data,
+					 buf->len,
+					 &actual_len,
+					 GX_USB_DATAOUT_TIMEOUT,
+					 NULL,
+					 error)) {
 		g_prefix_error(error, "failed to req: ");
 		return FALSE;
 	}
@@ -88,12 +87,11 @@ goodixmoc_device_cmd_send(FuGoodixMocDevice *self,
 }
 
 static gboolean
-goodixmoc_device_cmd_recv(FuGoodixMocDevice *self,
-			  GxfpCmdResp *presponse,
-			  gboolean data_reply,
-			  GError **error)
+fu_goodixmoc_device_cmd_recv(FuGoodixMocDevice *self,
+			     GxfpCmdResp *presponse,
+			     gboolean data_reply,
+			     GError **error)
 {
-	GUsbDevice *usb_device = fu_usb_device_get_dev(FU_USB_DEVICE(self));
 	guint32 crc_actual = 0;
 	guint32 crc_calculated = 0;
 	gsize actual_len = 0;
@@ -110,14 +108,14 @@ goodixmoc_device_cmd_recv(FuGoodixMocDevice *self,
 		guint8 header_cmd0 = 0x0;
 		g_autoptr(GByteArray) reply = g_byte_array_new();
 		fu_byte_array_set_size(reply, GX_FLASH_TRANSFER_BLOCK_SIZE, 0x00);
-		if (!g_usb_device_bulk_transfer(usb_device,
-						GX_USB_BULK_EP_IN,
-						reply->data,
-						reply->len,
-						&actual_len, /* allowed to return short read */
-						GX_USB_DATAIN_TIMEOUT,
-						NULL,
-						error)) {
+		if (!fu_usb_device_bulk_transfer(FU_USB_DEVICE(self),
+						 GX_USB_BULK_EP_IN,
+						 reply->data,
+						 reply->len,
+						 &actual_len, /* allowed to return short read */
+						 GX_USB_DATAIN_TIMEOUT,
+						 NULL,
+						 error)) {
 			g_prefix_error(error, "failed to reply: ");
 			return FALSE;
 		}
@@ -143,7 +141,7 @@ goodixmoc_device_cmd_recv(FuGoodixMocDevice *self,
 					    error))
 			return FALSE;
 		offset = sizeof(GxfpPkgHeader) + header_len - GX_SIZE_CRC32;
-		crc_actual = fu_crc32(reply->data, offset);
+		crc_actual = fu_crc32(FU_CRC_KIND_B32_STANDARD, reply->data, offset);
 		if (!fu_memread_uint32_safe(reply->data,
 					    reply->len,
 					    offset,
@@ -215,9 +213,9 @@ fu_goodixmoc_device_cmd_xfer(FuGoodixMocDevice *device,
 			     GError **error)
 {
 	FuGoodixMocDevice *self = FU_GOODIXMOC_DEVICE(device);
-	if (!goodixmoc_device_cmd_send(self, cmd0, cmd1, type, req, error))
+	if (!fu_goodixmoc_device_cmd_send(self, cmd0, cmd1, type, req, error))
 		return FALSE;
-	return goodixmoc_device_cmd_recv(self, presponse, data_reply, error);
+	return fu_goodixmoc_device_cmd_recv(self, presponse, data_reply, error);
 }
 
 static gboolean
@@ -353,7 +351,10 @@ fu_goodixmoc_device_write_firmware(FuDevice *device,
 		return FALSE;
 
 	/* build packets */
-	chunks = fu_chunk_array_new_from_bytes(fw, 0x00, GX_FLASH_TRANSFER_BLOCK_SIZE);
+	chunks = fu_chunk_array_new_from_bytes(fw,
+					       FU_CHUNK_ADDR_OFFSET_NONE,
+					       FU_CHUNK_PAGESZ_NONE,
+					       GX_FLASH_TRANSFER_BLOCK_SIZE);
 
 	/* don't auto-boot firmware */
 	if (!fu_goodixmoc_device_update_init(self, &error_local)) {
@@ -368,10 +369,14 @@ fu_goodixmoc_device_write_firmware(FuDevice *device,
 
 	/* write each block */
 	for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
-		g_autoptr(FuChunk) chk = fu_chunk_array_index(chunks, i);
+		g_autoptr(FuChunk) chk = NULL;
 		g_autoptr(GByteArray) req = g_byte_array_new();
 		g_autoptr(GError) error_block = NULL;
 
+		/* prepare chunk */
+		chk = fu_chunk_array_index(chunks, i, error);
+		if (chk == NULL)
+			return FALSE;
 		g_byte_array_append(req, fu_chunk_get_data(chk), fu_chunk_get_data_sz(chk));
 
 		/* the last chunk */
@@ -421,6 +426,7 @@ fu_goodixmoc_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 94, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, "attach");
@@ -432,7 +438,7 @@ fu_goodixmoc_device_init(FuGoodixMocDevice *self)
 {
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SELF_RECOVERY);
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_USE_RUNTIME_VERSION);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_USE_RUNTIME_VERSION);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_PLAIN);
 	fu_device_set_remove_delay(FU_DEVICE(self), 5000);
@@ -449,9 +455,9 @@ fu_goodixmoc_device_init(FuGoodixMocDevice *self)
 static void
 fu_goodixmoc_device_class_init(FuGoodixMocDeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
-	klass_device->write_firmware = fu_goodixmoc_device_write_firmware;
-	klass_device->setup = fu_goodixmoc_device_setup;
-	klass_device->attach = fu_goodixmoc_device_attach;
-	klass_device->set_progress = fu_goodixmoc_device_set_progress;
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	device_class->write_firmware = fu_goodixmoc_device_write_firmware;
+	device_class->setup = fu_goodixmoc_device_setup;
+	device_class->attach = fu_goodixmoc_device_attach;
+	device_class->set_progress = fu_goodixmoc_device_set_progress;
 }
