@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2022 Richard Hughes <richard@hughsie.com>
+ * Copyright 2022 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #pragma once
@@ -19,7 +19,7 @@ struct _FuIntelMeHeciDeviceClass {
 	FuMeiDeviceClass parent_class;
 };
 
-#define FU_INTEL_ME_HECI_DEVICE_FLAG_LEAKED_KM (1 << 0)
+#define FU_INTEL_ME_HECI_DEVICE_FLAG_LEAKED_KM "leaked-km"
 
 GByteArray *
 fu_intel_me_heci_device_read_file(FuIntelMeHeciDevice *self, const gchar *filename, GError **error);
