@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2023 GN Audio
+ * Copyright 2023 GN Audio
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -10,8 +10,9 @@
 #include "fu-jabra-gnp-firmware.h"
 #include "fu-jabra-gnp-image.h"
 
-#define FU_JABRA_GNP_BUF_SIZE			63
+#define FU_JABRA_GNP_BUF_SIZE			64
 #define FU_JABRA_GNP_MAX_RETRIES		3
+#define FU_JABRA_GNP_PRELOAD_COUNT		10
 #define FU_JABRA_GNP_RETRY_DELAY		100   /* ms */
 #define FU_JABRA_GNP_STANDARD_SEND_TIMEOUT	3000  /* ms */
 #define FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT	1000  /* ms */
@@ -20,20 +21,24 @@
 
 #define FU_JABRA_GNP_IFACE 0x05
 
+#define FU_JABRA_GNP_ADDRESS_PARENT    0x01
+#define FU_JABRA_GNP_ADDRESS_OTA_CHILD 0x04
+
 struct _FuJabraGnpDevice {
 	FuUsbDevice parent_instance;
 	guint8 iface_hid;
 	guint8 sequence_number;
+	guint8 address;
 	guint dfu_pid;
 };
 
 typedef struct {
-	guint8 *txbuf;
+	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE];
 	const guint timeout;
 } FuJabraGnpTxData;
 
 typedef struct {
-	guint8 *rxbuf;
+	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE];
 	const guint timeout;
 } FuJabraGnpRxData;
 
@@ -43,22 +48,23 @@ static void
 fu_jabra_gnp_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
-	fu_string_append_kx(str, idt, "IfaceHid", self->iface_hid);
-	fu_string_append_kx(str, idt, "SequenceNumber", self->sequence_number);
-	fu_string_append_kx(str, idt, "DfuPid", self->dfu_pid);
+	fwupd_codec_string_append_hex(str, idt, "IfaceHid", self->iface_hid);
+	fwupd_codec_string_append_hex(str, idt, "SequenceNumber", self->sequence_number);
+	fwupd_codec_string_append_hex(str, idt, "Address", self->address);
+	fwupd_codec_string_append_hex(str, idt, "DfuPid", self->dfu_pid);
 }
 
 static guint8
-_g_usb_device_get_interface_for_class(GUsbDevice *dev, guint8 intf_class, GError **error)
+_fu_usb_device_get_interface_for_class(FuUsbDevice *usb_device, guint8 intf_class, GError **error)
 {
 	g_autoptr(GPtrArray) intfs = NULL;
-	intfs = g_usb_device_get_interfaces(dev, error);
+	intfs = fu_usb_device_get_interfaces(usb_device, error);
 	if (intfs == NULL)
 		return 0xFF;
 	for (guint i = 0; i < intfs->len; i++) {
-		GUsbInterface *intf = g_ptr_array_index(intfs, i);
-		if (g_usb_interface_get_class(intf) == intf_class)
-			return g_usb_interface_get_number(intf);
+		FuUsbInterface *intf = g_ptr_array_index(intfs, i);
+		if (fu_usb_interface_get_class(intf) == intf_class)
+			return fu_usb_interface_get_number(intf);
 	}
 	return 0xFF;
 }
@@ -68,19 +74,22 @@ fu_jabra_gnp_device_tx_cb(FuDevice *device, gpointer user_data, GError **error)
 {
 	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
 	FuJabraGnpTxData *tx_data = (FuJabraGnpTxData *)user_data;
-	if (!g_usb_device_control_transfer(fu_usb_device_get_dev(FU_USB_DEVICE(self)),
-					   G_USB_DEVICE_DIRECTION_HOST_TO_DEVICE,
-					   G_USB_DEVICE_REQUEST_TYPE_CLASS,
-					   G_USB_DEVICE_RECIPIENT_INTERFACE,
-					   0x09,
-					   0x0200 | FU_JABRA_GNP_IFACE,
-					   self->iface_hid,
-					   tx_data->txbuf,
-					   FU_JABRA_GNP_BUF_SIZE,
-					   NULL,
-					   tx_data->timeout,
-					   NULL, /* cancellable */
-					   error)) {
+	FuUsbDevice *target = self->address == FU_JABRA_GNP_ADDRESS_OTA_CHILD
+				  ? FU_USB_DEVICE(fu_device_get_parent(device))
+				  : FU_USB_DEVICE(self);
+	if (!fu_usb_device_control_transfer(FU_USB_DEVICE(target),
+					    FU_USB_DIRECTION_HOST_TO_DEVICE,
+					    FU_USB_REQUEST_TYPE_CLASS,
+					    FU_USB_RECIPIENT_INTERFACE,
+					    0x09,
+					    0x0200 | FU_JABRA_GNP_IFACE,
+					    self->iface_hid,
+					    tx_data->txbuf,
+					    FU_JABRA_GNP_BUF_SIZE,
+					    NULL,
+					    tx_data->timeout,
+					    NULL, /* cancellable */
+					    error)) {
 		g_prefix_error(error, "failed to write to device: ");
 		return FALSE;
 	}
@@ -90,38 +99,53 @@ fu_jabra_gnp_device_tx_cb(FuDevice *device, gpointer user_data, GError **error)
 static gboolean
 fu_jabra_gnp_device_rx_cb(FuDevice *device, gpointer user_data, GError **error)
 {
-	const guint8 match_buf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, 0x00, 0x0A, 0x12, 0x02};
 	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
+	const guint8 match_buf[FU_JABRA_GNP_BUF_SIZE] =
+	    {FU_JABRA_GNP_IFACE, 0x00, self->address, 0x00, 0x0A, 0x12, 0x02};
+	const guint8 empty_buf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
 	FuJabraGnpRxData *rx_data = (FuJabraGnpRxData *)user_data;
-	if (!g_usb_device_interrupt_transfer(fu_usb_device_get_dev(FU_USB_DEVICE(self)),
-					     0x81,
-					     rx_data->rxbuf,
-					     FU_JABRA_GNP_BUF_SIZE,
-					     NULL,
-					     rx_data->timeout,
-					     NULL, /* cancellable */
-					     error)) {
+	FuUsbDevice *target = self->address == FU_JABRA_GNP_ADDRESS_OTA_CHILD
+				  ? FU_USB_DEVICE(fu_device_get_parent(device))
+				  : FU_USB_DEVICE(self);
+
+	if (!fu_usb_device_interrupt_transfer(target,
+					      0x81,
+					      rx_data->rxbuf,
+					      FU_JABRA_GNP_BUF_SIZE,
+					      NULL,
+					      rx_data->timeout,
+					      NULL, /* cancellable */
+					      error)) {
 		g_prefix_error(error, "failed to read from device: ");
 		return FALSE;
-	} else {
-		if (rx_data->rxbuf[5] == match_buf[5] && rx_data->rxbuf[6] == match_buf[6]) {
-			/* battery report, ignore and rx again */
-			if (!g_usb_device_interrupt_transfer(
-				fu_usb_device_get_dev(FU_USB_DEVICE(self)),
-				0x81,
-				rx_data->rxbuf,
-				FU_JABRA_GNP_BUF_SIZE,
-				NULL,
-				rx_data->timeout,
-				NULL, /* cancellable */
-				error)) {
-				g_prefix_error(error, "failed to read from device: ");
-				return FALSE;
-			}
+	}
+	if (rx_data->rxbuf[2] == match_buf[2] && rx_data->rxbuf[5] == match_buf[5] &&
+	    rx_data->rxbuf[6] == match_buf[6]) {
+		/* battery report, ignore and rx again */
+		if (!fu_usb_device_interrupt_transfer(target,
+						      0x81,
+						      rx_data->rxbuf,
+						      FU_JABRA_GNP_BUF_SIZE,
+						      NULL,
+						      rx_data->timeout,
+						      NULL, /* cancellable */
+						      error)) {
+			g_prefix_error(error, "failed to read from device: ");
+			return FALSE;
 		}
 	}
 
+	if (fu_memcmp_safe(rx_data->rxbuf,
+			   sizeof(rx_data->rxbuf),
+			   0,
+			   empty_buf,
+			   sizeof(rx_data->rxbuf),
+			   0,
+			   sizeof(rx_data->rxbuf),
+			   error)) {
+		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "error readuing from device");
+		return FALSE;
+	}
 	return TRUE;
 }
 
@@ -131,7 +155,12 @@ fu_jabra_gnp_device_rx_with_sequence_cb(FuDevice *device, gpointer user_data, GE
 	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
 	FuJabraGnpRxData *rx_data = (FuJabraGnpRxData *)user_data;
 
-	if (!fu_jabra_gnp_device_rx_cb(device, user_data, error))
+	if (!fu_device_retry_full(FU_DEVICE(self),
+				  fu_jabra_gnp_device_rx_cb,
+				  FU_JABRA_GNP_MAX_RETRIES,
+				  FU_JABRA_GNP_RETRY_DELAY,
+				  rx_data,
+				  error))
 		return FALSE;
 	if (self->sequence_number != rx_data->rxbuf[3]) {
 		g_set_error(error,
@@ -149,12 +178,23 @@ fu_jabra_gnp_device_rx_with_sequence_cb(FuDevice *device, gpointer user_data, GE
 static gboolean
 fu_jabra_gnp_device_read_name(FuJabraGnpDevice *self, GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, self->sequence_number, 0x46, 0x02, 0x00};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x46,
+		    0x02,
+		    0x00,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 	g_autofree gchar *name = NULL;
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
@@ -170,22 +210,135 @@ fu_jabra_gnp_device_read_name(FuJabraGnpDevice *self, GError **error)
 				  (gpointer)&rx_data,
 				  error))
 		return FALSE;
-	name = fu_memstrsafe(rxbuf, sizeof(rxbuf), 0x8, sizeof(rxbuf) - 8, error);
+	name = fu_memstrsafe(rx_data.rxbuf,
+			     sizeof(rx_data.rxbuf),
+			     0x8,
+			     sizeof(rx_data.rxbuf) - 8,
+			     error);
 	if (name == NULL)
 		return FALSE;
 	fu_device_set_name(FU_DEVICE(self), name);
 	return TRUE;
 }
 
+static gboolean
+fu_jabra_gnp_device_read_child_dfu_pid(FuJabraGnpDevice *self,
+				       guint16 *child_dfu_pid,
+				       GError **error)
+{
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    FU_JABRA_GNP_ADDRESS_OTA_CHILD,
+		    0x00,
+		    self->sequence_number,
+		    0x46,
+		    0x02,
+		    0x13,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
+
+	g_return_val_if_fail(child_dfu_pid != NULL, FALSE);
+
+	if (!fu_device_retry_full(FU_DEVICE(self),
+				  fu_jabra_gnp_device_tx_cb,
+				  FU_JABRA_GNP_MAX_RETRIES,
+				  FU_JABRA_GNP_RETRY_DELAY,
+				  (gpointer)&tx_data,
+				  error))
+		return FALSE;
+	if (!fu_device_retry_full(FU_DEVICE(self),
+				  fu_jabra_gnp_device_rx_with_sequence_cb,
+				  FU_JABRA_GNP_MAX_RETRIES,
+				  FU_JABRA_GNP_RETRY_DELAY,
+				  (gpointer)&rx_data,
+				  error))
+		return FALSE;
+	/* no child device to respond properly */
+	if (rx_data.rxbuf[5] == 0xFE && rx_data.rxbuf[6] == 0xF4) {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INTERNAL,
+			    "internal error: no child device responded");
+		return FALSE;
+	}
+	/* success */
+	*child_dfu_pid = fu_memread_uint16(rx_data.rxbuf + 7, G_LITTLE_ENDIAN);
+	return TRUE;
+}
+
+static gboolean
+fu_jabra_gnp_device_read_child_battery_level(FuJabraGnpDevice *self, GError **error)
+{
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x46,
+		    0x12,
+		    0x02,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
+	guint8 battery_level = 0;
+	if (!fu_device_retry_full(FU_DEVICE(self),
+				  fu_jabra_gnp_device_tx_cb,
+				  FU_JABRA_GNP_MAX_RETRIES,
+				  FU_JABRA_GNP_RETRY_DELAY,
+				  (gpointer)&tx_data,
+				  error))
+		return FALSE;
+	if (!fu_device_retry_full(FU_DEVICE(self),
+				  fu_jabra_gnp_device_rx_with_sequence_cb,
+				  FU_JABRA_GNP_MAX_RETRIES,
+				  FU_JABRA_GNP_RETRY_DELAY,
+				  (gpointer)&rx_data,
+				  error))
+		return FALSE;
+	if (!fu_memread_uint8_safe(rx_data.rxbuf, FU_JABRA_GNP_BUF_SIZE, 8, &battery_level, error))
+		return FALSE;
+	if (battery_level == 0x00) {
+		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "battery level was 0");
+		return FALSE;
+	}
+	fu_device_set_battery_level(FU_DEVICE(self), battery_level);
+	fu_device_set_battery_threshold(FU_DEVICE(self), 30);
+	return TRUE;
+}
+
 static gboolean
 fu_jabra_gnp_device_read_dfu_pid(FuJabraGnpDevice *self, GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, self->sequence_number, 0x46, 0x02, 0x13};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x46,
+		    0x02,
+		    0x13,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
@@ -208,14 +361,24 @@ fu_jabra_gnp_device_read_dfu_pid(FuJabraGnpDevice *self, GError **error)
 static gboolean
 fu_jabra_gnp_device_read_version(FuJabraGnpDevice *self, GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, self->sequence_number, 0x46, 0x02, 0x03};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x46,
+		    0x02,
+		    0x03,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 	g_autofree gchar *version = NULL;
-
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
 				  FU_JABRA_GNP_MAX_RETRIES,
@@ -230,7 +393,11 @@ fu_jabra_gnp_device_read_version(FuJabraGnpDevice *self, GError **error)
 				  (gpointer)&rx_data,
 				  error))
 		return FALSE;
-	version = fu_memstrsafe(rxbuf, sizeof(rxbuf), 0x8, sizeof(rxbuf) - 8, error);
+	version = fu_memstrsafe(rx_data.rxbuf,
+				sizeof(rx_data.rxbuf),
+				0x8,
+				sizeof(rx_data.rxbuf) - 8,
+				error);
 	if (version == NULL)
 		return FALSE;
 	fu_device_set_version(FU_DEVICE(self), version);
@@ -240,12 +407,24 @@ fu_jabra_gnp_device_read_version(FuJabraGnpDevice *self, GError **error)
 static gboolean
 fu_jabra_gnp_device_write_partition(FuJabraGnpDevice *self, guint8 part, GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, self->sequence_number, 0x87, 0x0F, 0x2D, part};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x87,
+		    0x0F,
+		    0x2D,
+		    part,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
 				  FU_JABRA_GNP_MAX_RETRIES,
@@ -275,12 +454,23 @@ fu_jabra_gnp_device_write_partition(FuJabraGnpDevice *self, guint8 part, GError
 static gboolean
 fu_jabra_gnp_device_start(FuJabraGnpDevice *self, GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, self->sequence_number, 0x86, 0x0F, 0x17};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x86,
+		    0x0F,
+		    0x17,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
 				  FU_JABRA_GNP_MAX_RETRIES,
@@ -310,11 +500,12 @@ fu_jabra_gnp_device_start(FuJabraGnpDevice *self, GError **error)
 static gboolean
 fu_jabra_gnp_device_flash_erase_done(FuJabraGnpDevice *self, GError **error)
 {
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
 	const guint8 match_buf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, 0x00, 0x06, 0x0F, 0x18};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_EXTRA_LONG_RECEIVE_TIMEOUT};
+	    {FU_JABRA_GNP_IFACE, 0x00, self->address, 0x00, 0x06, 0x0F, 0x18};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_EXTRA_LONG_RECEIVE_TIMEOUT,
+	};
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_rx_cb,
 				  FU_JABRA_GNP_MAX_RETRIES,
@@ -339,16 +530,27 @@ fu_jabra_gnp_device_write_crc(FuJabraGnpDevice *self,
 			      guint preload_count,
 			      GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, self->sequence_number, 0x8E, 0x0F, 0x19};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x8E,
+		    0x0F,
+		    0x19,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 
-	fu_memwrite_uint32(txbuf + 7, crc, G_LITTLE_ENDIAN);
-	fu_memwrite_uint16(txbuf + 11, total_chunks, G_LITTLE_ENDIAN);
-	fu_memwrite_uint16(txbuf + 13, preload_count, G_LITTLE_ENDIAN);
+	fu_memwrite_uint32(tx_data.txbuf + 7, crc, G_LITTLE_ENDIAN);
+	fu_memwrite_uint16(tx_data.txbuf + 11, total_chunks, G_LITTLE_ENDIAN);
+	fu_memwrite_uint16(tx_data.txbuf + 13, preload_count, G_LITTLE_ENDIAN);
 
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
@@ -380,16 +582,33 @@ static gboolean
 fu_jabra_gnp_device_write_chunk(FuJabraGnpDevice *self,
 				guint32 chunk_number,
 				const guint8 *buf,
-				guint32 bufsz,
+				gsize bufsz,
 				GError **error)
 {
 	guint8 write_length = 0x00 + bufsz + 10;
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, 0x00, write_length, 0x0F, 0x1A};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	fu_memwrite_uint16(txbuf + 7, chunk_number, G_LITTLE_ENDIAN);
-	fu_memwrite_uint16(txbuf + 9, bufsz, G_LITTLE_ENDIAN);
-	if (!fu_memcpy_safe(txbuf, sizeof(txbuf), 11, buf, bufsz, 0x0, bufsz, error))
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    0x00,
+		    write_length,
+		    0x0F,
+		    0x1A,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	fu_memwrite_uint16(tx_data.txbuf + 7, chunk_number, G_LITTLE_ENDIAN);
+	fu_memwrite_uint16(tx_data.txbuf + 9, bufsz, G_LITTLE_ENDIAN);
+	if (!fu_memcpy_safe(tx_data.txbuf,
+			    sizeof(tx_data.txbuf),
+			    11,
+			    buf,
+			    bufsz,
+			    0x0,
+			    bufsz,
+			    error))
 		return FALSE;
 	return fu_device_retry_full(FU_DEVICE(self),
 				    fu_jabra_gnp_device_tx_cb,
@@ -406,25 +625,39 @@ fu_jabra_gnp_device_write_chunks(FuJabraGnpDevice *self,
 				 GError **error)
 {
 	gboolean failed_chunk = FALSE;
-	guint32 preload_count = 100;
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	const guint8 match_buf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, 0x00, 0x06, 0x0F, 0x1B};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf, .timeout = FU_JABRA_GNP_LONG_RECEIVE_TIMEOUT};
+
+	const guint8 match_buf[FU_JABRA_GNP_BUF_SIZE] = {
+	    FU_JABRA_GNP_IFACE,
+	    0x00,
+	    self->address,
+	    0x00,
+	    0x06,
+	    0x0F,
+	    0x1B,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_LONG_RECEIVE_TIMEOUT,
+	};
 
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_set_steps(progress, fu_chunk_array_length(chunks));
 	for (gint chunk_number = 0; (guint)chunk_number < fu_chunk_array_length(chunks);
 	     chunk_number++) {
-		g_autoptr(FuChunk) chk = fu_chunk_array_index(chunks, chunk_number);
+		g_autoptr(FuChunk) chk = NULL;
+
+		/* prepare chunk */
+		chk = fu_chunk_array_index(chunks, chunk_number, error);
+		if (chk == NULL)
+			return FALSE;
 		if (!fu_jabra_gnp_device_write_chunk(self,
 						     chunk_number,
 						     fu_chunk_get_data(chk),
 						     fu_chunk_get_data_sz(chk),
 						     error))
 			return FALSE;
-		if (((chunk_number % preload_count) == 0) ||
+		if (((chunk_number % FU_JABRA_GNP_PRELOAD_COUNT) == 0) ||
 		    (guint)chunk_number == fu_chunk_array_length(chunks) - 1) {
 			if (!fu_device_retry_full(FU_DEVICE(self),
 						  fu_jabra_gnp_device_rx_cb,
@@ -440,7 +673,7 @@ fu_jabra_gnp_device_write_chunks(FuJabraGnpDevice *self,
 					    "internal error, buf did not match");
 				return FALSE;
 			}
-			if (fu_memread_uint16(rxbuf + 7, G_LITTLE_ENDIAN) != chunk_number) {
+			if (fu_memread_uint16(rx_data.rxbuf + 7, G_LITTLE_ENDIAN) != chunk_number) {
 				chunk_number--;
 				failed_chunk = TRUE;
 			} else
@@ -457,10 +690,12 @@ fu_jabra_gnp_device_write_chunks(FuJabraGnpDevice *self,
 static gboolean
 fu_jabra_gnp_device_read_verify_status(FuJabraGnpDevice *self, GError **error)
 {
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
 	const guint8 match_buf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, 0x00, 0x06, 0x0F, 0x1C};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf, .timeout = FU_JABRA_GNP_LONG_RECEIVE_TIMEOUT};
+	    {FU_JABRA_GNP_IFACE, 0x00, self->address, 0x00, 0x06, 0x0F, 0x1C};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_LONG_RECEIVE_TIMEOUT,
+	};
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_rx_cb,
 				  FU_JABRA_GNP_MAX_RETRIES,
@@ -483,20 +718,26 @@ fu_jabra_gnp_device_write_version(FuJabraGnpDevice *self,
 				  FuJabraGnpVersionData *version_data,
 				  GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] = {FU_JABRA_GNP_IFACE,
-					       0x08,
-					       0x00,
-					       self->sequence_number,
-					       0x89,
-					       0x0F,
-					       0x1E,
-					       version_data->major,
-					       version_data->minor,
-					       version_data->micro};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x89,
+		    0x0F,
+		    0x1E,
+		    version_data->major,
+		    version_data->minor,
+		    version_data->micro,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
 				  FU_JABRA_GNP_MAX_RETRIES,
@@ -526,12 +767,23 @@ fu_jabra_gnp_device_write_version(FuJabraGnpDevice *self,
 static gboolean
 fu_jabra_gnp_device_write_dfu_from_squif(FuJabraGnpDevice *self, GError **error)
 {
-	guint8 txbuf[FU_JABRA_GNP_BUF_SIZE] =
-	    {FU_JABRA_GNP_IFACE, 0x08, 0x00, self->sequence_number, 0x86, 0x0F, 0x1D};
-	guint8 rxbuf[FU_JABRA_GNP_BUF_SIZE] = {0x00};
-	FuJabraGnpTxData tx_data = {.txbuf = txbuf, .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT};
-	FuJabraGnpRxData rx_data = {.rxbuf = rxbuf,
-				    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT};
+	FuJabraGnpTxData tx_data = {
+	    .txbuf =
+		{
+		    FU_JABRA_GNP_IFACE,
+		    self->address,
+		    0x00,
+		    self->sequence_number,
+		    0x86,
+		    0x0F,
+		    0x1D,
+		},
+	    .timeout = FU_JABRA_GNP_STANDARD_SEND_TIMEOUT,
+	};
+	FuJabraGnpRxData rx_data = {
+	    .rxbuf = {0x00},
+	    .timeout = FU_JABRA_GNP_STANDARD_RECEIVE_TIMEOUT,
+	};
 	if (!fu_device_retry_full(FU_DEVICE(self),
 				  fu_jabra_gnp_device_tx_cb,
 				  FU_JABRA_GNP_MAX_RETRIES,
@@ -560,7 +812,8 @@ fu_jabra_gnp_device_write_dfu_from_squif(FuJabraGnpDevice *self, GError **error)
 
 static FuFirmware *
 fu_jabra_gnp_device_prepare_firmware(FuDevice *device,
-				     GBytes *fw,
+				     GInputStream *stream,
+				     FuProgress *progress,
 				     FwupdInstallFlags flags,
 				     GError **error)
 {
@@ -568,12 +821,12 @@ fu_jabra_gnp_device_prepare_firmware(FuDevice *device,
 	g_autoptr(FuFirmware) firmware = fu_jabra_gnp_firmware_new();
 
 	/* unzip and get images */
-	if (!fu_firmware_parse(firmware, fw, flags, error))
+	if (!fu_firmware_parse_stream(firmware, stream, 0x0, flags, error))
 		return NULL;
 	if (fu_jabra_gnp_firmware_get_dfu_pid(FU_JABRA_GNP_FIRMWARE(firmware)) != self->dfu_pid) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "wrong DFU PID, got 0x%x, expected 0x%x",
 			    fu_jabra_gnp_firmware_get_dfu_pid(FU_JABRA_GNP_FIRMWARE(firmware)),
 			    self->dfu_pid);
@@ -587,10 +840,14 @@ fu_jabra_gnp_device_probe(FuDevice *device, GError **error)
 {
 	g_autoptr(GError) error_local = NULL;
 	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
-	self->iface_hid =
-	    _g_usb_device_get_interface_for_class(fu_usb_device_get_dev(FU_USB_DEVICE(self)),
-						  G_USB_DEVICE_CLASS_HID,
-						  &error_local);
+
+	/* already set by parent */
+	if (self->address == FU_JABRA_GNP_ADDRESS_OTA_CHILD)
+		return TRUE;
+
+	self->iface_hid = _fu_usb_device_get_interface_for_class(FU_USB_DEVICE(self),
+								 FU_USB_CLASS_HID,
+								 &error_local);
 	if (self->iface_hid == 0xFF) {
 		g_set_error(error,
 			    FWUPD_ERROR,
@@ -603,16 +860,94 @@ fu_jabra_gnp_device_probe(FuDevice *device, GError **error)
 	return TRUE;
 }
 
+static gboolean
+fu_jabra_gnp_device_add_child(FuDevice *device, guint dfu_pid, GError **error)
+{
+	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
+	g_autoptr(FuJabraGnpDevice) child = NULL;
+
+	/* sanity check */
+	if (self->address != FU_JABRA_GNP_ADDRESS_PARENT) {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "expected address 0x%x, and got 0x%x",
+			    (guint)FU_JABRA_GNP_ADDRESS_OTA_CHILD,
+			    self->address);
+		return FALSE;
+	}
+
+	child = g_object_new(FU_TYPE_JABRA_GNP_DEVICE, "parent", FU_DEVICE(self), NULL);
+	child->iface_hid = self->iface_hid;
+	child->sequence_number = 0;
+	child->address = FU_JABRA_GNP_ADDRESS_OTA_CHILD;
+	child->dfu_pid = dfu_pid;
+
+	/* prohibit to close parent's communication descriptor */
+	fu_device_add_private_flag(FU_DEVICE(child), FU_DEVICE_PRIVATE_FLAG_USE_PARENT_FOR_OPEN);
+
+	fu_device_incorporate(FU_DEVICE(child),
+			      FU_DEVICE(self),
+			      FU_DEVICE_INCORPORATE_FLAG_PHYSICAL_ID);
+	fu_device_set_logical_id(FU_DEVICE(child), "ota_device");
+
+	if (!fu_device_setup(FU_DEVICE(child), error)) {
+		g_prefix_error(error, "failed to setup child device: ");
+		return FALSE;
+	}
+
+	fu_device_add_instance_u16(FU_DEVICE(child), "VID", fu_device_get_vid(FU_DEVICE(self)));
+	fu_device_add_instance_u16(FU_DEVICE(child), "PID", dfu_pid);
+	if (!fu_device_build_instance_id_full(FU_DEVICE(child),
+					      FU_DEVICE_INSTANCE_FLAG_QUIRKS |
+						  FU_DEVICE_INSTANCE_FLAG_VISIBLE,
+					      error,
+					      "USB",
+					      "VID",
+					      "PID",
+					      NULL))
+		return FALSE;
+	fu_device_add_child(FU_DEVICE(self), FU_DEVICE(child));
+
+	/* success */
+	return TRUE;
+}
+
 static gboolean
 fu_jabra_gnp_device_setup(FuDevice *device, GError **error)
 {
 	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
+	g_autoptr(GError) error_local = NULL;
 	if (!fu_jabra_gnp_device_read_name(self, error))
 		return FALSE;
 	if (!fu_jabra_gnp_device_read_version(self, error))
 		return FALSE;
 	if (!fu_jabra_gnp_device_read_dfu_pid(self, error))
 		return FALSE;
+	if (self->address == FU_JABRA_GNP_ADDRESS_PARENT) {
+		guint16 child_dfu_pid = 0;
+		if (!fu_jabra_gnp_device_read_child_dfu_pid(self, &child_dfu_pid, &error_local)) {
+			g_debug("unable to read child's PID, %s", error_local->message);
+			return TRUE;
+		}
+		if (child_dfu_pid > 0x0) {
+			if (!fu_jabra_gnp_device_add_child(FU_DEVICE(self), child_dfu_pid, error)) {
+				g_prefix_error(
+				    error,
+				    "found child device with PID 0x%x, but failed to add as child "
+				    "of parent with PID 0x%x, unpair or turn off child device to "
+				    "update parent device: ",
+				    child_dfu_pid,
+				    self->dfu_pid);
+				return FALSE;
+			}
+		}
+	}
+	if (self->address == FU_JABRA_GNP_ADDRESS_OTA_CHILD) {
+		if (!fu_jabra_gnp_device_read_child_battery_level(self, error))
+			return FALSE;
+	}
+
 	return TRUE;
 }
 
@@ -625,7 +960,7 @@ fu_jabra_gnp_device_write_image(FuJabraGnpDevice *self,
 {
 	const guint chunk_size = 52;
 	g_autoptr(FuChunkArray) chunks = NULL;
-	g_autoptr(GBytes) blob = NULL;
+	g_autoptr(GInputStream) stream = NULL;
 
 	/* progress */
 	fu_progress_set_id(progress, G_STRLOC);
@@ -637,8 +972,8 @@ fu_jabra_gnp_device_write_image(FuJabraGnpDevice *self,
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 1, "write-version");
 
 	/* write partition */
-	blob = fu_firmware_get_bytes(img, error);
-	if (blob == NULL)
+	stream = fu_firmware_get_stream(img, error);
+	if (stream == NULL)
 		return FALSE;
 	if (!fu_jabra_gnp_device_write_partition(self, fu_firmware_get_idx(img), error))
 		return FALSE;
@@ -655,11 +990,17 @@ fu_jabra_gnp_device_write_image(FuJabraGnpDevice *self,
 	fu_progress_step_done(progress);
 
 	/* write chunks */
-	chunks = fu_chunk_array_new_from_bytes(blob, 0x00, chunk_size);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						chunk_size,
+						error);
+	if (chunks == NULL)
+		return FALSE;
 	if (!fu_jabra_gnp_device_write_crc(self,
 					   fu_jabra_gnp_image_get_crc32(FU_JABRA_GNP_IMAGE(img)),
 					   fu_chunk_array_length(chunks),
-					   100,
+					   FU_JABRA_GNP_PRELOAD_COUNT,
 					   error))
 		return FALSE;
 	if (!fu_jabra_gnp_device_write_chunks(self, chunks, fu_progress_get_child(progress), error))
@@ -718,26 +1059,67 @@ fu_jabra_gnp_device_write_firmware(FuDevice *device,
 	/* write squif */
 	if (!fu_jabra_gnp_device_write_dfu_from_squif(self, error))
 		return FALSE;
+
 	return TRUE;
 }
 
+static gboolean
+fu_jabra_gnp_device_attach(FuDevice *device, FuProgress *progress, GError **error)
+{
+	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
+	/* ota device needs some time to restart and reconnect */
+	if (self->address == FU_JABRA_GNP_ADDRESS_OTA_CHILD) {
+		fu_device_sleep_full(FU_DEVICE(self), 45000, progress);
+		fu_device_set_remove_delay(FU_DEVICE(self), 10000);
+		fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
+	}
+	return TRUE;
+}
+
+static gboolean
+fu_jabra_gnp_device_set_quirk_kv(FuDevice *device,
+				 const gchar *key,
+				 const gchar *value,
+				 GError **error)
+{
+	FuJabraGnpDevice *self = FU_JABRA_GNP_DEVICE(device);
+
+	if (g_strcmp0(key, "JabraGnpAddress") == 0) {
+		guint64 val = 0;
+		if (!fu_strtoull(value, &val, 0x0, G_MAXUINT8, FU_INTEGER_BASE_AUTO, error))
+			return FALSE;
+		self->address = (guint8)val;
+		return TRUE;
+	}
+
+	/* failed */
+	g_set_error_literal(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "quirk key not supported");
+	return FALSE;
+}
+
 static void
-fu_colorhug_device_set_progress(FuDevice *self, FuProgress *progress)
+fu_jabra_gnp_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 5, "detach");
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 85, "write");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 75, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 5, "attach");
-	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 5, "reload");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 15, "reload");
 }
 
 static void
 fu_jabra_gnp_device_init(FuJabraGnpDevice *self)
 {
+	self->address = 0x08;
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_DUAL_IMAGE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SELF_RECOVERY);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_ADD_COUNTERPART_GUIDS);
 	fu_device_add_protocol(FU_DEVICE(self), "com.jabra.gnp");
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_TRIPLET);
 	fu_device_set_firmware_gtype(FU_DEVICE(self), FU_TYPE_JABRA_GNP_FIRMWARE);
@@ -746,11 +1128,13 @@ fu_jabra_gnp_device_init(FuJabraGnpDevice *self)
 static void
 fu_jabra_gnp_device_class_init(FuJabraGnpDeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
-	klass_device->to_string = fu_jabra_gnp_device_to_string;
-	klass_device->prepare_firmware = fu_jabra_gnp_device_prepare_firmware;
-	klass_device->probe = fu_jabra_gnp_device_probe;
-	klass_device->setup = fu_jabra_gnp_device_setup;
-	klass_device->write_firmware = fu_jabra_gnp_device_write_firmware;
-	klass_device->set_progress = fu_colorhug_device_set_progress;
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	device_class->to_string = fu_jabra_gnp_device_to_string;
+	device_class->prepare_firmware = fu_jabra_gnp_device_prepare_firmware;
+	device_class->probe = fu_jabra_gnp_device_probe;
+	device_class->setup = fu_jabra_gnp_device_setup;
+	device_class->write_firmware = fu_jabra_gnp_device_write_firmware;
+	device_class->attach = fu_jabra_gnp_device_attach;
+	device_class->set_quirk_kv = fu_jabra_gnp_device_set_quirk_kv;
+	device_class->set_progress = fu_jabra_gnp_device_set_progress;
 }
