@@ -1,5 +1,5 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #[derive(ToString)]
 enum FuLogitechHidppFeature {
@@ -102,7 +102,7 @@ enum FuLogitechHidppSubid {
 }
 
 enum FuLogitechHidppBootloaderCmd {
-    General_error = 0x01,
+    GeneralError = 0x01,
     Read = 0x10,
     Write = 0x20,
     WriteInvalidAddr = 0x21,
@@ -130,3 +130,67 @@ enum FuLogitechHidppBootloaderCmd {
     FlashRamInvalidOrder = 0xd4,
     WriteSignature = 0xe0,
 }
+
+// HID++1.0 error codes
+#[derive(ToString)]
+enum FuLogitechHidppErr {
+    Success,
+    InvalidSubid,
+    InvalidAddress,
+    InvalidValue,
+    ConnectFail,
+    TooManyDevices,
+    AlreadyExists,
+    Busy,
+    UnknownDevice,
+    ResourceError,
+    RequestUnavailable,
+    InvalidParamValue,
+    WrongPinCode,
+}
+
+// HID++2.0 error codes
+#[derive(ToString)]
+enum FuLogitechHidppErr2 {
+    NoError,
+    Unknown,
+    InvalidArgument,
+    OutOfRange,
+    HwError,
+    LogitechInternal,
+    InvalidFeatureIndex,
+    InvalidFunctionId,
+    Busy,
+    Unsupported,
+}
+
+#[derive(ToString)]
+enum FuLogitechHidppStatus {
+    Invalid,
+    PacketSuccess,
+    DfuSuccess,
+    WaitForEvent,
+    GenericError04,
+    DfuSuccessEntityRestartRequired,
+    DfuSuccessSystemRestartRequired,
+    GenericError10 = 0x10,
+    BadVoltage,
+    Unknown12,
+    UnsupportedEncryptionMode,
+    BadMagicString,
+    EraseFailure,
+    DfuNotStarted,
+    BadSequenceNumber,
+    UnsupportedCommand,
+    CommandInProgress,
+    AddressOutOfRange,
+    UnalignedAddress,
+    BadSize,
+    MissingProgramData,
+    MissingCheckData,
+    ProgramFailedToWrite,
+    ProgramFailedToVerify,
+    BadFirmware,
+    FirmwareCheckFailure,
+    BlockedCommand,
+}
