@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2021 Quectel Wireless Solutions Co., Ltd.
+ * Copyright 2021 Quectel Wireless Solutions Co., Ltd.
  *                    Ivan Mikhanchuk <ivan.mikhanchuk@quectel.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #pragma once
@@ -21,6 +21,8 @@ GByteArray *
 fu_sahara_loader_qdl_read(FuSaharaLoader *self, GError **error);
 gboolean
 fu_sahara_loader_qdl_write_bytes(FuSaharaLoader *self, GBytes *bytes, GError **error);
+void
+fu_sahara_loader_set_supports_zlp(FuSaharaLoader *self, gboolean supports_zlp);
 
 gboolean
 fu_sahara_loader_open(FuSaharaLoader *self, FuUsbDevice *usb_device, GError **error);
