@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2021 Denis Pynkin <denis.pynkin@collabora.com>
+ * Copyright 2021 Denis Pynkin <denis.pynkin@collabora.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -39,10 +39,7 @@ fu_nordic_hid_firmware_b0_write(FuFirmware *firmware, GError **error)
 }
 
 static gboolean
-fu_nordic_hid_firmware_b0_read_fwinfo(FuFirmware *firmware,
-				      guint8 const *buf,
-				      gsize bufsz,
-				      GError **error)
+fu_nordic_hid_firmware_b0_read_fwinfo(FuFirmware *firmware, GInputStream *stream, GError **error)
 {
 	guint32 magic_common;
 	guint32 magic_fwinfo;
@@ -58,34 +55,30 @@ fu_nordic_hid_firmware_b0_read_fwinfo(FuFirmware *firmware,
 	/* find correct offset to fwinfo */
 	for (guint32 i = 0; i < G_N_ELEMENTS(hdr_offset); i++) {
 		offset = hdr_offset[i];
-		if (!fu_memread_uint32_safe(buf,
-					    bufsz,
-					    offset,
-					    &magic_common,
-					    G_LITTLE_ENDIAN,
-					    error))
+		if (!fu_input_stream_read_u32(stream,
+					      offset,
+					      &magic_common,
+					      G_LITTLE_ENDIAN,
+					      error))
 			return FALSE;
-		if (!fu_memread_uint32_safe(buf,
-					    bufsz,
-					    offset + 0x04,
-					    &magic_fwinfo,
-					    G_LITTLE_ENDIAN,
-					    error))
+		if (!fu_input_stream_read_u32(stream,
+					      offset + 0x04,
+					      &magic_fwinfo,
+					      G_LITTLE_ENDIAN,
+					      error))
 			return FALSE;
-		if (!fu_memread_uint32_safe(buf,
-					    bufsz,
-					    offset + 0x08,
-					    &magic_compat,
-					    G_LITTLE_ENDIAN,
-					    error))
+		if (!fu_input_stream_read_u32(stream,
+					      offset + 0x08,
+					      &magic_compat,
+					      G_LITTLE_ENDIAN,
+					      error))
 			return FALSE;
 		/* version */
-		if (!fu_memread_uint32_safe(buf,
-					    bufsz,
-					    offset + 0x14,
-					    &ver_build_nr,
-					    G_LITTLE_ENDIAN,
-					    error))
+		if (!fu_input_stream_read_u32(stream,
+					      offset + 0x14,
+					      &ver_build_nr,
+					      G_LITTLE_ENDIAN,
+					      error))
 			return FALSE;
 
 		if (magic_common != UPDATE_IMAGE_MAGIC_COMMON ||
@@ -116,28 +109,14 @@ fu_nordic_hid_firmware_b0_read_fwinfo(FuFirmware *firmware,
 
 static gboolean
 fu_nordic_hid_firmware_b0_parse(FuFirmware *firmware,
-				GBytes *fw,
-				gsize offset,
+				GInputStream *stream,
 				FwupdInstallFlags flags,
 				GError **error)
 {
-	const guint8 *buf;
-	gsize bufsz = 0;
-
 	if (!FU_FIRMWARE_CLASS(fu_nordic_hid_firmware_b0_parent_class)
-		 ->parse(firmware, fw, offset, flags, error))
+		 ->parse(firmware, stream, flags, error))
 		return FALSE;
-
-	buf = g_bytes_get_data(fw, &bufsz);
-	if (buf == NULL) {
-		g_set_error_literal(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_INVALID_FILE,
-				    "unable to get the image binary");
-		return FALSE;
-	}
-
-	return fu_nordic_hid_firmware_b0_read_fwinfo(firmware, buf, bufsz, error);
+	return fu_nordic_hid_firmware_b0_read_fwinfo(firmware, stream, error);
 }
 
 static void
@@ -148,7 +127,7 @@ fu_nordic_hid_firmware_b0_init(FuNordicHidFirmwareB0 *self)
 static void
 fu_nordic_hid_firmware_b0_class_init(FuNordicHidFirmwareB0Class *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
-	klass_firmware->parse = fu_nordic_hid_firmware_b0_parse;
-	klass_firmware->write = fu_nordic_hid_firmware_b0_write;
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->parse = fu_nordic_hid_firmware_b0_parse;
+	firmware_class->write = fu_nordic_hid_firmware_b0_write;
 }
