@@ -1,7 +1,8 @@
 // Copyright 2024 Richard Hughes <richard@hughsie.com>
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
-#[derive(ValidateBytes)]
+#[derive(ValidateStream, Default)]
+#[repr(C, packed)]
 struct FuStructParadeUsbhubHdr {
     signature: u16be == 0x55AA,
 }
