@@ -8,6 +8,7 @@
 
 #include <curl/curl.h>
 
+#include "fu-redfish-device.h"
 #include "fu-redfish-multipart-device.h"
 #include "fu-redfish-request.h"
 
@@ -123,13 +124,17 @@ fu_redfish_multipart_device_write_firmware(FuDevice *device,
 		return FALSE;
 	}
 	location = json_object_get_string_member(json_obj, "@odata.id");
-	return fu_redfish_device_poll_task(FU_REDFISH_DEVICE(self), location, progress, error);
+	return fu_redfish_device_poll_task(FU_REDFISH_DEVICE(self),
+					   location,
+					   progress,
+					   error);
 }
 
 static void
 fu_redfish_multipart_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 100, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "attach");
