@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -10,12 +10,17 @@
 #include "fu-ipmi-device.h"
 #endif
 
+#include "fu-ipmi-device.h"
 #include "fu-redfish-backend.h"
 #include "fu-redfish-common.h"
 #include "fu-redfish-device.h"
+#include "fu-redfish-hpe-device.h"
+#include "fu-redfish-legacy-device.h"
+#include "fu-redfish-multipart-device.h"
 #include "fu-redfish-network.h"
 #include "fu-redfish-plugin.h"
 #include "fu-redfish-smbios.h"
+#include "fu-redfish-smc-device.h"
 #include "fu-redfish-struct.h"
 
 #define FU_REDFISH_PLUGIN_CLEANUP_RETRIES_DELAY 10 /* seconds */
@@ -35,21 +40,24 @@ fu_redfish_plugin_to_string(FuPlugin *plugin, guint idt, GString *str)
 	fu_backend_add_string(FU_BACKEND(self->backend), idt, str);
 	if (self->smbios != NULL) {
 		g_autofree gchar *smbios = fu_firmware_to_string(FU_FIRMWARE(self->smbios));
-		fu_string_append(str, idt, "Smbios", smbios);
+		fwupd_codec_string_append(str, idt, "Smbios", smbios);
 	}
-	fu_string_append(str, idt, "Vendor", fu_redfish_backend_get_vendor(self->backend));
-	fu_string_append(str, idt, "Version", fu_redfish_backend_get_version(self->backend));
-	fu_string_append(str, idt, "UUID", fu_redfish_backend_get_uuid(self->backend));
+	fwupd_codec_string_append(str, idt, "Vendor", fu_redfish_backend_get_vendor(self->backend));
+	fwupd_codec_string_append(str,
+				  idt,
+				  "Version",
+				  fu_redfish_backend_get_version(self->backend));
+	fwupd_codec_string_append(str, idt, "UUID", fu_redfish_backend_get_uuid(self->backend));
 }
 
 static gchar *
-fu_common_generate_password(guint length)
+fu_redfish_plugin_generate_password(guint length)
 {
 	GString *str = g_string_sized_new(length);
 
 	/* get a random password string */
 	while (str->len < length) {
-		gchar tmp = (gchar)g_random_int_range(0x0, 0xff);
+		gchar tmp = (gchar)g_random_int_range(0x0, 0xff); /* nocheck:blocked */
 		if (g_ascii_isalnum(tmp))
 			g_string_append_c(str, tmp);
 	}
@@ -60,7 +68,7 @@ static gboolean
 fu_redfish_plugin_change_expired(FuPlugin *plugin, GError **error)
 {
 	FuRedfishPlugin *self = FU_REDFISH_PLUGIN(plugin);
-	g_autofree gchar *password_new = fu_common_generate_password(15);
+	g_autofree gchar *password_new = fu_redfish_plugin_generate_password(15);
 	g_autofree gchar *uri = NULL;
 	g_autoptr(FuRedfishRequest) request = NULL;
 	g_autoptr(JsonBuilder) builder = json_builder_new();
@@ -140,9 +148,23 @@ fu_redfish_plugin_set_credentials(FuPlugin *plugin, const gchar *username, const
 	fu_redfish_backend_set_password(self->backend, password);
 }
 
+gboolean
+fu_redfish_plugin_reload(FuPlugin *plugin, FuProgress *progress, GError **error)
+{
+	FuRedfishPlugin *self = FU_REDFISH_PLUGIN(plugin);
+
+	fu_backend_invalidate(FU_BACKEND(self->backend));
+	return fu_backend_setup(FU_BACKEND(self->backend),
+				FU_BACKEND_SETUP_FLAG_NONE,
+				progress,
+				error);
+}
+
 static gboolean
 fu_redfish_plugin_discover_uefi_credentials(FuPlugin *plugin, GError **error)
 {
+	FuContext *ctx = fu_plugin_get_context(plugin);
+	FuEfivars *efivars = fu_context_get_efivars(ctx);
 	FuRedfishPlugin *self = FU_REDFISH_PLUGIN(plugin);
 	gsize bufsz = 0;
 	guint32 indications = 0x0;
@@ -152,12 +174,13 @@ fu_redfish_plugin_discover_uefi_credentials(FuPlugin *plugin, GError **error)
 	g_autoptr(GBytes) userpass = NULL;
 
 	/* get the uint32 specifying if there are EFI variables set */
-	if (!fu_efivar_get_data(REDFISH_EFI_INFORMATION_GUID,
-				REDFISH_EFI_INFORMATION_INDICATIONS,
-				&buf,
-				&bufsz,
-				NULL,
-				error))
+	if (!fu_efivars_get_data(efivars,
+				 REDFISH_EFI_INFORMATION_GUID,
+				 REDFISH_EFI_INFORMATION_INDICATIONS,
+				 &buf,
+				 &bufsz,
+				 NULL,
+				 error))
 		return FALSE;
 	if (!fu_memread_uint32_safe(buf, bufsz, 0x0, &indications, G_LITTLE_ENDIAN, error))
 		return FALSE;
@@ -170,10 +193,11 @@ fu_redfish_plugin_discover_uefi_credentials(FuPlugin *plugin, GError **error)
 	}
 
 	/* read the correct EFI var for runtime */
-	userpass = fu_efivar_get_data_bytes(REDFISH_EFI_INFORMATION_GUID,
-					    REDFISH_EFI_INFORMATION_OS_CREDENTIALS,
-					    NULL,
-					    error);
+	userpass = fu_efivars_get_data_bytes(efivars,
+					     REDFISH_EFI_INFORMATION_GUID,
+					     REDFISH_EFI_INFORMATION_OS_CREDENTIALS,
+					     NULL,
+					     error);
 	if (userpass == NULL)
 		return FALSE;
 
@@ -204,15 +228,9 @@ fu_redfish_plugin_discover_smbios_table(FuPlugin *plugin, GError **error)
 	/* in self tests */
 	smbios_data_fn = g_getenv("FWUPD_REDFISH_SMBIOS_DATA");
 	if (smbios_data_fn != NULL) {
-		g_autoptr(GBytes) type42_blob = fu_bytes_get_contents(smbios_data_fn, error);
 		g_autoptr(FuRedfishSmbios) smbios = fu_redfish_smbios_new();
-		if (type42_blob == NULL)
-			return FALSE;
-		if (!fu_firmware_parse(FU_FIRMWARE(smbios),
-				       type42_blob,
-				       FWUPD_INSTALL_FLAG_NO_SEARCH,
-				       error)) {
-			g_prefix_error(error, "failed to parse SMBIOS entry type 42: ");
+		if (!fu_firmware_build_from_filename(FU_FIRMWARE(smbios), smbios_data_fn, error)) {
+			g_prefix_error(error, "failed to build SMBIOS entry type 42: ");
 			return FALSE;
 		}
 		g_set_object(&self->smbios, smbios);
@@ -220,16 +238,20 @@ fu_redfish_plugin_discover_smbios_table(FuPlugin *plugin, GError **error)
 	}
 
 	/* is optional */
-	type42_tables = fu_context_get_smbios_data(ctx, REDFISH_SMBIOS_TABLE_TYPE, NULL);
+	type42_tables = fu_context_get_smbios_data(ctx,
+						   REDFISH_SMBIOS_TABLE_TYPE,
+						   FU_SMBIOS_STRUCTURE_LENGTH_ANY,
+						   NULL);
 	if (type42_tables == NULL)
 		return TRUE;
 	for (guint i = 0; i < type42_tables->len; i++) {
 		GBytes *type42_blob = g_ptr_array_index(type42_tables, i);
 		g_autoptr(FuRedfishSmbios) smbios = fu_redfish_smbios_new();
-		if (!fu_firmware_parse(FU_FIRMWARE(smbios),
-				       type42_blob,
-				       FWUPD_INSTALL_FLAG_NO_SEARCH,
-				       error)) {
+		if (!fu_firmware_parse_bytes(FU_FIRMWARE(smbios),
+					     type42_blob,
+					     0x0,
+					     FWUPD_INSTALL_FLAG_NO_SEARCH,
+					     error)) {
 			g_prefix_error(error, "failed to parse SMBIOS entry type 42: ");
 			return FALSE;
 		}
@@ -247,6 +269,7 @@ fu_redfish_plugin_discover_smbios_table(FuPlugin *plugin, GError **error)
 static gboolean
 fu_redfish_plugin_autoconnect_network_device(FuRedfishPlugin *self, GError **error)
 {
+	FuContext *ctx = fu_plugin_get_context(FU_PLUGIN(self));
 	g_autofree gchar *hostname = NULL;
 	g_autoptr(FuRedfishNetworkDevice) device = NULL;
 
@@ -262,7 +285,8 @@ fu_redfish_plugin_autoconnect_network_device(FuRedfishPlugin *self, GError **err
 		const gchar *mac_addr = fu_redfish_smbios_get_mac_addr(self->smbios);
 		if (mac_addr != NULL) {
 			g_autoptr(GError) error_network = NULL;
-			device = fu_redfish_network_device_for_mac_addr(mac_addr, &error_network);
+			device =
+			    fu_redfish_network_device_for_mac_addr(ctx, mac_addr, &error_network);
 			if (device == NULL)
 				g_debug("failed to get device: %s", error_network->message);
 		}
@@ -272,7 +296,8 @@ fu_redfish_plugin_autoconnect_network_device(FuRedfishPlugin *self, GError **err
 		guint16 pid = fu_redfish_smbios_get_pid(self->smbios);
 		if (vid != 0x0 && pid != 0x0) {
 			g_autoptr(GError) error_network = NULL;
-			device = fu_redfish_network_device_for_vid_pid(vid, pid, &error_network);
+			device =
+			    fu_redfish_network_device_for_vid_pid(ctx, vid, pid, &error_network);
 			if (device == NULL)
 				g_debug("failed to get device: %s", error_network->message);
 		}
@@ -313,8 +338,8 @@ fu_redfish_plugin_ipmi_create_user(FuPlugin *plugin, GError **error)
 	FuRedfishPlugin *self = FU_REDFISH_PLUGIN(plugin);
 	const gchar *username_fwupd = "fwupd";
 	guint8 user_id = G_MAXUINT8;
-	g_autofree gchar *password_new = fu_common_generate_password(15);
-	g_autofree gchar *password_tmp = fu_common_generate_password(15);
+	g_autofree gchar *password_new = fu_redfish_plugin_generate_password(15);
+	g_autofree gchar *password_tmp = fu_redfish_plugin_generate_password(15);
 	g_autofree gchar *uri = NULL;
 	g_autoptr(FuDeviceLocker) locker = NULL;
 	g_autoptr(FuIpmiDevice) device = fu_ipmi_device_new(fu_plugin_get_context(plugin));
@@ -360,9 +385,7 @@ fu_redfish_plugin_ipmi_create_user(FuPlugin *plugin, GError **error)
 	/* OEM specific for Advantech manufacture */
 	if (fu_context_has_hwid_guid(fu_plugin_get_context(plugin),
 				     "18789130-a714-53c0-b025-fa93801d3995")) {
-		if (!fu_redfish_device_set_user_group_redfish_enable_advantech(device,
-									       user_id,
-									       error))
+		if (!fu_ipmi_device_set_user_group_redfish_enable_advantech(device, user_id, error))
 			return FALSE;
 	}
 	fu_redfish_backend_set_username(self->backend, username_fwupd);
@@ -455,8 +478,15 @@ fu_redfish_plugin_startup(FuPlugin *plugin, FuProgress *progress, GError **error
 
 		split = g_strsplit(ip_str, ":", 2);
 		fu_redfish_backend_set_hostname(self->backend, split[0]);
-		if (g_strv_length(split) > 1)
-			port = g_ascii_strtoull(split[1], NULL, 10);
+		if (g_strv_length(split) > 1) {
+			if (!fu_strtoull(split[1],
+					 &port,
+					 0,
+					 G_MAXUINT64,
+					 FU_INTEGER_BASE_10,
+					 error))
+				return FALSE;
+		}
 		if (port == 0 || port == G_MAXUINT64) {
 			g_set_error_literal(error,
 					    FWUPD_ERROR,
@@ -517,7 +547,10 @@ fu_redfish_plugin_startup(FuPlugin *plugin, FuProgress *progress, GError **error
 	}
 #endif
 
-	return fu_backend_setup(FU_BACKEND(self->backend), progress, error);
+	return fu_backend_setup(FU_BACKEND(self->backend),
+				FU_BACKEND_SETUP_FLAG_NONE,
+				progress,
+				error);
 }
 
 static gboolean
@@ -529,7 +562,10 @@ fu_redfish_plugin_cleanup_setup_cb(FuDevice *device, gpointer user_data, GError
 	/* the network adaptor might not auto-connect when coming back */
 	if (!fu_redfish_plugin_autoconnect_network_device(self, error))
 		return FALSE;
-	return fu_backend_setup(FU_BACKEND(self->backend), progress, error);
+	return fu_backend_setup(FU_BACKEND(self->backend),
+				FU_BACKEND_SETUP_FLAG_NONE,
+				progress,
+				error);
 }
 
 static gboolean
@@ -606,7 +642,12 @@ fu_redfish_plugin_cleanup(FuPlugin *plugin,
 
 	/* read the config file to work out how long to wait */
 	restart_timeout_str = fu_plugin_get_config_value(plugin, "ManagerResetTimeout");
-	if (!fu_strtoull(restart_timeout_str, &reset_timeout, 1, 86400, error))
+	if (!fu_strtoull(restart_timeout_str,
+			 &reset_timeout,
+			 1,
+			 86400,
+			 FU_INTEGER_BASE_AUTO,
+			 error))
 		return FALSE;
 
 	/* wait for the BMC to come back */
@@ -643,6 +684,31 @@ fu_redfish_plugin_cleanup(FuPlugin *plugin,
 	return TRUE;
 }
 
+static gboolean
+fu_redfish_plugin_modify_config(FuPlugin *plugin,
+				const gchar *key,
+				const gchar *value,
+				GError **error)
+{
+	const gchar *keys[] = {"CACheck",
+			       "IpmiDisableCreateUser",
+			       "ManagerResetTimeout",
+			       "Password",
+			       "Uri",
+			       "Username",
+			       "UserUri",
+			       NULL};
+	if (!g_strv_contains(keys, key)) {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "config key %s not supported",
+			    key);
+		return FALSE;
+	}
+	return fu_plugin_set_config_value(plugin, key, value, error);
+}
+
 static void
 fu_redfish_plugin_init(FuRedfishPlugin *self)
 {
@@ -659,6 +725,11 @@ fu_redfish_plugin_constructed(GObject *obj)
 	self->backend = fu_redfish_backend_new(ctx);
 	fu_plugin_add_firmware_gtype(plugin, NULL, FU_TYPE_REDFISH_SMBIOS);
 	fu_plugin_add_flag(plugin, FWUPD_PLUGIN_FLAG_SECURE_CONFIG);
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_REDFISH_HPE_DEVICE);	      /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_REDFISH_LEGACY_DEVICE);    /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_REDFISH_MULTIPART_DEVICE); /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_IPMI_DEVICE);	      /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_REDFISH_SMC_DEVICE);	      /* coverage */
 
 	/* defaults changed here will also be reflected in the fwupd.conf man page */
 	fu_plugin_set_config_default(plugin, "CACheck", "false");
@@ -671,7 +742,7 @@ fu_redfish_plugin_constructed(GObject *obj)
 }
 
 static void
-fu_redfish_finalize(GObject *obj)
+fu_redfish_plugin_finalize(GObject *obj)
 {
 	FuRedfishPlugin *self = FU_REDFISH_PLUGIN(obj);
 	if (self->smbios != NULL)
@@ -687,10 +758,11 @@ fu_redfish_plugin_class_init(FuRedfishPluginClass *klass)
 	FuPluginClass *plugin_class = FU_PLUGIN_CLASS(klass);
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
-	object_class->finalize = fu_redfish_finalize;
+	object_class->finalize = fu_redfish_plugin_finalize;
 	plugin_class->constructed = fu_redfish_plugin_constructed;
 	plugin_class->to_string = fu_redfish_plugin_to_string;
 	plugin_class->startup = fu_redfish_plugin_startup;
 	plugin_class->coldplug = fu_redfish_plugin_coldplug;
 	plugin_class->cleanup = fu_redfish_plugin_cleanup;
+	plugin_class->modify_config = fu_redfish_plugin_modify_config;
 }
