@@ -1,7 +1,8 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
-#[derive(New, ParseBytes)]
+#[derive(New, ParseStream)]
+#[repr(C, packed)]
 struct FuStructRedfishProtocolOverIp {
     service_uuid: Guid,
     host_ip_assignment_type: u8,
@@ -55,7 +56,8 @@ enum FuRedfishSmbiosInterfaceType {
     Oem = 0xF0,
 }
 
-#[derive(ParseBytes)]
+#[derive(ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructRedfishSmbiosType42 {
     type: u8 == 42,
     length: u8,
