@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2021 Realtek Corporation
- * Copyright (C) 2021 Ricky Wu <ricky_wu@realtek.com> <spring1527@gmail.com>
- * Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
+ * Copyright 2021 Realtek Corporation
+ * Copyright 2021 Ricky Wu <ricky_wu@realtek.com> <spring1527@gmail.com>
+ * Copyright 2021 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -34,7 +34,8 @@ typedef enum {
 } IspCmd;
 
 static gboolean
-fu_rts54hub_rtd21xx_ensure_version_unlocked(FuRts54hubRtd21xxForeground *self, GError **error)
+fu_rts54hub_rtd21xx_foreground_ensure_version_unlocked(FuRts54hubRtd21xxForeground *self,
+						       GError **error)
 {
 	guint8 buf_rep[7] = {0x00};
 	guint8 buf_req[] = {ISP_CMD_GET_FW_INFO};
@@ -197,7 +198,7 @@ fu_rts54hub_rtd21xx_foreground_setup(FuDevice *device, GError **error)
 					   error);
 	if (locker == NULL)
 		return FALSE;
-	if (!fu_rts54hub_rtd21xx_ensure_version_unlocked(self, error))
+	if (!fu_rts54hub_rtd21xx_foreground_ensure_version_unlocked(self, error))
 		return FALSE;
 
 	/* success */
@@ -225,14 +226,12 @@ fu_rts54hub_rtd21xx_foreground_write_firmware(FuDevice *device,
 					      GError **error)
 {
 	FuRts54hubRtd21xxForeground *self = FU_RTS54HUB_RTD21XX_FOREGROUND(device);
-	const guint8 *fwbuf;
-	gsize fwbufsz = 0;
 	guint32 project_addr;
 	guint8 project_id_count;
 	guint8 read_buf[10] = {0x0};
 	guint8 write_buf[ISP_PACKET_SIZE] = {0x0};
 	g_autoptr(FuDeviceLocker) locker = NULL;
-	g_autoptr(GBytes) fw = NULL;
+	g_autoptr(GInputStream) stream = NULL;
 	g_autoptr(FuChunkArray) chunks = NULL;
 
 	/* progress */
@@ -248,10 +247,9 @@ fu_rts54hub_rtd21xx_foreground_write_firmware(FuDevice *device,
 		return FALSE;
 
 	/* simple image */
-	fw = fu_firmware_get_bytes(firmware, error);
-	if (fw == NULL)
+	stream = fu_firmware_get_stream(firmware, error);
+	if (stream == NULL)
 		return FALSE;
-	fwbuf = g_bytes_get_data(fw, &fwbufsz);
 
 	/* enable ISP high priority */
 	write_buf[0] = ISP_CMD_ENTER_FW_UPDATE;
@@ -293,8 +291,8 @@ fu_rts54hub_rtd21xx_foreground_write_firmware(FuDevice *device,
 	}
 	if (read_buf[0] != ISP_STATUS_IDLE_SUCCESS) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "failed project ID with error 0x%02x: ",
 			    read_buf[0]);
 		return FALSE;
@@ -304,14 +302,13 @@ fu_rts54hub_rtd21xx_foreground_write_firmware(FuDevice *device,
 	project_addr = fu_memread_uint32(read_buf + 1, G_BIG_ENDIAN);
 	project_id_count = read_buf[5];
 	write_buf[0] = ISP_CMD_SYNC_IDENTIFY_CODE;
-	if (!fu_memcpy_safe(write_buf,
-			    sizeof(write_buf),
-			    0x1, /* dst */
-			    fwbuf,
-			    fwbufsz,
-			    project_addr, /* src */
-			    project_id_count,
-			    error)) {
+	if (!fu_input_stream_read_safe(stream,
+				       write_buf,
+				       sizeof(write_buf),
+				       0x1,	     /* dst */
+				       project_addr, /* src */
+				       project_id_count,
+				       error)) {
 		g_prefix_error(error, "failed to write project ID from 0x%04x: ", project_addr);
 		return FALSE;
 	}
@@ -342,9 +339,20 @@ fu_rts54hub_rtd21xx_foreground_write_firmware(FuDevice *device,
 	fu_progress_step_done(progress);
 
 	/* send data */
-	chunks = fu_chunk_array_new_from_bytes(fw, 0x00, ISP_DATA_BLOCKSIZE);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						ISP_DATA_BLOCKSIZE,
+						error);
+	if (chunks == NULL)
+		return FALSE;
 	for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
-		g_autoptr(FuChunk) chk = fu_chunk_array_index(chunks, i);
+		g_autoptr(FuChunk) chk = NULL;
+
+		/* prepare chunk */
+		chk = fu_chunk_array_index(chunks, i, error);
+		if (chk == NULL)
+			return FALSE;
 		if (!fu_rts54hub_rtd21xx_device_read_status(FU_RTS54HUB_RTD21XX_DEVICE(self),
 							    NULL,
 							    error))
@@ -357,7 +365,7 @@ fu_rts54hub_rtd21xx_foreground_write_firmware(FuDevice *device,
 							  error)) {
 			g_prefix_error(error,
 				       "failed to write @0x%04x: ",
-				       fu_chunk_get_address(chk));
+				       (guint)fu_chunk_get_address(chk));
 			return FALSE;
 		}
 
@@ -395,10 +403,10 @@ fu_rts54hub_rtd21xx_foreground_init(FuRts54hubRtd21xxForeground *self)
 static void
 fu_rts54hub_rtd21xx_foreground_class_init(FuRts54hubRtd21xxForegroundClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
-	klass_device->setup = fu_rts54hub_rtd21xx_foreground_setup;
-	klass_device->reload = fu_rts54hub_rtd21xx_foreground_reload;
-	klass_device->attach = fu_rts54hub_rtd21xx_foreground_attach;
-	klass_device->detach = fu_rts54hub_rtd21xx_foreground_detach;
-	klass_device->write_firmware = fu_rts54hub_rtd21xx_foreground_write_firmware;
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	device_class->setup = fu_rts54hub_rtd21xx_foreground_setup;
+	device_class->reload = fu_rts54hub_rtd21xx_foreground_reload;
+	device_class->attach = fu_rts54hub_rtd21xx_foreground_attach;
+	device_class->detach = fu_rts54hub_rtd21xx_foreground_detach;
+	device_class->write_firmware = fu_rts54hub_rtd21xx_foreground_write_firmware;
 }
