@@ -1,20 +1,16 @@
 /*
- * Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2017 Christian J. Kellner <christian@kellner.me>
- * Copyright (C) 2020 Mario Limonciello <mario.limonciello@dell.com>
+ * Copyright 2021 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Christian J. Kellner <christian@kellner.me>
+ * Copyright 2020 Mario Limonciello <mario.limonciello@dell.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
 
 #include "fu-thunderbolt-common.h"
 #include "fu-thunderbolt-controller.h"
-
-typedef enum {
-	FU_THUNDERBOLT_CONTROLLER_KIND_DEVICE,
-	FU_THUNDERBOLT_CONTROLLER_KIND_HOST,
-} FuThunderboltControllerKind;
+#include "fu-thunderbolt-struct.h"
 
 struct _FuThunderboltController {
 	FuThunderboltDevice parent_instance;
@@ -47,54 +43,74 @@ fu_thunderbolt_controller_check_safe_mode(FuThunderboltController *self)
 	fu_device_set_metadata_boolean(FU_DEVICE(self), FU_DEVICE_METADATA_TBT_IS_SAFE_MODE, TRUE);
 }
 
-static const gchar *
-fu_thunderbolt_controller_kind_to_string(FuThunderboltController *self)
+static void
+fu_thunderbolt_controller_ensure_fallback_name(FuThunderboltController *self)
 {
 	if (self->controller_kind == FU_THUNDERBOLT_CONTROLLER_KIND_HOST) {
-		if (self->gen >= 4)
-			return "USB4 host controller";
-		return "Thunderbolt host controller";
+		if (self->gen >= 4) {
+			fu_device_set_name(FU_DEVICE(self), "USB4 host controller");
+			return;
+		}
+		fu_device_set_name(FU_DEVICE(self), "Thunderbolt host controller");
 	}
 	if (self->controller_kind == FU_THUNDERBOLT_CONTROLLER_KIND_DEVICE) {
-		if (self->gen >= 4)
-			return "USB4 device controller";
-		return "Thunderbolt device controller";
+		if (self->gen >= 4) {
+			fu_device_set_name(FU_DEVICE(self), "USB4 device controller");
+			return;
+		}
+		fu_device_set_name(FU_DEVICE(self), "Thunderbolt device controller");
 	}
-	return "Unknown";
 }
 
 static void
 fu_thunderbolt_controller_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuThunderboltController *self = FU_THUNDERBOLT_CONTROLLER(device);
-
-	/* FuThunderboltDevice->to_string */
-	FU_DEVICE_CLASS(fu_thunderbolt_controller_parent_class)->to_string(device, idt, str);
-
-	fu_string_append(str, idt, "DeviceType", fu_thunderbolt_controller_kind_to_string(self));
-	fu_string_append_kb(str, idt, "SafeMode", self->safe_mode);
-	fu_string_append_kb(str, idt, "NativeMode", self->is_native);
-	fu_string_append_ku(str, idt, "Generation", self->gen);
+	fwupd_codec_string_append(str,
+				  idt,
+				  "DeviceType",
+				  fu_thunderbolt_controller_kind_to_string(self->controller_kind));
+	fwupd_codec_string_append_bool(str, idt, "SafeMode", self->safe_mode);
+	fwupd_codec_string_append_bool(str, idt, "NativeMode", self->is_native);
+	fwupd_codec_string_append_int(str, idt, "Generation", self->gen);
 }
 
 static gboolean
 fu_thunderbolt_controller_probe(FuDevice *device, GError **error)
 {
 	FuThunderboltController *self = FU_THUNDERBOLT_CONTROLLER(device);
-	const gchar *unique_id;
-	g_autofree gchar *parent_name = NULL;
+	g_autofree gchar *attr_unique_id = NULL;
+	g_autofree gchar *prop_type = NULL;
 
 	/* FuUdevDevice->probe */
 	if (!FU_DEVICE_CLASS(fu_thunderbolt_controller_parent_class)->probe(device, error))
 		return FALSE;
 
 	/* determine if host controller or not */
-	parent_name = fu_udev_device_get_parent_name(FU_UDEV_DEVICE(self));
-	if (parent_name != NULL && g_str_has_prefix(parent_name, "domain"))
-		self->controller_kind = FU_THUNDERBOLT_CONTROLLER_KIND_HOST;
-	unique_id = fu_udev_device_get_sysfs_attr(FU_UDEV_DEVICE(device), "unique_id", NULL);
-	if (unique_id != NULL)
-		fu_device_set_physical_id(device, unique_id);
+	prop_type = fu_udev_device_read_property(FU_UDEV_DEVICE(self), "USB4_TYPE", NULL);
+	if (prop_type != NULL) {
+		self->controller_kind = fu_thunderbolt_controller_kind_from_string(prop_type);
+	} else {
+		g_autoptr(FuDevice) device_parent =
+		    fu_device_get_backend_parent_with_subsystem(FU_DEVICE(self),
+								"thunderbolt:thunderbolt_domain",
+								NULL);
+		if (device_parent != NULL) {
+			g_autofree gchar *parent_name = g_path_get_basename(
+			    fu_udev_device_get_sysfs_path(FU_UDEV_DEVICE(device_parent)));
+			if (g_str_has_prefix(parent_name, "domain"))
+				self->controller_kind = FU_THUNDERBOLT_CONTROLLER_KIND_HOST;
+			else
+				self->controller_kind = FU_THUNDERBOLT_CONTROLLER_KIND_DEVICE;
+		}
+	}
+
+	attr_unique_id = fu_udev_device_read_sysfs(FU_UDEV_DEVICE(device),
+						   "unique_id",
+						   FU_UDEV_DEVICE_ATTR_READ_TIMEOUT_DEFAULT,
+						   NULL);
+	if (attr_unique_id != NULL)
+		fu_device_set_physical_id(device, attr_unique_id);
 
 	/* success */
 	return TRUE;
@@ -104,9 +120,10 @@ static gboolean
 fu_thunderbolt_controller_read_status_block(FuThunderboltController *self, GError **error)
 {
 	gsize nr_chunks;
+	g_autoptr(GBytes) blob = NULL;
 	g_autoptr(GFile) nvmem = NULL;
-	g_autoptr(GBytes) controller_fw = NULL;
 	g_autoptr(GInputStream) istr = NULL;
+	g_autoptr(GInputStream) istr_partial = NULL;
 	g_autoptr(FuFirmware) firmware = NULL;
 
 	nvmem = fu_thunderbolt_device_find_nvmem(FU_THUNDERBOLT_DEVICE(self), TRUE, error);
@@ -118,10 +135,11 @@ fu_thunderbolt_controller_read_status_block(FuThunderboltController *self, GErro
 	istr = G_INPUT_STREAM(g_file_read(nvmem, NULL, error));
 	if (istr == NULL)
 		return FALSE;
-	controller_fw = g_input_stream_read_bytes(istr, nr_chunks * FU_TBT_CHUNK_SZ, NULL, error);
-	if (controller_fw == NULL)
+	blob = fu_input_stream_read_bytes(istr, 0x0, nr_chunks * FU_TBT_CHUNK_SZ, NULL, error);
+	if (blob == NULL)
 		return FALSE;
-	firmware = fu_firmware_new_from_gtypes(controller_fw,
+	istr_partial = g_memory_input_stream_new_from_bytes(blob);
+	firmware = fu_firmware_new_from_gtypes(istr_partial,
 					       0x0,
 					       FWUPD_INSTALL_FLAG_NO_SEARCH,
 					       error,
@@ -172,38 +190,43 @@ fu_thunderbolt_controller_setup_usb4(FuThunderboltController *self, GError **err
 {
 	if (!fu_thunderbolt_udev_set_port_offline(FU_UDEV_DEVICE(self), error))
 		return FALSE;
+	if (!fu_thunderbolt_udev_rescan_port(FU_UDEV_DEVICE(self), error))
+		return FALSE;
+	if (self->host_online_timer_id > 0)
+		g_source_remove(self->host_online_timer_id);
 	self->host_online_timer_id =
-	    g_timeout_add_seconds_full(G_PRIORITY_DEFAULT,
-				       5, /* seconds */
-				       fu_thunderbolt_controller_set_port_online_cb,
-				       g_object_ref(self),
-				       g_object_unref);
+	    g_timeout_add_seconds(5, fu_thunderbolt_controller_set_port_online_cb, self);
 	return TRUE;
 }
 
-static void
-fu_thunderbolt_controller_set_signed(FuDevice *device)
-{
-	FuThunderboltController *self = FU_THUNDERBOLT_CONTROLLER(device);
-	GUdevDevice *udev_device = fu_udev_device_get_dev(FU_UDEV_DEVICE(device));
-	const gchar *tmp;
-
-	/* if it's a USB4 type not of host and generation 3; it's Intel */
-	tmp = g_udev_device_get_property(udev_device, "USB4_TYPE");
-	if (g_strcmp0(tmp, "host") != 0 && self->gen == 3)
-		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
-}
-
 static gboolean
 fu_thunderbolt_controller_setup(FuDevice *device, GError **error)
 {
 	FuThunderboltController *self = FU_THUNDERBOLT_CONTROLLER(device);
-	const gchar *tmp = NULL;
 	guint16 did;
 	guint16 vid;
+	g_autofree gchar *attr_nvm_authenticate_on_disconnect = NULL;
+	g_autofree gchar *attr_vendor_name = NULL;
 	g_autoptr(GError) error_gen = NULL;
 	g_autoptr(GError) error_version = NULL;
 
+	/* requires kernel 5.5 or later, non-fatal if not available */
+	self->gen =
+	    fu_thunderbolt_udev_get_attr_uint16(FU_UDEV_DEVICE(self), "generation", &error_gen);
+	if (self->gen == 0)
+		g_debug("unable to read generation: %s", error_gen->message);
+	if (self->gen >= 4)
+		fu_thunderbolt_device_set_retries(FU_THUNDERBOLT_DEVICE(self), 1);
+
+	/* discover retimers */
+	if (self->controller_kind == FU_THUNDERBOLT_CONTROLLER_KIND_HOST &&
+	    fu_device_has_private_flag(FU_DEVICE(self),
+				       FU_THUNDERBOLT_DEVICE_FLAG_FORCE_ENUMERATION)) {
+		g_autoptr(GError) error_local = NULL;
+		if (!fu_thunderbolt_controller_setup_usb4(self, &error_local))
+			g_warning("failed to setup host: %s", error_local->message);
+	}
+
 	/* try to read the version */
 	if (!fu_thunderbolt_device_get_version(FU_THUNDERBOLT_DEVICE(self), &error_version)) {
 		if (self->controller_kind != FU_THUNDERBOLT_CONTROLLER_KIND_HOST &&
@@ -215,36 +238,37 @@ fu_thunderbolt_controller_setup(FuDevice *device, GError **error)
 	}
 
 	/* these may be missing on ICL or later */
-	vid = fu_udev_device_get_vendor(FU_UDEV_DEVICE(self));
+	vid = fu_device_get_vid(device);
 	if (vid == 0x0)
 		g_debug("failed to get Vendor ID");
 
-	did = fu_udev_device_get_model(FU_UDEV_DEVICE(self));
+	did = fu_device_get_pid(device);
 	if (did == 0x0)
 		g_debug("failed to get Device ID");
 
-	/* requires kernel 5.5 or later, non-fatal if not available */
-	self->gen =
-	    fu_thunderbolt_udev_get_attr_uint16(FU_UDEV_DEVICE(self), "generation", &error_gen);
-	if (self->gen == 0)
-		g_debug("unable to read generation: %s", error_gen->message);
-
 	if (self->controller_kind == FU_THUNDERBOLT_CONTROLLER_KIND_HOST) {
 		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_INTERNAL);
 		fu_device_set_summary(device, "Unmatched performance for high-speed I/O");
 	} else {
-		tmp = fu_udev_device_get_sysfs_attr(FU_UDEV_DEVICE(self), "device_name", NULL);
+		g_autofree gchar *attr_device_name =
+		    fu_udev_device_read_sysfs(FU_UDEV_DEVICE(self),
+					      "device_name",
+					      FU_UDEV_DEVICE_ATTR_READ_TIMEOUT_DEFAULT,
+					      NULL);
+		fu_device_set_name(device, attr_device_name);
 	}
 
 	/* set the controller name */
-	if (tmp == NULL)
-		tmp = fu_thunderbolt_controller_kind_to_string(self);
-	fu_device_set_name(device, tmp);
+	if (fu_device_get_name(device) == NULL)
+		fu_thunderbolt_controller_ensure_fallback_name(self);
 
 	/* set vendor string */
-	tmp = fu_udev_device_get_sysfs_attr(FU_UDEV_DEVICE(self), "vendor_name", NULL);
-	if (tmp != NULL)
-		fu_device_set_vendor(device, tmp);
+	attr_vendor_name = fu_udev_device_read_sysfs(FU_UDEV_DEVICE(self),
+						     "vendor_name",
+						     FU_UDEV_DEVICE_ATTR_READ_TIMEOUT_DEFAULT,
+						     NULL);
+	if (attr_vendor_name != NULL)
+		fu_device_set_vendor(device, attr_vendor_name);
 
 	if (fu_device_get_version(device) == NULL)
 		fu_thunderbolt_controller_check_safe_mode(self);
@@ -256,7 +280,6 @@ fu_thunderbolt_controller_setup(FuDevice *device, GError **error)
 		g_autofree gchar *domain_id = NULL;
 		if (fu_thunderbolt_controller_can_update(self)) {
 			const gchar *devpath = fu_udev_device_get_sysfs_path(FU_UDEV_DEVICE(self));
-			g_autofree gchar *vendor_id = NULL;
 			g_autofree gchar *domain = g_path_get_basename(devpath);
 			/* USB4 controllers don't have a concept of legacy vs native
 			 * so don't try to read a native attribute from their NVM */
@@ -273,8 +296,7 @@ fu_thunderbolt_controller_setup(FuDevice *device, GError **error)
 						    (guint)did,
 						    self->is_native ? "-native" : "",
 						    domain);
-			vendor_id = g_strdup_printf("TBT:0x%04X", (guint)vid);
-			fu_device_add_vendor_id(device, vendor_id);
+			fu_device_build_vendor_id_u16(device, "TBT", vid);
 			device_id = g_strdup_printf("TBT-%04x%04x%s",
 						    (guint)vid,
 						    (guint)did,
@@ -300,30 +322,27 @@ fu_thunderbolt_controller_setup(FuDevice *device, GError **error)
 	}
 
 	/* determine if we can update on unplug */
-	if (fu_udev_device_get_sysfs_attr(FU_UDEV_DEVICE(device),
-					  "nvm_authenticate_on_disconnect",
-					  NULL) != NULL) {
+	attr_nvm_authenticate_on_disconnect =
+	    fu_udev_device_read_sysfs(FU_UDEV_DEVICE(device),
+				      "nvm_authenticate_on_disconnect",
+				      FU_UDEV_DEVICE_ATTR_READ_TIMEOUT_DEFAULT,
+				      NULL);
+	if (attr_nvm_authenticate_on_disconnect != NULL) {
 		fu_thunderbolt_device_set_auth_method(FU_THUNDERBOLT_DEVICE(self),
 						      "nvm_authenticate_on_disconnect");
 		/* flushes image */
 		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE);
 		/* forces the device to write to authenticate on disconnect attribute */
-		fu_device_remove_flag(device, FWUPD_DEVICE_FLAG_SKIPS_RESTART);
+		fu_device_remove_private_flag(device, FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART);
 		/* control the order of activation (less relevant; install too though) */
-		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_INSTALL_PARENT_FIRST);
+		fu_device_add_private_flag(device, FU_DEVICE_PRIVATE_FLAG_INSTALL_PARENT_FIRST);
 	} else {
-		fu_device_add_internal_flag(device, FU_DEVICE_INTERNAL_FLAG_REPLUG_MATCH_GUID);
-	}
-	if (self->controller_kind == FU_THUNDERBOLT_CONTROLLER_KIND_HOST &&
-	    fu_device_has_private_flag(FU_DEVICE(self),
-				       FU_THUNDERBOLT_DEVICE_FLAG_FORCE_ENUMERATION)) {
-		g_autoptr(GError) error_local = NULL;
-		if (!fu_thunderbolt_controller_setup_usb4(self, &error_local))
-			g_warning("failed to setup host: %s", error_local->message);
+		fu_device_add_private_flag(device, FU_DEVICE_PRIVATE_FLAG_REPLUG_MATCH_GUID);
 	}
 
 	/* set up signed payload attribute */
-	fu_thunderbolt_controller_set_signed(device);
+	if (self->controller_kind == FU_THUNDERBOLT_CONTROLLER_KIND_HOST && self->gen >= 3)
+		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
 
 	/* success */
 	return TRUE;
@@ -350,8 +369,7 @@ fu_thunderbolt_controller_init(FuThunderboltController *self)
 {
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	fu_device_register_private_flag(FU_DEVICE(self),
-					FU_THUNDERBOLT_DEVICE_FLAG_FORCE_ENUMERATION,
-					"force-enumeration");
+					FU_THUNDERBOLT_DEVICE_FLAG_FORCE_ENUMERATION);
 }
 
 static void
@@ -369,10 +387,10 @@ static void
 fu_thunderbolt_controller_class_init(FuThunderboltControllerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 	object_class->finalize = fu_thunderbolt_controller_finalize;
-	klass_device->setup = fu_thunderbolt_controller_setup;
-	klass_device->probe = fu_thunderbolt_controller_probe;
-	klass_device->to_string = fu_thunderbolt_controller_to_string;
-	klass_device->write_firmware = fu_thunderbolt_controller_write_firmware;
+	device_class->setup = fu_thunderbolt_controller_setup;
+	device_class->probe = fu_thunderbolt_controller_probe;
+	device_class->to_string = fu_thunderbolt_controller_to_string;
+	device_class->write_firmware = fu_thunderbolt_controller_write_firmware;
 }
