@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright 2020 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -12,23 +12,23 @@
 #include "fu-uefi-pk-device.h"
 
 struct _FuUefiPkDevice {
-	FuDevice parent_instance;
+	FuUefiDevice parent_instance;
 	gboolean has_pk_test_key;
 };
 
-G_DEFINE_TYPE(FuUefiPkDevice, fu_uefi_pk_device, FU_TYPE_DEVICE)
+G_DEFINE_TYPE(FuUefiPkDevice, fu_uefi_pk_device, FU_TYPE_UEFI_DEVICE)
 
 static void
 fu_uefi_pk_device_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuUefiPkDevice *self = FU_UEFI_PK_DEVICE(device);
-	fu_string_append_kb(str, idt, "HasPkTestKey", self->has_pk_test_key);
+	fwupd_codec_string_append_bool(str, idt, "HasPkTestKey", self->has_pk_test_key);
 }
 
 #define FU_UEFI_PK_CHECKSUM_AMI_TEST_KEY "a773113bafaf5129aa83fd0912e95da4fa555f91"
 
 static void
-_gnutls_datum_deinit(gnutls_datum_t *d)
+fu_uefi_pk_device_gnutls_datum_deinit(gnutls_datum_t *d)
 {
 	gnutls_free(d->data);
 	gnutls_free(d);
@@ -36,7 +36,7 @@ _gnutls_datum_deinit(gnutls_datum_t *d)
 
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wunused-function"
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(gnutls_datum_t, _gnutls_datum_deinit)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(gnutls_datum_t, fu_uefi_pk_device_gnutls_datum_deinit)
 G_DEFINE_AUTO_CLEANUP_FREE_FUNC(gnutls_x509_crt_t, gnutls_x509_crt_deinit, NULL)
 #pragma clang diagnostic pop
 
@@ -95,8 +95,8 @@ fu_uefi_pk_device_parse_signature(FuUefiPkDevice *self, FuEfiSignature *sig, GEr
 	rc = gnutls_x509_crt_init(&crt);
 	if (rc < 0) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "crt_init: %s [%i]",
 			    gnutls_strerror(rc),
 			    rc);
@@ -112,8 +112,8 @@ fu_uefi_pk_device_parse_signature(FuUefiPkDevice *self, FuEfiSignature *sig, GEr
 	rc = gnutls_x509_crt_import(crt, &d, GNUTLS_X509_FMT_DER);
 	if (rc < 0) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "crt_import: %s [%i]",
 			    gnutls_strerror(rc),
 			    rc);
@@ -143,8 +143,8 @@ fu_uefi_pk_device_parse_signature(FuUefiPkDevice *self, FuEfiSignature *sig, GEr
 	rc = gnutls_x509_crt_get_key_id(crt, 0, key_id, &key_idsz);
 	if (rc < 0) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "failed to get key ID: %s [%i]",
 			    gnutls_strerror(rc),
 			    rc);
@@ -153,8 +153,8 @@ fu_uefi_pk_device_parse_signature(FuUefiPkDevice *self, FuEfiSignature *sig, GEr
 	key_idstr = g_compute_checksum_for_data(G_CHECKSUM_SHA1, key_id, key_idsz);
 	if (key_idstr == NULL) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_DATA,
 			    "failed to calculate key ID for 0x%x bytes",
 			    (guint)key_idsz);
 		return FALSE;
@@ -170,14 +170,12 @@ fu_uefi_pk_device_probe(FuDevice *device, GError **error)
 {
 	FuUefiPkDevice *self = FU_UEFI_PK_DEVICE(device);
 	g_autoptr(FuFirmware) img = NULL;
-	g_autoptr(FuFirmware) pk = fu_efi_signature_list_new();
-	g_autoptr(GBytes) pk_blob = NULL;
+	g_autoptr(FuFirmware) pk = NULL;
+	g_autoptr(FuProgress) progress = fu_progress_new(G_STRLOC);
 	g_autoptr(GPtrArray) sigs = NULL;
 
-	pk_blob = fu_efivar_get_data_bytes(FU_EFIVAR_GUID_EFI_GLOBAL, "PK", NULL, error);
-	if (pk_blob == NULL)
-		return FALSE;
-	if (!fu_firmware_parse(pk, pk_blob, FU_FIRMWARE_FLAG_NONE, error)) {
+	pk = fu_device_read_firmware(device, progress, error);
+	if (pk == NULL) {
 		g_prefix_error(error, "failed to parse PK: ");
 		return FALSE;
 	}
@@ -228,19 +226,14 @@ fu_uefi_pk_device_init(FuUefiPkDevice *self)
 	fu_device_set_physical_id(FU_DEVICE(self), "pk");
 	fu_device_set_name(FU_DEVICE(self), "UEFI Platform Key");
 	fu_device_add_parent_guid(FU_DEVICE(self), "main-system-firmware");
+	fu_device_set_firmware_gtype(FU_DEVICE(self), FU_TYPE_EFI_SIGNATURE_LIST);
 }
 
 static void
 fu_uefi_pk_device_class_init(FuUefiPkDeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
-	klass_device->to_string = fu_uefi_pk_device_to_string;
-	klass_device->add_security_attrs = fu_uefi_pk_device_add_security_attrs;
-	klass_device->probe = fu_uefi_pk_device_probe;
-}
-
-FuUefiPkDevice *
-fu_uefi_pk_device_new(FuContext *ctx)
-{
-	return g_object_new(FU_TYPE_UEFI_PK_DEVICE, "context", ctx, NULL);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	device_class->to_string = fu_uefi_pk_device_to_string;
+	device_class->add_security_attrs = fu_uefi_pk_device_add_security_attrs;
+	device_class->probe = fu_uefi_pk_device_probe;
 }
