@@ -1,5 +1,5 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #[derive(ToString)]
 enum FuUsiDockSpiState {
@@ -17,6 +17,7 @@ enum FuUsiDockSpiState {
     FlashNotFound,
 }
 
+#[derive(ToString)]
 enum FuUsiDockFirmwareIdx {
     None  = 0x00,
     DmcPd = 0x01,
@@ -70,7 +71,8 @@ enum FuUsiDockSpiCmd {
     ErrorEnd = 0x09,
 }
 
-#[derive(New)]
+#[derive(New, Default)]
+#[repr(C, packed)]
 struct FuStructUsiDockHidReq {
     id: u8 == 2,
     length: u8,
@@ -78,7 +80,8 @@ struct FuStructUsiDockHidReq {
     tag3: FuUsiDockTag2,
 }
 
-#[derive(New)]
+#[derive(New, Default)]
+#[repr(C, packed)]
 struct FuStructUsiDockMcuCmdReq {
     id: u8 == 2,
     length: u8,
@@ -88,7 +91,8 @@ struct FuStructUsiDockMcuCmdReq {
     tag3: FuUsiDockTag2,
 }
 
-#[derive(Parse)]
+#[derive(Parse, Default)]
+#[repr(C, packed)]
 struct FuStructUsiDockMcuCmdRes {
     id: u8 == 2,
     cmd1: FuUsiDockMcuCmd,
@@ -99,6 +103,7 @@ struct FuStructUsiDockMcuCmdRes {
     tag3: FuUsiDockTag2,
 }
 
+#[repr(C, packed)]
 struct FuStructUsiDockIspVersion {
     dmc: [u8; 5],
     pd: [u8; 5],
