@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2017 VIA Corporation
- * Copyright (C) 2019 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 VIA Corporation
+ * Copyright 2019 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -150,7 +150,7 @@ fu_vli_device_spi_wait_finish(FuVliDevice *self, GError **error)
 		}
 		fu_device_sleep(FU_DEVICE(self), 500); /* ms */
 	}
-	g_set_error(error, G_IO_ERROR, G_IO_ERROR_FAILED, "failed to wait for SPI");
+	g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_TIMED_OUT, "failed to wait for SPI");
 	return FALSE;
 }
 
@@ -191,8 +191,8 @@ fu_vli_device_spi_erase_sector(FuVliDevice *self, guint32 addr, GError **error)
 		for (guint i = 0; i < sizeof(buf); i++) {
 			if (buf[i] != 0xff) {
 				g_set_error(error,
-					    G_IO_ERROR,
-					    G_IO_ERROR_FAILED,
+					    FWUPD_ERROR,
+					    FWUPD_ERROR_READ,
 					    "failed to check blank @0x%x",
 					    addr + offset + i);
 				return FALSE;
@@ -227,7 +227,7 @@ fu_vli_device_spi_read(FuVliDevice *self,
 						  error)) {
 			g_prefix_error(error,
 				       "SPI data read failed @0x%x: ",
-				       fu_chunk_get_address(chk));
+				       (guint)fu_chunk_get_address(chk));
 			return NULL;
 		}
 		fu_progress_step_done(progress);
@@ -248,8 +248,8 @@ fu_vli_device_spi_write_block(FuVliDevice *self,
 	/* sanity check */
 	if (bufsz > FU_VLI_DEVICE_TXSIZE) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_FAILED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "cannot write 0x%x in one block",
 			    (guint)bufsz);
 		return FALSE;
@@ -360,8 +360,8 @@ fu_vli_device_spi_erase_all(FuVliDevice *self, FuProgress *progress, GError **er
 		for (guint i = 0; i < sizeof(buf); i++) {
 			if (buf[i] != 0xff) {
 				g_set_error(error,
-					    G_IO_ERROR,
-					    G_IO_ERROR_FAILED,
+					    FWUPD_ERROR,
+					    FWUPD_ERROR_READ,
 					    "failed to verify erase @0x%x: ",
 					    addr);
 				return FALSE;
@@ -388,13 +388,13 @@ fu_vli_device_spi_erase(FuVliDevice *self,
 	fu_progress_set_steps(progress, chunks->len);
 	for (guint i = 0; i < chunks->len; i++) {
 		FuChunk *chk = g_ptr_array_index(chunks, i);
-		g_debug("erasing @0x%x", fu_chunk_get_address(chk));
+		g_debug("erasing @0x%x", (guint)fu_chunk_get_address(chk));
 		if (!fu_vli_device_spi_erase_sector(FU_VLI_DEVICE(self),
 						    fu_chunk_get_address(chk),
 						    error)) {
 			g_prefix_error(error,
 				       "failed to erase FW sector @0x%x: ",
-				       fu_chunk_get_address(chk));
+				       (guint)fu_chunk_get_address(chk));
 			return FALSE;
 		}
 		fu_progress_step_done(progress);
@@ -522,16 +522,15 @@ fu_vli_device_to_string(FuDevice *device, guint idt, GString *str)
 	FuVliDevice *self = FU_VLI_DEVICE(device);
 	FuVliDevicePrivate *priv = GET_PRIVATE(self);
 
-	/* parent */
-	FU_DEVICE_CLASS(fu_vli_device_parent_class)->to_string(device, idt, str);
-
-	if (priv->kind != FU_VLI_DEVICE_KIND_UNKNOWN) {
-		fu_string_append(str, idt, "DeviceKind", fu_vli_device_kind_to_string(priv->kind));
-	}
-	fu_string_append_kb(str, idt, "SpiAutoDetect", priv->spi_auto_detect);
+	if (priv->kind != FU_VLI_DEVICE_KIND_UNKNOWN)
+		fwupd_codec_string_append(str,
+					  idt,
+					  "DeviceKind",
+					  fu_vli_device_kind_to_string(priv->kind));
+	fwupd_codec_string_append_bool(str, idt, "SpiAutoDetect", priv->spi_auto_detect);
 	if (priv->flash_id != 0x0) {
 		g_autofree gchar *tmp = fu_vli_device_get_flash_id_str(self);
-		fu_string_append(str, idt, "FlashId", tmp);
+		fwupd_codec_string_append(str, idt, "FlashId", tmp);
 	}
 	fu_device_add_string(FU_DEVICE(priv->cfi_device), idt + 1, str);
 }
@@ -540,25 +539,24 @@ static gboolean
 fu_vli_device_spi_read_flash_id(FuVliDevice *self, GError **error)
 {
 	FuVliDevicePrivate *priv = GET_PRIVATE(self);
-	GUsbDevice *usb_device = fu_usb_device_get_dev(FU_USB_DEVICE(self));
 	guint8 buf[4] = {0x0};
 	guint8 spi_cmd = 0x0;
 
 	if (!fu_cfi_device_get_cmd(priv->cfi_device, FU_CFI_DEVICE_CMD_READ_ID, &spi_cmd, error))
 		return FALSE;
-	if (!g_usb_device_control_transfer(usb_device,
-					   G_USB_DEVICE_DIRECTION_DEVICE_TO_HOST,
-					   G_USB_DEVICE_REQUEST_TYPE_VENDOR,
-					   G_USB_DEVICE_RECIPIENT_DEVICE,
-					   0xc0 | (priv->spi_cmd_read_id_sz * 2),
-					   spi_cmd,
-					   0x0000,
-					   buf,
-					   sizeof(buf),
-					   NULL,
-					   FU_VLI_DEVICE_TIMEOUT,
-					   NULL,
-					   error)) {
+	if (!fu_usb_device_control_transfer(FU_USB_DEVICE(self),
+					    FU_USB_DIRECTION_DEVICE_TO_HOST,
+					    FU_USB_REQUEST_TYPE_VENDOR,
+					    FU_USB_RECIPIENT_DEVICE,
+					    0xc0 | (priv->spi_cmd_read_id_sz * 2),
+					    spi_cmd,
+					    0x0000,
+					    buf,
+					    sizeof(buf),
+					    NULL,
+					    FU_VLI_DEVICE_TIMEOUT,
+					    NULL,
+					    error)) {
 		g_prefix_error(error, "failed to read chip ID: ");
 		return FALSE;
 	}
@@ -642,13 +640,13 @@ fu_vli_device_set_quirk_kv(FuDevice *device, const gchar *key, const gchar *valu
 	guint64 tmp = 0;
 
 	if (g_strcmp0(key, "CfiDeviceCmdReadIdSz") == 0) {
-		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT8, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT8, FU_INTEGER_BASE_AUTO, error))
 			return FALSE;
 		priv->spi_cmd_read_id_sz = tmp;
 		return TRUE;
 	}
 	if (g_strcmp0(key, "VliSpiAutoDetect") == 0) {
-		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT8, error))
+		if (!fu_strtoull(value, &tmp, 0, G_MAXUINT8, FU_INTEGER_BASE_AUTO, error))
 			return FALSE;
 		priv->spi_auto_detect = tmp > 0;
 		return TRUE;
@@ -658,8 +656,8 @@ fu_vli_device_set_quirk_kv(FuDevice *device, const gchar *key, const gchar *valu
 		device_kind = fu_vli_device_kind_from_string(value);
 		if (device_kind == FU_VLI_DEVICE_KIND_UNKNOWN) {
 			g_set_error(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_NOT_SUPPORTED,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_NOT_SUPPORTED,
 				    "VliDeviceKind %s is not supported",
 				    value);
 			return FALSE;
@@ -724,9 +722,9 @@ fu_vli_device_init(FuVliDevice *self)
 	FuVliDevicePrivate *priv = GET_PRIVATE(self);
 	priv->spi_cmd_read_id_sz = 2;
 	priv->spi_auto_detect = TRUE;
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_ADD_COUNTERPART_GUIDS);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_NO_SERIAL_NUMBER);
-	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_ADD_INSTANCE_ID_REV);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_ADD_COUNTERPART_GUIDS);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_NO_SERIAL_NUMBER);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_ADD_INSTANCE_ID_REV);
 }
 
 static void
@@ -741,7 +739,7 @@ fu_vli_device_finalize(GObject *obj)
 static void
 fu_vli_device_class_init(FuVliDeviceClass *klass)
 {
-	FuDeviceClass *klass_device = FU_DEVICE_CLASS(klass);
+	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	GParamSpec *pspec;
 
@@ -765,8 +763,8 @@ fu_vli_device_class_init(FuVliDeviceClass *klass)
 				  G_PARAM_READWRITE | G_PARAM_STATIC_NAME);
 	g_object_class_install_property(object_class, PROP_KIND, pspec);
 
-	klass_device->to_string = fu_vli_device_to_string;
-	klass_device->set_quirk_kv = fu_vli_device_set_quirk_kv;
-	klass_device->setup = fu_vli_device_setup;
-	klass_device->report_metadata_pre = fu_vli_device_report_metadata_pre;
+	device_class->to_string = fu_vli_device_to_string;
+	device_class->set_quirk_kv = fu_vli_device_set_quirk_kv;
+	device_class->setup = fu_vli_device_setup;
+	device_class->report_metadata_pre = fu_vli_device_report_metadata_pre;
 }
