@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2017 VIA Corporation
- * Copyright (C) 2019 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 VIA Corporation
+ * Copyright 2019 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -43,24 +43,21 @@ fu_vli_usbhub_firmware_export(FuFirmware *firmware, FuFirmwareExportFlags flags,
 
 static gboolean
 fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
-			     GBytes *fw,
-			     gsize offset,
+			     GInputStream *stream,
 			     FwupdInstallFlags flags,
 			     GError **error)
 {
 	FuVliUsbhubFirmware *self = FU_VLI_USBHUB_FIRMWARE(firmware);
-	gsize bufsz = 0;
 	guint16 adr_ofs = 0;
 	guint16 version = 0x0;
 	guint32 adr_ofs32 = 0;
 	guint8 tmp = 0x0;
 	guint8 fwtype = 0x0;
 	guint8 strapping1;
-	const guint8 *buf = g_bytes_get_data(fw, &bufsz);
 	g_autoptr(GByteArray) st = NULL;
 
 	/* map into header */
-	st = fu_struct_vli_usbhub_hdr_parse(buf, bufsz, 0x0, error);
+	st = fu_struct_vli_usbhub_hdr_parse_stream(stream, 0x0, error);
 	if (st == NULL) {
 		g_prefix_error(error, "failed to read header: ");
 		return FALSE;
@@ -72,13 +69,13 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 	switch (self->dev_id) {
 	case 0x0d12:
 		/* VL81x */
-		if (!fu_memread_uint16_safe(buf, bufsz, 0x1f4c, &version, G_LITTLE_ENDIAN, error)) {
+		if (!fu_input_stream_read_u16(stream, 0x1f4c, &version, G_LITTLE_ENDIAN, error)) {
 			g_prefix_error(error, "failed to get version: ");
 			return FALSE;
 		}
 		version |= (strapping1 >> 4) & 0x07;
 		if ((version & 0x0f) == 0x04) {
-			if (!fu_memread_uint8_safe(buf, bufsz, 0x700d, &tmp, error)) {
+			if (!fu_input_stream_read_u8(stream, 0x700d, &tmp, error)) {
 				g_prefix_error(error, "failed to get version increment: ");
 				return FALSE;
 			}
@@ -88,7 +85,7 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 		break;
 	case 0x0507:
 		/* VL210 */
-		if (!fu_memread_uint16_safe(buf, bufsz, 0x8f0c, &version, G_LITTLE_ENDIAN, error)) {
+		if (!fu_input_stream_read_u16(stream, 0x8f0c, &version, G_LITTLE_ENDIAN, error)) {
 			g_prefix_error(error, "failed to get version: ");
 			return FALSE;
 		}
@@ -98,12 +95,7 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 		break;
 	case 0x0566:
 		/* U4ID_Address_In_FW_Zone */
-		if (!fu_memread_uint24_safe(buf,
-					    bufsz,
-					    0x3F80,
-					    &adr_ofs32,
-					    G_LITTLE_ENDIAN,
-					    error)) {
+		if (!fu_input_stream_read_u24(stream, 0x3F80, &adr_ofs32, G_LITTLE_ENDIAN, error)) {
 			g_prefix_error(error, "failed to get offset addr: ");
 			return FALSE;
 		}
@@ -115,12 +107,11 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 				    adr_ofs32);
 			return FALSE;
 		}
-		if (!fu_memread_uint16_safe(buf,
-					    bufsz,
-					    adr_ofs32 - 0x20000 + 0x2000 + 4,
-					    &version,
-					    G_LITTLE_ENDIAN,
-					    error)) {
+		if (!fu_input_stream_read_u16(stream,
+					      adr_ofs32 - 0x20000 + 0x2000 + 4,
+					      &version,
+					      G_LITTLE_ENDIAN,
+					      error)) {
 			g_prefix_error(error, "failed to get offset version: ");
 			return FALSE;
 		}
@@ -128,16 +119,15 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 		break;
 	default:
 		/* U3ID_Address_In_FW_Zone */
-		if (!fu_memread_uint16_safe(buf, bufsz, 0x8000, &adr_ofs, G_BIG_ENDIAN, error)) {
+		if (!fu_input_stream_read_u16(stream, 0x8000, &adr_ofs, G_BIG_ENDIAN, error)) {
 			g_prefix_error(error, "failed to get offset addr: ");
 			return FALSE;
 		}
-		if (!fu_memread_uint16_safe(buf,
-					    bufsz,
-					    adr_ofs + 0x2000 + 0x04, /* U3-M? */
-					    &version,
-					    G_LITTLE_ENDIAN,
-					    error)) {
+		if (!fu_input_stream_read_u16(stream,
+					      adr_ofs + 0x2000 + 0x04, /* U3-M? */
+					      &version,
+					      G_LITTLE_ENDIAN,
+					      error)) {
 			g_prefix_error(error, "failed to get offset version: ");
 			return FALSE;
 		}
@@ -145,12 +135,8 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 	}
 
 	/* version is set */
-	if (version != 0x0) {
-		g_autofree gchar *version_str = NULL;
-		version_str = fu_version_from_uint16(version, FWUPD_VERSION_FORMAT_BCD);
-		fu_firmware_set_version(firmware, version_str);
+	if (version != 0x0)
 		fu_firmware_set_version_raw(firmware, version);
-	}
 
 	/* get device type from firmware image */
 	switch (self->dev_id) {
@@ -159,21 +145,19 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 		guint16 binver2 = 0x0;
 		guint16 usb2_fw_addr = fu_struct_vli_usbhub_hdr_get_usb2_fw_addr(st) + 0x1ff1;
 		guint16 usb3_fw_addr = fu_struct_vli_usbhub_hdr_get_usb3_fw_addr(st) + 0x1ffa;
-		if (!fu_memread_uint16_safe(buf,
-					    bufsz,
-					    usb2_fw_addr,
-					    &binver1,
-					    G_LITTLE_ENDIAN,
-					    error)) {
+		if (!fu_input_stream_read_u16(stream,
+					      usb2_fw_addr,
+					      &binver1,
+					      G_LITTLE_ENDIAN,
+					      error)) {
 			g_prefix_error(error, "failed to get binver1: ");
 			return FALSE;
 		}
-		if (!fu_memread_uint16_safe(buf,
-					    bufsz,
-					    usb3_fw_addr,
-					    &binver2,
-					    G_LITTLE_ENDIAN,
-					    error)) {
+		if (!fu_input_stream_read_u16(stream,
+					      usb3_fw_addr,
+					      &binver2,
+					      G_LITTLE_ENDIAN,
+					      error)) {
 			g_prefix_error(error, "failed to get binver2: ");
 			return FALSE;
 		}
@@ -215,18 +199,18 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 		break;
 	case 0x0518:
 		/* VL819~VL822 == VT3518 */
-		if (!fu_memread_uint8_safe(buf, bufsz, 0x8021, &tmp, error)) {
+		if (!fu_input_stream_read_u8(stream, 0x8021, &tmp, error)) {
 			g_prefix_error(error, "failed to get 820/822 byte: ");
 			return FALSE;
 		}
 		/* Q5/Q7/Q8 requires searching two addresses for offset value */
-		if (!fu_memread_uint16_safe(buf, bufsz, 0x8018, &adr_ofs, G_BIG_ENDIAN, error)) {
+		if (!fu_input_stream_read_u16(stream, 0x8018, &adr_ofs, G_BIG_ENDIAN, error)) {
 			g_prefix_error(error, "failed to get Q7/Q8 offset mapping: ");
 			return FALSE;
 		}
 		/* VL819, VL821, VL822 */
 		if (tmp == 0xF0) {
-			if (!fu_memread_uint8_safe(buf, bufsz, adr_ofs + 0x2000, &tmp, error)) {
+			if (!fu_input_stream_read_u8(stream, adr_ofs + 0x2000, &tmp, error)) {
 				g_prefix_error(error, "failed to get offset version: ");
 				return FALSE;
 			}
@@ -266,7 +250,7 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 		} else if (tmp == 0xC0 || tmp == 0xC1) {
 			self->device_kind = FU_VLI_DEVICE_KIND_VL822C0;
 		} else {
-			if (!fu_memread_uint8_safe(buf, bufsz, 0xf000, &tmp, error)) {
+			if (!fu_input_stream_read_u8(stream, 0xf000, &tmp, error)) {
 				g_prefix_error(error, "failed to get Q7/Q8 difference: ");
 				return FALSE;
 			}
@@ -300,12 +284,11 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 		/* VL830 VL832 = VT3566 */
 		guint32 binveraddr = 0;
 		guint8 binver = 0;
-		if (!fu_memread_uint24_safe(buf,
-					    bufsz,
-					    0x3FBC,
-					    &binveraddr,
-					    G_LITTLE_ENDIAN,
-					    error)) {
+		if (!fu_input_stream_read_u24(stream,
+					      0x3FBC,
+					      &binveraddr,
+					      G_LITTLE_ENDIAN,
+					      error)) {
 			g_prefix_error(error, "failed to get binveraddr: ");
 			return FALSE;
 		}
@@ -313,14 +296,13 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 			g_set_error_literal(error,
 					    FWUPD_ERROR,
 					    FWUPD_ERROR_INVALID_FILE,
-					    "buf was too small");
+					    "stream was too small");
 			return FALSE;
 		}
-		if (!fu_memread_uint8_safe(buf,
-					   bufsz,
-					   binveraddr - 0x20000 + 0x2000,
-					   &binver,
-					   error)) {
+		if (!fu_input_stream_read_u8(stream,
+					     binveraddr - 0x20000 + 0x2000,
+					     &binver,
+					     error)) {
 			g_prefix_error(error, "failed to get binver2: ");
 			return FALSE;
 		}
@@ -339,8 +321,8 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 	/* device not supported */
 	if (self->device_kind == FU_VLI_DEVICE_KIND_UNKNOWN) {
 		g_set_error_literal(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_NOT_SUPPORTED,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_NOT_SUPPORTED,
 				    "device kind unknown");
 		return FALSE;
 	}
@@ -349,17 +331,25 @@ fu_vli_usbhub_firmware_parse(FuFirmware *firmware,
 	return TRUE;
 }
 
+static gchar *
+fu_vli_usbhub_firmware_convert_version(FuFirmware *firmware, guint64 version_raw)
+{
+	return fu_version_from_uint16(version_raw, fu_firmware_get_version_format(firmware));
+}
+
 static void
 fu_vli_usbhub_firmware_init(FuVliUsbhubFirmware *self)
 {
+	fu_firmware_set_version_format(FU_FIRMWARE(self), FWUPD_VERSION_FORMAT_BCD);
 }
 
 static void
 fu_vli_usbhub_firmware_class_init(FuVliUsbhubFirmwareClass *klass)
 {
-	FuFirmwareClass *klass_firmware = FU_FIRMWARE_CLASS(klass);
-	klass_firmware->parse = fu_vli_usbhub_firmware_parse;
-	klass_firmware->export = fu_vli_usbhub_firmware_export;
+	FuFirmwareClass *firmware_class = FU_FIRMWARE_CLASS(klass);
+	firmware_class->convert_version = fu_vli_usbhub_firmware_convert_version;
+	firmware_class->parse = fu_vli_usbhub_firmware_parse;
+	firmware_class->export = fu_vli_usbhub_firmware_export;
 }
 
 FuFirmware *
