@@ -1,13 +1,16 @@
-// Copyright (C) 2023 Richard Hughes <richard@hughsie.com>
-// SPDX-License-Identifier: LGPL-2.1+
+// Copyright 2023 Richard Hughes <richard@hughsie.com>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
-#[derive(Parse)]
+#[derive(Parse, ParseStream)]
+#[repr(C, packed)]
 struct FuStructVliPdHdr {
     fwver: u32be,
     vid: u16le,
     pid: u16le,
 }
-#[derive(New, Parse, ToString)]
+
+#[derive(New, Parse, ParseStream, ToString)]
+#[repr(C, packed)]
 struct FuStructVliUsbhubHdr {
     dev_id: u16be,
     strapping1: u8,
@@ -27,6 +30,7 @@ struct FuStructVliUsbhubHdr {
     variant: u8,
     checksum: u8,
 }
+
 #[derive(ToString, FromString)]
 enum FuVliDeviceKind {
     Unknown = 0x0,
