@@ -81,7 +81,7 @@ fu_wac_module_sub_cpu_parse_chunks(FuSrecFirmware *srec_firmware, guint32 *data_
 {
 	GPtrArray *chunks = g_ptr_array_new_with_free_func(g_free);
 	guint record_num = 0;
-	GPtrArray *records = fu_srec_firmware_get_records(srec_firmware);
+	g_autoptr(GPtrArray) records = fu_srec_firmware_get_records(srec_firmware);
 
 	*data_len = 0;
 	while (record_num < records->len) {
@@ -207,16 +207,17 @@ fu_wac_module_sub_cpu_write_firmware(FuDevice *device,
 
 static FuFirmware *
 fu_wac_module_sub_cpu_prepare_firmware(FuDevice *self,
-				       GBytes *fw,
+				       GInputStream *stream,
+				       FuProgress *progress,
 				       FwupdInstallFlags flags,
 				       GError **error)
 {
 	FuFirmware *firmware_srec = fu_srec_firmware_new();
-	if (!fu_firmware_parse_full(firmware_srec,
-				    fw,
-				    0,
-				    flags | FWUPD_INSTALL_FLAG_NO_SEARCH,
-				    error)) {
+	if (!fu_firmware_parse_stream(firmware_srec,
+				      stream,
+				      0,
+				      flags | FWUPD_INSTALL_FLAG_NO_SEARCH,
+				      error)) {
 		g_prefix_error(error, "wacom sub_cpu failed to parse firmware: ");
 		return NULL;
 	}
