@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #pragma once
@@ -30,16 +30,18 @@ typedef enum {
 FuCabinet *
 fu_cabinet_new(void);
 void
-fu_cabinet_set_jcat_context(FuCabinet *self, JcatContext *jcat_context);
+fu_cabinet_set_jcat_context(FuCabinet *self, JcatContext *jcat_context) G_GNUC_NON_NULL(1);
 gboolean
 fu_cabinet_sign(FuCabinet *self,
 		GBytes *cert,
 		GBytes *privkey,
 		FuCabinetSignFlags flags,
-		GError **error) G_GNUC_WARN_UNUSED_RESULT;
+		GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1, 2, 3);
 void
-fu_cabinet_add_file(FuCabinet *self, const gchar *basename, GBytes *data);
-GBytes *
-fu_cabinet_get_file(FuCabinet *self, const gchar *basename, GError **error);
+fu_cabinet_add_file(FuCabinet *self, const gchar *basename, GBytes *data) G_GNUC_NON_NULL(1, 2, 3);
 XbSilo *
-fu_cabinet_get_silo(FuCabinet *self, GError **error);
+fu_cabinet_get_silo(FuCabinet *self, GError **error) G_GNUC_NON_NULL(1);
+GPtrArray *
+fu_cabinet_get_components(FuCabinet *self, GError **error) G_GNUC_NON_NULL(1);
+XbNode *
+fu_cabinet_get_component(FuCabinet *self, const gchar *id, GError **error) G_GNUC_NON_NULL(1);
