@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuProgressBar"
@@ -48,16 +48,16 @@ fu_console_setup(FuConsole *self, GError **error)
 	hOut = GetStdHandle(STD_OUTPUT_HANDLE);
 	if (hOut == INVALID_HANDLE_VALUE) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "failed to get stdout [%u]",
 			    (guint)GetLastError());
 		return FALSE;
 	}
 	if (!GetConsoleMode(hOut, &dwMode)) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "failed to get mode [%u]",
 			    (guint)GetLastError());
 		return FALSE;
@@ -65,31 +65,31 @@ fu_console_setup(FuConsole *self, GError **error)
 	dwMode |= ENABLE_VIRTUAL_TERMINAL_PROCESSING;
 	if (!SetConsoleMode(hOut, dwMode)) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "failed to set mode [%u]",
 			    (guint)GetLastError());
 		return FALSE;
 	}
 	if (!SetConsoleOutputCP(CP_UTF8)) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "failed to set output UTF-8 [%u]",
 			    (guint)GetLastError());
 		return FALSE;
 	}
 	if (!SetConsoleCP(CP_UTF8)) {
 		g_set_error(error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_NOT_SUPPORTED,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
 			    "failed to set UTF-8 [%u]",
 			    (guint)GetLastError());
 		return FALSE;
 	}
 #else
-	if (isatty(fileno(stdout)) == 0) {
-		g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "not a TTY");
+	if (isatty(fileno(stdin)) == 0) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "not a TTY");
 		return FALSE;
 	}
 #endif
@@ -204,9 +204,8 @@ fu_console_input_bool(FuConsole *self, gboolean def, const gchar *format, ...)
 		if (g_strcmp0(buffer, "N\n") == 0)
 			return FALSE;
 
-		/* TRANSLATORS: the user isn't reading the question -- please don't translate
-		 * 'Y' or 'N' as these are hardcoded */
-		fu_console_print_literal(self, _("Please enter either Y or N: "));
+		/* TRANSLATORS: the user isn't reading the question -- %1 is 'Y' and %2 is 'N' */
+		fu_console_print(self, _("Please enter either %s or %s: "), "Y", "N");
 	} while (TRUE);
 	return FALSE;
 }
