@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
+ * Copyright 2017 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuEngineConfig"
@@ -14,7 +14,6 @@
 
 struct _FuEngineConfig {
 	FuConfig parent_instance;
-	GHashTable *os_release;	      /* (element-type utf-8 utf-8) */
 	GPtrArray *disabled_devices;  /* (element-type utf-8) */
 	GPtrArray *disabled_plugins;  /* (element-type utf-8) */
 	GPtrArray *approved_firmware; /* (element-type utf-8) */
@@ -36,8 +35,8 @@ fu_engine_config_report_from_flags(FwupdReport *report, const gchar *flags_str,
 		FwupdReportFlags flag = fwupd_report_flag_from_string(flags_strv[i]);
 		if (flag == FWUPD_REPORT_FLAG_UNKNOWN) {
 			g_set_error(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "report flag '%s' unknown",
 				    flags_strv[i]);
 			return FALSE;
@@ -54,26 +53,31 @@ fu_engine_config_report_from_spec(FuEngineConfig *self, const gchar *report_spec
 	g_autoptr(FwupdReport) report = fwupd_report_new();
 
 	for (guint i = 0; parts[i] != NULL; i++) {
-		const gchar *value = NULL;
+		g_autofree gchar *value = NULL;
 		g_auto(GStrv) kv = g_strsplit(parts[i], "=", 2);
 		if (g_strv_length(kv) != 2) {
 			g_set_error(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "failed to parse report specifier key=value %s",
 				    parts[i]);
 			return NULL;
 		}
-		if (self->os_release != NULL && g_str_has_prefix(kv[1], "$"))
-			value = g_hash_table_lookup(self->os_release, kv[1] + 1);
+		if (g_str_has_prefix(kv[1], "$"))
+			value = g_get_os_info(kv[1] + 1);
 		if (value == NULL)
-			value = kv[1];
+			value = g_strdup(kv[1]);
 		if (g_strcmp0(kv[0], "VendorId") == 0) {
 			guint64 tmp = 0;
 			if (g_strcmp0(value, "$OEM") == 0) {
 				fwupd_report_add_flag(report, FWUPD_REPORT_FLAG_FROM_OEM);
 			} else {
-				if (!fu_strtoull(value, &tmp, 0, G_MAXUINT32, error)) {
+				if (!fu_strtoull(value,
+						 &tmp,
+						 0,
+						 G_MAXUINT32,
+						 FU_INTEGER_BASE_AUTO,
+						 error)) {
 					g_prefix_error(error, "failed to parse '%s': ", value);
 					return NULL;
 				}
@@ -92,8 +96,8 @@ fu_engine_config_report_from_spec(FuEngineConfig *self, const gchar *report_spec
 				return NULL;
 		} else {
 			g_set_error(error,
-				    G_IO_ERROR,
-				    G_IO_ERROR_INVALID_DATA,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
 				    "failed to parse report specifier key %s",
 				    kv[0]);
 			return NULL;
@@ -109,7 +113,7 @@ fu_engine_config_reload(FuEngineConfig *self)
 {
 	g_auto(GStrv) approved_firmware = NULL;
 	g_auto(GStrv) blocked_firmware = NULL;
-	g_auto(GStrv) devices = NULL;
+	g_auto(GStrv) disabled_devices = NULL;
 	g_auto(GStrv) plugins = NULL;
 	g_auto(GStrv) report_specs = NULL;
 	g_auto(GStrv) uids = NULL;
@@ -120,10 +124,10 @@ fu_engine_config_reload(FuEngineConfig *self)
 
 	/* get disabled devices */
 	g_ptr_array_set_size(self->disabled_devices, 0);
-	devices = fu_config_get_value_strv(FU_CONFIG(self), "fwupd", "DisabledDevices");
-	if (devices != NULL) {
-		for (guint i = 0; devices[i] != NULL; i++)
-			g_ptr_array_add(self->disabled_devices, g_strdup(devices[i]));
+	disabled_devices = fu_config_get_value_strv(FU_CONFIG(self), "fwupd", "DisabledDevices");
+	if (disabled_devices != NULL) {
+		for (guint i = 0; disabled_devices[i] != NULL; i++)
+			g_ptr_array_add(self->disabled_devices, g_strdup(disabled_devices[i]));
 	}
 
 	/* get disabled plugins */
@@ -166,7 +170,7 @@ fu_engine_config_reload(FuEngineConfig *self)
 	/* get the domains to run in verbose */
 	domains = fu_config_get_value(FU_CONFIG(self), "fwupd", "VerboseDomains");
 	if (domains != NULL && domains[0] != '\0')
-		(void)g_setenv("FWUPD_VERBOSE", domains, TRUE);
+		(void)g_setenv("FWUPD_VERBOSE", domains, FALSE);
 
 	/* fetch host best known configuration */
 	host_bkc = fu_config_get_value(FU_CONFIG(self), "fwupd", "HostBkc");
@@ -185,7 +189,12 @@ fu_engine_config_reload(FuEngineConfig *self)
 		for (guint i = 0; uids[i] != NULL; i++) {
 			guint64 val = 0;
 			g_autoptr(GError) error_local = NULL;
-			if (!fu_strtoull(uids[i], &val, 0, G_MAXUINT64, &error_local)) {
+			if (!fu_strtoull(uids[i],
+					 &val,
+					 0,
+					 G_MAXUINT64,
+					 FU_INTEGER_BASE_AUTO,
+					 &error_local)) {
 				g_warning("failed to parse UID '%s': %s",
 					  uids[i],
 					  error_local->message);
@@ -314,24 +323,12 @@ fu_engine_config_get_test_devices(FuEngineConfig *self)
 	return fu_config_get_value_bool(FU_CONFIG(self), "fwupd", "TestDevices");
 }
 
-gboolean
-fu_engine_config_get_allow_emulation(FuEngineConfig *self)
-{
-	return fu_config_get_value_bool(FU_CONFIG(self), "fwupd", "AllowEmulation");
-}
-
 gboolean
 fu_engine_config_get_ignore_requirements(FuEngineConfig *self)
 {
 	return fu_config_get_value_bool(FU_CONFIG(self), "fwupd", "IgnoreRequirements");
 }
 
-gboolean
-fu_engine_config_get_ignore_efivars_free_space(FuEngineConfig *self)
-{
-	return fu_config_get_value_bool(FU_CONFIG(self), "fwupd", "IgnoreEfivarsFreeSpace");
-}
-
 gboolean
 fu_engine_config_get_release_dedupe(FuEngineConfig *self)
 {
@@ -386,7 +383,7 @@ fu_engine_config_archive_size_max_default(void)
 }
 
 static void
-fu_engine_set_config_default(FuEngineConfig *self, const gchar *key, const gchar *value)
+fu_engine_config_set_default(FuEngineConfig *self, const gchar *key, const gchar *value)
 {
 	fu_config_set_default(FU_CONFIG(self), "fwupd", key, value);
 }
@@ -405,35 +402,30 @@ fu_engine_config_init(FuEngineConfig *self)
 	g_signal_connect(self, "loaded", G_CALLBACK(fu_engine_config_changed_cb), NULL);
 	g_signal_connect(self, "changed", G_CALLBACK(fu_engine_config_changed_cb), NULL);
 
-	/* optionally used for substitutions */
-	self->os_release = fwupd_get_os_release(NULL);
-
 	/* defaults changed here will also be reflected in the fwupd.conf man page */
-	fu_engine_set_config_default(self, "AllowEmulation", "false");
-	fu_engine_set_config_default(self, "ApprovedFirmware", NULL);
-	fu_engine_set_config_default(self, "ArchiveSizeMax", archive_size_max_default);
-	fu_engine_set_config_default(self, "BlockedFirmware", NULL);
-	fu_engine_set_config_default(self, "DisabledDevices", NULL);
-	fu_engine_set_config_default(self, "DisabledPlugins", "");
-	fu_engine_set_config_default(self, "EnumerateAllDevices", "false");
-	fu_engine_set_config_default(self, "EspLocation", NULL);
-	fu_engine_set_config_default(self, "HostBkc", NULL);
-	fu_engine_set_config_default(self, "IdleTimeout", "300");		  /* s */
-	fu_engine_set_config_default(self, "IdleInhibitStartupThreshold", "500"); /* ms */
-	fu_engine_set_config_default(self, "IgnoreEfivarsFreeSpace", "false");
-	fu_engine_set_config_default(self, "IgnorePower", "false");
-	fu_engine_set_config_default(self, "IgnoreRequirements", "false");
-	fu_engine_set_config_default(self, "OnlyTrusted", "true");
-	fu_engine_set_config_default(self, "P2pPolicy", FU_DEFAULT_P2P_POLICY);
-	fu_engine_set_config_default(self, "ReleaseDedupe", "true");
-	fu_engine_set_config_default(self, "ReleasePriority", "local");
-	fu_engine_set_config_default(self, "ShowDevicePrivate", "true");
-	fu_engine_set_config_default(self, "TestDevices", "false");
-	fu_engine_set_config_default(self, "TrustedReports", "VendorId=$OEM");
-	fu_engine_set_config_default(self, "TrustedUids", NULL);
-	fu_engine_set_config_default(self, "UpdateMotd", "true");
-	fu_engine_set_config_default(self, "UriSchemes", "file;https;http;ipfs");
-	fu_engine_set_config_default(self, "VerboseDomains", NULL);
+	fu_engine_config_set_default(self, "ApprovedFirmware", NULL);
+	fu_engine_config_set_default(self, "ArchiveSizeMax", archive_size_max_default);
+	fu_engine_config_set_default(self, "BlockedFirmware", NULL);
+	fu_engine_config_set_default(self, "DisabledDevices", NULL);
+	fu_engine_config_set_default(self, "DisabledPlugins", "");
+	fu_engine_config_set_default(self, "EnumerateAllDevices", "false");
+	fu_engine_config_set_default(self, "EspLocation", NULL);
+	fu_engine_config_set_default(self, "HostBkc", NULL);
+	fu_engine_config_set_default(self, "IdleTimeout", "300");		  /* s */
+	fu_engine_config_set_default(self, "IdleInhibitStartupThreshold", "500"); /* ms */
+	fu_engine_config_set_default(self, "IgnorePower", "false");
+	fu_engine_config_set_default(self, "IgnoreRequirements", "false");
+	fu_engine_config_set_default(self, "OnlyTrusted", "true");
+	fu_engine_config_set_default(self, "P2pPolicy", FU_DEFAULT_P2P_POLICY);
+	fu_engine_config_set_default(self, "ReleaseDedupe", "true");
+	fu_engine_config_set_default(self, "ReleasePriority", "local");
+	fu_engine_config_set_default(self, "ShowDevicePrivate", "true");
+	fu_engine_config_set_default(self, "TestDevices", "false");
+	fu_engine_config_set_default(self, "TrustedReports", "VendorId=$OEM");
+	fu_engine_config_set_default(self, "TrustedUids", NULL);
+	fu_engine_config_set_default(self, "UpdateMotd", "true");
+	fu_engine_config_set_default(self, "UriSchemes", "file;https;http;ipfs");
+	fu_engine_config_set_default(self, "VerboseDomains", NULL);
 }
 
 static void
@@ -441,8 +433,6 @@ fu_engine_config_finalize(GObject *obj)
 {
 	FuEngineConfig *self = FU_ENGINE_CONFIG(obj);
 
-	if (self->os_release != NULL)
-		g_hash_table_unref(self->os_release);
 	g_ptr_array_unref(self->disabled_devices);
 	g_ptr_array_unref(self->disabled_plugins);
 	g_ptr_array_unref(self->approved_firmware);
