@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2015 Richard Hughes <richard@hughsie.com>
+ * Copyright 2015 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuMain"
@@ -29,8 +29,10 @@ static gboolean
 fu_main_sigterm_cb(gpointer user_data)
 {
 	FuDaemon *daemon = FU_DAEMON(user_data);
+	g_autoptr(GError) error = NULL;
 	g_warning("Received SIGTERM");
-	fu_daemon_stop(daemon);
+	if (!fu_daemon_stop(daemon, &error))
+		g_warning("failed to stop daemon, will wait: %s\n", error->message);
 	return G_SOURCE_CONTINUE;
 }
 #endif
@@ -39,7 +41,9 @@ static gboolean
 fu_main_timed_exit_cb(gpointer user_data)
 {
 	FuDaemon *daemon = FU_DAEMON(user_data);
-	fu_daemon_stop(daemon);
+	g_autoptr(GError) error = NULL;
+	if (!fu_daemon_stop(daemon, &error))
+		g_warning("failed to stop daemon, will wait: %s\n", error->message);
 	return G_SOURCE_REMOVE;
 }
 
@@ -51,8 +55,10 @@ fu_main_argv_changed_cb(GFileMonitor *monitor,
 			gpointer user_data)
 {
 	FuDaemon *daemon = FU_DAEMON(user_data);
+	g_autoptr(GError) error = NULL;
 	g_info("binary changed, shutting down");
-	fu_daemon_stop(daemon);
+	if (!fu_daemon_stop(daemon, &error))
+		g_warning("failed to stop daemon, will wait: %s\n", error->message);
 }
 
 static void
@@ -60,18 +66,25 @@ fu_main_memory_monitor_warning_cb(GMemoryMonitor *memory_monitor,
 				  GMemoryMonitorWarningLevel level,
 				  FuDaemon *daemon)
 {
+	g_autoptr(GError) error = NULL;
 	g_info("OOM event, shutting down");
-	fu_daemon_stop(daemon);
+	if (!fu_daemon_stop(daemon, &error))
+		g_warning("failed to stop daemon, will wait: %s\n", error->message);
 }
 
 static gboolean
 fu_main_is_hypervisor(void)
 {
-	g_autofree gchar *buf = NULL;
-	gsize bufsz = 0;
-	if (!g_file_get_contents("/proc/cpuinfo", &buf, &bufsz, NULL))
+	const gchar *flags;
+	g_autoptr(GHashTable) cpu_attrs = NULL;
+
+	cpu_attrs = fu_cpu_get_attrs(NULL);
+	if (cpu_attrs == NULL)
 		return FALSE;
-	return g_strstr_len(buf, (gssize)bufsz, "hypervisor") != NULL;
+	flags = g_hash_table_lookup(cpu_attrs, "flags");
+	if (flags == NULL)
+		return FALSE;
+	return g_strstr_len(flags, -1, "hypervisor") != NULL;
 }
 
 static gboolean
@@ -126,11 +139,6 @@ main(int argc, char *argv[])
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 
-#ifndef SUPPORTED_BUILD
-	/* make critical warnings fatal */
-	(void)g_setenv("G_DEBUG", "fatal-criticals", FALSE);
-#endif
-
 	/* TRANSLATORS: program name */
 	g_set_application_name(_("Firmware Update Daemon"));
 	context = g_option_context_new(NULL);
@@ -206,8 +214,11 @@ main(int argc, char *argv[])
 		g_timeout_add_seconds(5, fu_main_timed_exit_cb, daemon);
 
 	/* wait */
-	g_message("fwupd %s ready for requests (locale %s)", VERSION, g_getenv("LANG"));
-	fu_daemon_start(daemon);
+	g_message("Daemon ready for requests (locale %s)", g_getenv("LANG"));
+	if (!fu_daemon_start(daemon, &error)) {
+		g_printerr("Failed to start daemon: %s\n", error->message);
+		return EXIT_FAILURE;
+	}
 
 #ifdef HAVE_SYSTEMD
 	/* notify the service manager */
